@@ -191,10 +191,10 @@ class KFoldSplitAlgo(SplitAlgoBase):
         n_splits     = self.config["n_splits"]
         random_state = self.config["random_state"]
         
-        self.stdout.info("INFO: Data will be split using the KFold method.")
-        self.stdout.info(f"   Analysis datasets used: {self.get_analysis_dataset_names()}")
-        self.stdout.info(f"         Number of splits: {n_splits}")
-        self.stdout.info(f"       KFold random state: {random_state}")
+        self.stdout.info("Data will be split using the KFold method.")
+        self.stdout.info(f"   Analysis datasets used: {self.get_analysis_dataset_names()}", bare=True)
+        self.stdout.info(f"         Number of splits: {n_splits}", bare=True)
+        self.stdout.info(f"       KFold random state: {random_state}", bare=True)
         
         from sklearn.model_selection import KFold
         
@@ -269,10 +269,10 @@ class CustomSplitAlgo(SplitAlgoBase):
     
     def train_val_test_split(self, analysis_datasets:Dict, data_transformer:DataTransformer):
         dataset_map = self.get_train_val_test_map()
-        self.stdout.info("INFO: Data will be split using custom analysis datasets.")
-        self.stdout.info(f'        Train dataset: {dataset_map["train"]}')
-        self.stdout.info(f'   Validation dataset: {dataset_map.get("val", None)}')
-        self.stdout.info(f'         Test dataset: {dataset_map.get("test", None)}')
+        self.stdout.info("Data will be split using custom analysis datasets.")
+        self.stdout.info(f'        Train dataset: {dataset_map["train"]}', bare=True)
+        self.stdout.info(f'   Validation dataset: {dataset_map.get("val", None)}', bare=True)
+        self.stdout.info(f'         Test dataset: {dataset_map.get("test", None)}', bare=True)
         result = {}
         for dataset_type, dataset_names in dataset_map.items():
             selected_datasets = [analysis_datasets[dataset_name] for dataset_name in dataset_names]
@@ -319,8 +319,8 @@ class IndexSplitAlgo(SplitAlgoBase):
     def train_val_test_split(self, analysis_datasets:Dict, data_transformer:DataTransformer):
         
         index_variable = self.config["index_variable"]
-        self.stdout.info("INFO: Data will be split using custom event indices.")
-        self.stdout.info(f"       Index variable: {index_variable}")
+        self.stdout.info("Data will be split using custom event indices.")
+        self.stdout.info(f"       Index variable: {index_variable}", bare=True)
         
         analysis_dataset = analysis_datasets[None]
         concat_kwargs = {
@@ -393,7 +393,7 @@ class ManualSplitAlgo(SplitAlgoBase):
     
     def train_val_test_split(self, analysis_datasets:Dict, data_transformer:DataTransformer):
         
-        self.stdout.info("INFO: Data will be split using a user-defined function.")
+        self.stdout.info("Data will be split using a user-defined function.")
         
         split_func = self.config["split_func"]
         extra_variables = self.config.get("extra_variables", [])
@@ -465,10 +465,10 @@ class DataLoader(AnalysisBase):
             self.sample_weight_scales = None
         self.lumi_weight_scale = lumi_scale
         if self.lumi_weight_scale is not None:
-            self.stdout.info(f'INFO: Set luminosity weight scale to {self.lumi_weight_scale}')
+            self.stdout.info(f'Set luminosity weight scale to {self.lumi_weight_scale}')
         if self.sample_weight_scales is not None:
             for sample, scale_factor in self.sample_weight_scales.items():
-                self.stdout.info(f'INFO: Set event weight scale for the sample "{sample}" to {scale_factor}')
+                self.stdout.info(f'Set event weight scale for the sample "{sample}" to {scale_factor}')
 
     def set_data_transformer(self, data_transformer:Optional[DataTransformer]=None):
         if data_transformer is None:
@@ -563,7 +563,7 @@ class DataLoader(AnalysisBase):
             sample_path = self.get_file("train_sample",
                                         sample=sample,
                                         fmt=self.data_format)
-        self.stdout.info(f'INFO: Loading inputs for the sample "{sample}" from "{sample_path}"')
+        self.stdout.info(f'Loading inputs for the sample "{sample}" from "{sample_path}"')
         df = self._read_data_from_file(sample_path, key=key, columns=columns, selection=selection)
         if self.lumi_weight_scale is not None:
             if self.weight_variable is None:
@@ -738,7 +738,7 @@ class DataLoader(AnalysisBase):
             "kinematic_region"       : channel_config.get('kinematic_region', None),
             "split_algo"             : split_algo
         }
-        self.stdout.info(f"INFO: Loading data for the channel: {channel}")
+        self.stdout.info(f"Loading data for the channel: {channel}")
         return self.load_train_data(**kwargs)
         
     def print_channel_summary(self, channels:Optional[Union[List[str],str]]=None):
@@ -758,22 +758,22 @@ class DataLoader(AnalysisBase):
             test_samples = self.resolve_samples(test_samples)
             train_variables = self.resolve_variables(train_variables)
             scale_factors = self.config['channels'][channel]['SF']
-            self.stdout.info("=============================== CHANNEL SUMMARY ===============================")
-            self.stdout.info("Channel Name: ".rjust(20) + f"{channel}")
-            self.stdout.info("Channel Selection: ".rjust(20) + f"{selection}")
-            self.stdout.info("*******************************************************************************")
-            self.stdout.info("Train Samples: ".rjust(20) + ", ".join(train_samples))
-            self.stdout.info("Test Samples: ".rjust(20) + ", ".join(test_samples))
-            self.stdout.info("*******************************************************************************")
-            self.stdout.info("Class Labels: ".rjust(20))
+            self.stdout.info("=============================== CHANNEL SUMMARY ===============================", bare=True)
+            self.stdout.info("Channel Name: ".rjust(20) + f"{channel}", bare=True)
+            self.stdout.info("Channel Selection: ".rjust(20) + f"{selection}", bare=True)
+            self.stdout.info("*******************************************************************************", bare=True)
+            self.stdout.info("Train Samples: ".rjust(20) + ", ".join(train_samples), bare=True)
+            self.stdout.info("Test Samples: ".rjust(20) + ", ".join(test_samples), bare=True)
+            self.stdout.info("*******************************************************************************", bare=True)
+            self.stdout.info("Class Labels: ".rjust(20), bare=True)
             for label in class_labels:
                 samples = self.resolve_samples(class_labels[label])
-                self.stdout.info(f"\t{label}: " + ", ".join(samples))
-            self.stdout.info("*******************************************************************************")
+                self.stdout.info(f"\t{label}: " + ", ".join(samples), bare=True)
+            self.stdout.info("*******************************************************************************", bare=True)
             self.stdout.info("Train Variables: ".rjust(20) + ", ".join(train_variables))
-            self.stdout.info("*******************************************************************************")
+            self.stdout.info("*******************************************************************************", bare=True)
             self.stdout.info("Scale Factors: ".rjust(20) + str(scale_factors))
-            self.stdout.info("*******************************************************************************")
+            self.stdout.info("*******************************************************************************", bare=True)
             self.stdout.info("")
             
             
