@@ -188,21 +188,21 @@ class EventCategorization(DataLoader):
         bc_ds = self.config["categorization"]["boundary_scan"].get("datasets", None)
         eval_ds = self.config["categorization"]["evaluation"].get("datasets", None)
         if bc_ds is not None:
-            self.stdout.info("INFO: Boundary scan will be performed on the following analysis "
+            self.stdout.info("Boundary scan will be performed on the following analysis "
                             f"dataset(s): {', '.join(bc_ds)}")
         else:
-            self.stdout.info("INFO: Boundary scan will be performed on the full analysis dataset.")
+            self.stdout.info("Boundary scan will be performed on the full analysis dataset.")
         if eval_ds is not None:
-            self.stdout.info("INFO: Statistical evaluation will be performed on the following analysis "
+            self.stdout.info("Statistical evaluation will be performed on the following analysis "
                             f"dataset(s): {', '.join(eval_ds)}")
         else:
-            self.stdout.info("INFO: Statistical evaluation will be performed on the full analysis dataset.")
+            self.stdout.info("Statistical evaluation will be performed on the full analysis dataset.")
             
         self.target_channels = self.resolve_channels(target_channels)
         channel_configs  = {}
         channel_metadata = {}
         for channel in self.target_channels:
-            self.stdout.info(f'INFO: Initializing configuration for the channel "{channel}"')
+            self.stdout.info(f'Initializing configuration for the channel "{channel}"')
             channel_config  = self.config['channels'][channel]
             train_variables = self.resolve_variables(channel_config['train_variables'])
             train_samples   = self.resolve_train_samples(channel)
@@ -664,6 +664,10 @@ class EventCategorization(DataLoader):
         # evaluate observable if not already exists
         self.apply_observable(df)
         
+        # apply blinding
+        if self.is_data_sample(sample) and self.do_blind:
+            df = self.get_blind_df(df.copy())
+        
         use_custom_events = event_indices is not None
         weight_name = self.names["weight"]
         # weight for categorization
@@ -718,7 +722,7 @@ class EventCategorization(DataLoader):
         channel = self.active_channel
         channel_selection = self.channel_config['selection']
         kinematic_region  = self.channel_config['kinematic_region']
-        self.stdout.info(f"INFO: Loading inputs for the channel: {channel}")
+        self.stdout.info(f"Loading inputs for the channel: {channel}")
         if samples is None:
             samples = self.get_samples_to_load()
             
@@ -866,7 +870,7 @@ class EventCategorization(DataLoader):
         with Timer() as t:
             boundary_indices  = self.get_boundary_indices(n_bins, n_boundaries)
             n_combinations = len(boundary_indices)
-            self.stdout.info(f"INFO: Total number of boundary combinations: {n_combinations}")
+            self.stdout.info(f"Total number of boundary combinations: {n_combinations}")
             signal_yields     = self.get_region_yields(signal_hists, boundary_indices, filter_regions=filter_regions)
             background_yields = self.get_region_yields(background_hists, boundary_indices, filter_regions=filter_regions)
             # filter boundaries which the regions contain no signal or background yields
@@ -880,9 +884,9 @@ class EventCategorization(DataLoader):
                     min_yield = minimum_yields[sample]
                     sample_yields = self.get_region_yields(all_hists[sample], boundary_indices, filter_regions=filter_regions)
                     valid_boundaries &= np.all(sample_yields > min_yield, axis=1)
-            self.stdout.info(f"INFO: Number of valid boundary combinations: {valid_boundaries.sum()}")
+            self.stdout.info(f"Number of valid boundary combinations: {valid_boundaries.sum()}")
             if valid_boundaries.sum() == 0:
-                self.stdout.info("INFO: Not enough combinations to scan due to minimum yield requirements. "
+                self.stdout.info("Not enough combinations to scan due to minimum yield requirements. "
                                  "No boundaries are set.")
                 return None
             combined_significances = get_combined_counting_significance(signal_yields[valid_boundaries],
@@ -893,12 +897,12 @@ class EventCategorization(DataLoader):
             min_boundary_idx = np.argmin(combined_significances)
             min_significance = combined_significances[min_boundary_idx]
             if ((max_significance - min_significance) / min_significance) < 0.1:
-                self.stdout.info("INFO: Improvement in Z is less than 10%. "
+                self.stdout.info("Improvement in Z is less than 10%. "
                                  "No boundaries are set.")
                 return None
             max_bins         = boundary_indices[valid_boundaries][max_boundary_idx]
             max_boundary     = [bins[i] for i in max_bins]
-        self.stdout.info(f"INFO: Boundary scan finished. Total time taken: {t.interval:.3f}s")
+        self.stdout.info(f"Boundary scan finished. Total time taken: {t.interval:.3f}s")
         result = {
             'boundaries'  : [round(v, 8) for v in max_boundary],
             'significance': max_significance 
@@ -923,7 +927,7 @@ class EventCategorization(DataLoader):
             filename = self.get_file("boundary_data", channel=channel)
         if not os.path.exists(filename):
             raise FileNotFoundError(f'boundary output "{filename}" does not exist')
-        self.stdout.info(f'INFO: Cached boundary output for the channel {channel} from "{filename}"')
+        self.stdout.info(f'Cached boundary output for the channel {channel} from "{filename}"')
         boundary_tree = MultiClassBoundaryTree(score_base_name=self.names["score"])
         boundary_tree.load_tree(filename)
         self.boundary_trees[channel] = boundary_tree
@@ -958,7 +962,7 @@ class EventCategorization(DataLoader):
             
         exclude_categories = channel_config["exclude_categories"]
         
-        self.stdout.info("INFO: Scanning score boundaries...")
+        self.stdout.info("Scanning score boundaries...")
         # run over class
         class_labels = self.get_class_labels()
         class_metadata = self.channel_metadata[channel]["classes"]
@@ -979,9 +983,9 @@ class EventCategorization(DataLoader):
                 if class_label:
                     if branch_index is not None:
                         preceding_category_label = self.get_category_label(branch_index)
-                        self.stdout.info(f"Class Name: {class_label} (region = {preceding_category_label})")
+                        self.stdout.info(f"Class Name: {class_label} (region = {preceding_category_label})", bare=True)
                     else:
-                        self.stdout.info(f"Class Name: {class_label}")
+                        self.stdout.info(f"Class Name: {class_label}", bare=True)
                 class_score_hist = self.get_class_score_hist(class_label, cut_expr)
                 result = self.scan_class_bounds(class_score_hist,
                                                 class_config['n_bins'],
@@ -991,8 +995,8 @@ class EventCategorization(DataLoader):
                 if result is not None:
                     # append the new boundaries to the preceding branch
                     boundary_tree.set_boundaries(class_label, **result, branch_index=branch_index)
-                    self.stdout.info("\t boundaries: {}".format(result['boundaries']))
-                    self.stdout.info("\t counting significance: {}".format(result['significance']))
+                    self.stdout.info("\t boundaries: {}".format(result['boundaries']), bare=True)
+                    self.stdout.info("\t counting significance: {}".format(result['significance']), bare=True)
         self.boundary_trees[channel] = boundary_tree
         self.update_valid_categories()
         self.update_summary()
@@ -1087,18 +1091,15 @@ class EventCategorization(DataLoader):
             category_name = self.get_category_name(category_index)
             yields[category_name]     = {}
             yields_err[category_name] = {}
-            for sample in samples:
-                is_data = self.is_data_sample(sample)
-                do_blind = is_data and self.do_blind                
-                category_df = self.get_category_df(sample, category_index,
-                                                   do_blind=do_blind)
+            for sample in samples:       
+                category_df = self.get_category_df(sample, category_index)
                 key = sample
-                if do_blind:
+                if self.do_blind:
                     key = self.get_blind_sample_name(sample)
                 if resampling:
                     category_df = self.get_resampled_df(category_df, random_state=resampling_random_state)
                 yields[category_name][key] = float(category_df[weight_name].sum())
-                if is_data:
+                if self.is_data_sample(sample):
                     #calculate poisson interval
                     poisson_interval = get_poisson_interval([yields[category_name][key]])
                     yields_err[category_name][key] = {"errlo": poisson_interval["lo"][0],
@@ -1112,7 +1113,7 @@ class EventCategorization(DataLoader):
         """
         Update category yields for the test samples in the current channel after category labels are applied.
         """        
-        self.stdout.info(f"INFO: Evaluating sample yields")
+        self.stdout.info(f"Evaluating sample yields")
         yields, yields_err = self.get_category_yields()
         channel = self.active_channel
         self.category_yields[channel]     = yields
@@ -1194,7 +1195,7 @@ class EventCategorization(DataLoader):
                         resampling_random_state:Optional[int]=None,
                         columns:Optional[List[str]]=None,
                         selection:Optional[str]=None,
-                        do_blind:bool=False):
+                        do_blind:Optional[bool]=None):
         """
         Return dataframe from a given category.
             
@@ -1217,9 +1218,12 @@ class EventCategorization(DataLoader):
             Columns to keep in the output dataframe.
         selection: (optional) str
             Additional selection applied to the dataframe.
-        do_blind: bool, default = False
-            Whether to apply a blind selection.
+        do_blind: (optional) bool
+            Whether to apply blind selection to the dataframe. If not specified, the internal value of
+            do_blind will be used.
         """
+        if do_blind is None:
+            do_blind = self.do_blind
         if isinstance(sample_or_df, str):
             if category_or_index in self.target_channels:
                 self.validate(check_df=True)
@@ -1234,7 +1238,7 @@ class EventCategorization(DataLoader):
                 df = df.query(query_str)
         if selection is not None:
             df = df.query(selection)
-        if do_blind:
+        if do_blind and isinstance(sample_or_df, str) and self.is_data_sample(sample_or_df):
             df = self.get_blind_df(df)
         if resampling:
             df = self.get_resampled_df(df, random_state=resampling_random_state)
@@ -1243,7 +1247,7 @@ class EventCategorization(DataLoader):
             valid_columns = [c for c in columns if c in available_columns]
             missing_columns = [c for c in columns if c not in available_columns]
             if len(missing_columns) > 0:
-                self.stdout.debug(f'DEBUG: Found missing columns in the datafarme: {", ".join(missing_columns)}')
+                self.stdout.debug(f'Found missing columns in the datafarme: {", ".join(missing_columns)}')
             df = df[valid_columns]
         return df
     
@@ -1289,7 +1293,7 @@ class EventCategorization(DataLoader):
     def evaluate_benchmark_significance(self):
         self.validate(check_df=True, check_boundaries=True, check_categories=True)
         if ('benchmark' not in self.config) or ('counting_significance' not in self.config['benchmark']):
-            self.stdout.info("INFO: No significance benchmark defined. Skipped evaluation.")
+            self.stdout.info("No significance benchmark defined. Skipped evaluation.")
         benchmark_config = self.config['benchmark']['counting_significance']
         benchmarks = list(benchmark_config)
         benchmark_significance = {}
@@ -1349,7 +1353,7 @@ class EventCategorization(DataLoader):
            given in the dictionary. The format is { sample: array of event numbers }.
         """
         if event_indices is not None:
-            self.stdout.info("INFO: Categorization will be done on a given set of event numbers for each sample")
+            self.stdout.info("Categorization will be done on a given set of event numbers for each sample")
         self.set_active_channel(channel)
         self.load_channel_df(event_indices=event_indices, apply_score=not cache_scores)
         self.scan_bounds(cache=cache_boundaries)
@@ -1404,7 +1408,7 @@ class EventCategorization(DataLoader):
         if len(neg_bin_indices) > 0:
             index_str = [f"{i}" for i in (neg_bin_indices + 1)]
             value_str = [f"{v}" for v in neg_bin_values]
-            self.stdout.warning(f'WARNING: Detected bins with negative weights '
+            self.stdout.warning(f'Detected bins with negative weights '
                                 f'(bins = {", ".join(index_str)}, '
                                 f'weights = {", ".join(value_str)}). '
                                 f'Bin weight will be set to 0.')
@@ -1441,13 +1445,13 @@ class EventCategorization(DataLoader):
             raise RuntimeError(f'category summary for the chanenl "{channel}" not initialized')
         with open(savepath, "w") as outfile:
             json.dump(self.summary[channel], outfile, indent=2)
-        self.stdout.info(f'INFO: Saved category summary to {savepath}')
+        self.stdout.info(f'Saved category summary to {savepath}')
         # save boundary data
         savepath = self.get_file("boundary_data", channel=channel)
         boundary_tree = self.get_boundary_tree(channel=channel)
         with open(savepath, "w") as outfile:
             json.dump(boundary_tree.tree, outfile, indent=2)
-        self.stdout.info(f'INFO: Saved boundary data to {savepath}')
+        self.stdout.info(f'Saved boundary data to {savepath}')
         
     def save_channel_yields(self, channel:Optional[str]=None,
                             resampling:bool=False,
@@ -1480,12 +1484,12 @@ class EventCategorization(DataLoader):
             savepath = self.get_file("yield_data", category=category)
             with open(savepath, "w") as outfile:
                 json.dump(category_yields, outfile, indent=2)
-            self.stdout.info(f'INFO: Saved yield information to {savepath}')
+            self.stdout.info(f'Saved yield information to {savepath}')
         for category, category_yields_err in yields_err.items():
             savepath = self.get_file("yield_err_data", category=category)
             with open(savepath, "w") as outfile:
                 json.dump(category_yields_err, outfile, indent=2)
-            self.stdout.info(f'INFO: Saved yield uncertainty information to {savepath}')
+            self.stdout.info(f'Saved yield uncertainty information to {savepath}')
             
     def _get_minimal_sample_outputs(self):
         """Get minimal set of output samples to be saved; analysis specific
@@ -1589,7 +1593,7 @@ class EventCategorization(DataLoader):
         if resampling:
             self.stdout.info("NOTE: Resampling mode is enabled. The following outputs will "
                              "be saved with resampled events: csv, minitree, histogram, "
-                             "yield, data point.")
+                             "yield, data point.", bare=True)
         save_methods = {
             CategorizationOutputFormat.ARRAY     : self.save_dataframe_as_array,
             CategorizationOutputFormat.MINITREE  : self.save_dataframe_as_minitree,
@@ -1609,10 +1613,8 @@ class EventCategorization(DataLoader):
                     category_name = channel
                 else:
                     category_name = self.get_category_name(category_index)
-                do_blind = self.is_data_sample(sample) and self.do_blind
                 df = self.get_category_df(sample, category_index,
                                           columns=columns,
-                                          do_blind=do_blind,
                                           resampling=resampling,
                                           resampling_random_state=resampling_random_state)
                 for output_format in CategorizationOutputFormat:
@@ -1621,10 +1623,10 @@ class EventCategorization(DataLoader):
                     savepath = self.get_file(output_format.filename, sample=sample,
                                              category=category_name, fmt=fmt)
                     if cache and os.path.exists(savepath):
-                        self.stdout.info(f'INFO: Cached {output_format.short_name} output from {savepath}')
+                        self.stdout.info(f'Cached {output_format.short_name} output from {savepath}')
                         continue
                     save_methods[output_format](df, savepath)
-                    self.stdout.info(f'INFO: Saved {output_format.short_name} output to {savepath}')
+                    self.stdout.info(f'Saved {output_format.short_name} output to {savepath}')
                     
         if save_yield:
             self.update_category_yields()
@@ -1652,15 +1654,15 @@ class EventCategorization(DataLoader):
             if missing_same_columns:
                 columns = list(missing_column_by_sample.values())[0]
                 if all_samples_missing:
-                    self.stdout.warning(f"WARNING: The following save columns are missing for all samples: "
+                    self.stdout.warning(f"The following save columns are missing for all samples: "
                                         f"{', '.join(columns)}")
                 else:
                     samples = list(missing_column_by_sample.keys())
-                    self.stdout.warning(f"WARNING: The following save columns are missing for the samples "
+                    self.stdout.warning(f"The following save columns are missing for the samples "
                                         f"{', '.join(samples)}: {', '.join(columns)}")
             else:
                 for sample, columns in missing_column_by_sample.items():
-                    self.stdout.warning(f"WARNING: The following save columns are missing for the sample {sample}: "
+                    self.stdout.warning(f"The following save columns are missing for the sample {sample}: "
                                         f"{', '.join(columns)}")
     
     def get_yield_files_and_description(self):
@@ -1678,7 +1680,7 @@ class EventCategorization(DataLoader):
             merged_yield_file = f"merged_{yield_file}"
             savepath = self.get_file(merged_yield_file)
             if cache and os.path.exists(savepath):
-                self.stdout.info(f"INFO: Cached combined {description} from {savepath}")
+                self.stdout.info(f"Cached combined {description} from {savepath}")
             yield_data = {}
             for category_name in category_names:
                 filepath = self.get_file(yield_file, validate=True, category=category_name)
@@ -1686,20 +1688,20 @@ class EventCategorization(DataLoader):
                     yield_data[category_name] = json.load(file)
             with open(savepath, "w") as outfile:
                 json.dump(yield_data, outfile, indent=2)
-                self.stdout.info(f"INFO: Saved combined {description} to {savepath}")
+                self.stdout.info(f"Saved combined {description} to {savepath}")
                 
     def save_combined_benchmark_significance(self, cache:bool=True):
         self.validate(check_channels=True)
         benchmark_savepath = self.get_file("benchmark_significance")
         if (is_valid_file(benchmark_savepath)) and cache:
-            self.stdout.info(f"INFO: Cached benchmark significance data from {benchmark_savepath}")
+            self.stdout.info(f"Cached benchmark significance data from {benchmark_savepath}")
             if not self.benchmark_significance:
                 with open(benchmark_savepath, "r") as file:
                     self.benchmark_significance = json.load(file)
         elif self.benchmark_significance:
             with open(benchmark_savepath, "w") as file:
                 json.dump(self.benchmark_significance, file)
-                self.stdout.info(f"INFO: Saved benchmark significance data to {benchmark_savepath}")
+                self.stdout.info(f"Saved benchmark significance data to {benchmark_savepath}")
                       
     def merge_category_outputs(self, save_yield:bool=True,
                                save_benchmark:bool=True,
@@ -1708,7 +1710,7 @@ class EventCategorization(DataLoader):
             if channel not in self.boundary_trees:
                 self.load_boundaries(channel=channel)
         
-        self.stdout.info("INFO: Merging outputs from all channels...")
+        self.stdout.info("Merging outputs from all channels...")
        
         if save_yield:
             self.save_combined_yields(cache=cache)
