@@ -182,6 +182,8 @@ class MultiClassBoundaryTree:
     def _get_str_repr(self, source:Dict, level:int=0):
         str_repr = ""
         class_name = source['class']
+        if class_name is None:
+            class_name = ""
         boundaries = source['boundaries']
         score_windows = self._get_score_windows(boundaries)
         if 'significance' in source:
