@@ -1,8 +1,11 @@
 import click
-
 import fnmatch
+
+from .core import cli
+
+__all__ = ['inspect_rfile']
     
-@click.command(name='inspect_rfile')
+@cli.command(name='inspect_rfile')
 @click.option('-i', '--file_expr', required=True, help='File name expression '
               '(supports wild card, multiple files separated by commas).')
 @click.option('-t', '--tree_name', required=True, help='Tree name.')
@@ -21,14 +24,14 @@ import fnmatch
               help='verbosity level ("DEBUG", "INFO", "WARNING", "ERROR")')
 def inspect_rfile(file_expr, tree_name, filter_expr=None, output_file=None,
                   include_patterns=None, exclude_patterns=None, verbosity="INFO"):
+    '''Tool for inspecting root files
     '''
-        Inspect root files
-    '''
-    file_expr = file_expr.split(',')
+    from quickstats.utils.string_utils import split_str
+    file_expr = split_str(file_expr, sep=',', remove_empty=True)
     if include_patterns is not None:
-        include_patterns = include_patterns.split(',')
+        include_patterns = split_str(include_patterns, sep=',', remove_empty=True)
     if exclude_patterns is not None:
-        exclude_patterns = exclude_patterns.split(',')
+        exclude_patterns = split_str(exclude_patterns, sep=',', remove_empty=True)
     from quickstats.components import RooInspector
     rinspector = RooInspector(tree_name, file_expr, filter_expr=filter_expr, verbosity=verbosity)
     rinspector.print_summary(include_patterns=include_patterns,
