@@ -1,12 +1,11 @@
 from .basics import *
 from .root_object import ROOTObject
+from .discrete_nuisance import DiscreteNuisance
 from .extended_minimizer import ExtendedMinimizer
 from .extended_model import ExtendedModel
 from .analysis_object import AnalysisObject
 from .asimov_generator import AsimovGenerator
 from .asimov_generator import AsimovType
-from .nuisance_parameter_pull import NuisanceParameterPull
-from .nuisance_parameter_harmonizer import NuisanceParameterHarmonizer
 from .asymptotic_cls import AsymptoticCLs
 from .likelihood import Likelihood
 from .pvalue_toys import PValueToys
@@ -14,6 +13,10 @@ from .toy_limit_calculator import ToyLimitCalculator
 from .analysis_base import AnalysisBase
 from .roo_inspector import RooInspector
 from .extended_rfile import ExtendedRFile
+from .nuisance_parameter_pull import NuisanceParameterPull
+from .nuisance_parameter_ranking import NuisanceParameterRanking
+from .nuisance_parameter_harmonizer import NuisanceParameterHarmonizer
+from .caching_nll_wrapper import CachingNLLWrapper
 #from .signal_modelling import SignalModelling
 
 import ROOT
