@@ -1,11 +1,13 @@
-from typing import Optional, Union, List
+from typing import Optional, Union, List, Dict
+
+import ROOT
 
 from quickstats import AbstractObject
 from quickstats.components import ExtendedModel, ExtendedMinimizer
 from quickstats.components.basics import WSArgument
-from quickstats.utils.common_utils import get_class_that_defined_method
-from quickstats.interface.root import RooArgSet
-import ROOT
+from quickstats.utils.common_utils import get_class_that_defined_method, update_config_dict, combine_dict
+from quickstats.utils.roostats_utils import set_prior_pdf
+from quickstats.interface.root import RooArgSet, ModelConfig
 
 class AnalysisObject(AbstractObject):
     
@@ -18,13 +20,14 @@ class AnalysisObject(AbstractObject):
                  fix_param:str='', profile_param:str='', ws_name:Optional[str]=None, 
                  mc_name:Optional[str]=None, snapshot_name:Optional[Union[List[str], str]]=None,
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', precision:float=0.001, 
-                 eps:float=1.0, retry:int=1, strategy:int=1, print_level:int=-1, timer:bool=False,
-                 num_cpu:int=1, offset:bool=True, optimize:int=2, eigen:bool=False, hesse:bool=False,
-                 improve:int=0, fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
-                 max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
+                 eps:float=1.0, retry:int=1, strategy:int=1, print_level:int=-1,
+                 num_cpu:int=1, offset:bool=True, optimize:int=2, constrain_nuis:bool=True,
+                 batch_mode:bool=False, prior_type:Optional[str]=None, runtimedef_expr:Optional[str]=None,
                  int_bin_precision:float=-1., preset_param:bool=False, minimizer_offset:int=1,
-                 minimizer_cls=None, verbosity:Optional[Union[int, str]]="INFO"):
-        super().__init__(verbosity=verbosity)    
+                 extra_minimizer_options:Optional[Union[Dict, str]]=None,
+                 minimizer_cls=None, verbosity:Optional[Union[int, str]]="INFO",
+                 **kwargs):
+        super().__init__(verbosity=verbosity)
         self.model = None
         self.minimizer = None
         self._use_blind_range = False
@@ -34,64 +37,89 @@ class AnalysisObject(AbstractObject):
             self.minimizer_cls = minimizer_cls
         if filename is not None:
             model_options = {
-                "fname"             : filename,
+                "filename"          : filename,
                 "ws_name"           : ws_name,
                 "mc_name"           : mc_name,
                 "data_name"         : data_name,
                 "snapshot_name"     : snapshot_name,
-                "binned_likelihood" : binned_likelihood,
-                "fix_cache"         : fix_cache,
-                "fix_multi"         : fix_multi
+                "binned_likelihood" : binned_likelihood
             }
             self.setup_model(**model_options)
             self.model._load_floating_auxiliary_variables()
             self.save_snapshot(self.kInitialSnapshotName, WSArgument.MUTABLE)
-            self.set_poi(poi_name)
-            if preset_param:
-                self.preset_parameters()
-            self.setup_parameters(fix_param, profile_param, update_snapshot=True)
             minimizer_options = {
+                "constrain_nuis"    : constrain_nuis,
                 "minimizer_type"    : minimizer_type,
                 "minimizer_algo"    : minimizer_algo,
                 "precision"         : precision,
                 "eps"               : eps,
-                "retry"             : retry,
                 "strategy"          : strategy,
-                "num_cpu"           : num_cpu,
-                "offset"            : offset,
                 "minimizer_offset"  : minimizer_offset,
+                "offset"            : offset,
                 "optimize"          : optimize,
-                "eigen"             : eigen,
-                "hesse"             : hesse,
-                "improve"           : improve,
-                "max_calls"         : max_calls,
-                "max_iters"         : max_iters,
+                "retry"             : retry,
+                "num_cpu"           : num_cpu,
                 "print_level"       : print_level,
-                "timer"             : timer,
-                "constrain_nuis"    : constrain_nuis,
                 "batch_mode"        : batch_mode,
                 "int_bin_precision" : int_bin_precision,
+                "runtimedef_expr"   : runtimedef_expr,
                 "verbosity"         : verbosity
             }
-            self.setup_minimizer(**minimizer_options)
+            if kwargs:
+                minimizer_options = combine_dict(minimizer_options, kwargs)
+            minimizer_options = update_config_dict(minimizer_options, extra_minimizer_options)
+            self.setup_minimizer(discrete_nuisance=self.model._discrete_nuisance,
+                                 **minimizer_options)
+            self.set_poi(poi_name)
+            if preset_param:
+                self.preset_parameters()
+            self.setup_parameters(fix_param, profile_param, update_snapshot=True)
+            set_prior_pdf(self.model.workspace,
+                          self.model.model_config,
+                          self.poi, prior_type)
             self.sanity_check()
+            
         
     def sanity_check(self):
-        aux_vars = self.model.get_variables("auxiliary")
-        floating_aux_vars = [v.GetName() for v in aux_vars if not v.isConstant()]
-        if len(floating_aux_vars) > 0:
-            self.stdout.warning("WARNING: The following auxiliary variables (variables that are not "
-                                "part of the POIs, observables, nuisance parameters and global "
-                                f"observables) are floating: {','.join(floating_aux_vars)}. If this is "
-                                "not intended, please make sure to fix them before fitting.", "red")            
-    
-    def preset_parameters(self):
-        ROOT.RooStats.SetAllConstant(self.model.global_observables, True)
-        #ROOT.RooStats.SetAllConstant(self.model.nuisance_parameters, False)
-        ROOT.RooStats.SetAllConstant(self.model.pois, True)
-        self.stdout.info("INFO: Preset POIs as constant parameters.")
-        #self.stdout.info("INFO: Preset nuisance parameters as floating parameters.")
-        self.stdout.info("INFO: Preset global observables as constant parameters.")
+        
+        def check_aux_vars_are_fixed():
+            aux_vars = self.model.get_variables("auxiliary")
+            ROOT.RooStats.RemoveConstantParameters(aux_vars)
+            if self.model.has_discrete_nuisance():
+                aux_vars = RooArgSet.get_set_difference(aux_vars, self.model.discrete_nuisance.multipdf_params)
+                aux_vars = RooArgSet.get_set_difference(aux_vars, ROOT.RooArgSet(self.model.discrete_nuisance.multipdf_cats))
+            if len(aux_vars) > 0:
+                aux_var_names = [v.GetName() for v in aux_vars]
+                self.stdout.warning("The following auxiliary variables (variables that are not "
+                                    "part of the POIs, observables, nuisance parameters, global "
+                                    "observables and discrete nuisances) are floating: "
+                                    f"{','.join(aux_var_names)}. If this is not intended, please "
+                                    "make sure to fix them before fitting.", "red")
+            passed = len(aux_vars) == 0
+            return passed
+        
+        def check_model_config():
+            passed = ModelConfig(verbosity=self.stdout.verbosity).sanity_check(self.model.model_config)
+            return passed
+        
+        for task_name, task in [('Auxiliary variable', check_aux_vars_are_fixed),
+                                ('Model config', check_model_config)]:
+            passed = task()
+            if passed:
+                self.stdout.info(f"{task_name} sanity check: PASSED")
+            else:
+                self.stdout.info(f"{task_name} sanity check: FAILED")
+            
+    def preset_parameters(self, fix_pois:bool=True, fix_globs:bool=True, float_nuis:bool=False):
+        if fix_pois:
+            ROOT.RooStats.SetAllConstant(self.model.pois, True)
+            self.stdout.info("Preset POIs as constant parameters.")
+        if fix_globs:
+            ROOT.RooStats.SetAllConstant(self.model.global_observables, True)
+            self.stdout.info("Preset global observables as constant parameters.")
+        if float_nuis:
+            ROOT.RooStats.SetAllConstant(self.model.nuisance_parameters, False)
+            self.stdout.info("INFO: Preset nuisance parameters as floating parameters.")
     
     @property
     def use_blind_range(self):
@@ -105,6 +133,14 @@ class AnalysisObject(AbstractObject):
     def nll_commands(self):
         return self.minimizer.nll_commands
     
+    @property
+    def get_poi(self):
+        return self.model.get_poi
+
+    @property
+    def poi(self):
+        return self.minimizer._poi
+    
     # black magic
     def _inherit_init(self, init_func, **kwargs):
         import inspect
@@ -120,23 +156,21 @@ class AnalysisObject(AbstractObject):
             this_kwargs = {k:v for k,v in kwargs.items() if k not in that_parameters}
             init_func(config=this_kwargs, **that_kwargs)
 
-    def set_poi(self, poi_name:Optional[Union[str, List[str]]]=None):
-        # multi POI case
-        if isinstance(poi_name, list):
-            self.poi = ROOT.RooArgSet()
-            true_poi_names = []
-            # remove duplicates
-            poi_name = list(set(poi_name))
-            for pname in poi_name:
-                poi = self.model.get_poi(pname)
-                self.poi.add(poi)
-                true_poi_names.append(poi.GetName())
-            if true_poi_names:
-                self.stdout.info('INFO: POIs set to {}'.format(", ".join([f"\"{name}\"" for name in true_poi_names])))
+    def set_poi(self, pois:Optional[Union[str, List[str], "ROOT.RooRealVar", "ROOT.RooArgSet"]]=None):
+        if (pois is not None) and not isinstance(pois, (ROOT.RooRealVar, ROOT.RooArgSet)):
+            pois = self.get_poi(pois)
+        self.minimizer.set_poi(pois)
+        if isinstance(pois, ROOT.RooRealVar):
+            poi_names = [pois.GetName()]
+        elif isinstance(pois, ROOT.RooArgSet):
+            poi_names = [poi.GetName() for poi in pois]
         else:
-            self.poi = self.model.get_poi(poi_name)
-            if poi_name is not None:
-                self.stdout.info(f'INFO: POI set to "{poi_name}"')
+            poi_names = []
+        if len(poi_names) == 1:
+            self.stdout.info(f'POI set to "{poi_names[0]}"')
+        elif len(poi_names) > 1:
+            text = ", ".join([f'"{name}"' for name in poi_names])
+            self.stdout.info(f'POIs set to {text}')
         
     def setup_model(self, **kwargs):
         model = ExtendedModel(**kwargs, verbosity=self.stdout.verbosity)
@@ -179,22 +213,25 @@ class AnalysisObject(AbstractObject):
         for profiled_param in profiled_parameters:
             if aux_pois.contains(profiled_param):
                 param_name = profiled_param.GetName()
-                self.stdout.warning(f"WARNING: Attemping to profile the parameter \"{param_name}\" which is a "
+                self.stdout.warning(f"Attemping to profile the parameter \"{param_name}\" which is a "
                                     f"POI defined in the workspace but not a POI used in this study. This "
                                     f"parameter will still be fixed during likelihood fit / limit setting. "
                                     f"Please designate the parameter as a POI in this study if intended.", "red")
         for fixed_param in fixed_parameters:
             if ana_pois.contains(fixed_param):
                 param_name = fixed_param.GetName()
-                self.stdout.warning(f"WARNING: Attemping to fix the parameter \"{param_name}\" which is a "
+                self.stdout.warning(f"Attemping to fix the parameter \"{param_name}\" which is a "
                                     f"POI used in this study. This parameter will still be floated during "
                                     f"unconditional likelihood fit / limit setting.", "red")
             
-    def setup_minimizer(self, constrain_nuis:bool=True, **kwargs):
-        
+    def setup_minimizer(self, constrain_nuis:bool=True,
+                        discrete_nuisance:Optional["DiscreteNuisance"]=None,
+                        runtimedef_expr:Optional[str]=None, **kwargs):
         minimizer = self.minimizer_cls("Minimizer", self.model.pdf, self.model.data,
                                        workspace=self.model.workspace,
-                                       verbosity=self.stdout.verbosity)
+                                       runtimedef_expr=runtimedef_expr,
+                                       verbosity=self.stdout.verbosity,
+                                       discrete_nuisance=discrete_nuisance)
         
         nll_options = {k:v for k,v in kwargs.items() if k in ExtendedMinimizer._DEFAULT_NLL_OPTION_}
         
@@ -207,17 +244,20 @@ class AnalysisObject(AbstractObject):
             #nll_commands.append(ROOT.RooFit.ExternalConstraints(ROOT.RooArgSet()))
         
         minimizer_options = {k:v for k,v in kwargs.items() if k in ExtendedMinimizer._DEFAULT_MINIMIZER_OPTION_}
-
         minimizer.configure_nll(**nll_options)
         minimizer.configure(**minimizer_options)
         self.minimizer = minimizer
+        self.model.set_minimizer(self.minimizer)
         self.default_nll_options = nll_options
         self.default_minimizer_options = minimizer_options
     
     def set_data(self, data_name:str='combData'):
-        data = self.model.workspace.data(data_name)
-        if not data:
-            raise RuntimeError(f'workspace does not contain the dataset "{data_name}"')
+        if isinstance(data_name, ROOT.RooDataSet):
+            data = data_name
+        else:
+            data = self.model.workspace.data(data_name)
+            if not data:
+                raise RuntimeError(f'workspace does not contain the dataset "{data_name}"')
         self.minimizer.set_data(data)
         self.model._data = data
         
@@ -231,40 +271,30 @@ class AnalysisObject(AbstractObject):
         self.minimizer.nll = None
         self.minimizer.nll_commands.pop("RangeWithName", None)
         self.minimizer.nll_commands.pop("SplitRange", None)
-        self.stdout.info("INFO: Blind range removed from list of  NLL commands. NLL is reset.")
+        self.stdout.info("Blind range removed from list of  NLL commands. NLL is reset.")
         self._use_blind_range = False
-        
-    def load_snapshot(self, snapshot_name:Optional[str]=None):
-        if snapshot_name is not None:
-            snapshot = self.model.workspace.getSnapshot(snapshot_name)
-            if (not snapshot) and ('0x(nil)' in snapshot.__repr__()):
-                self.stdout.warning(f'WARNING: Snapshot "{snapshot_name}" does not exist.')
-            else:
-                self.model.workspace.loadSnapshot(snapshot_name)
-                self.stdout.debug(f'DEBUG: Loaded snapshot "{snapshot_name}"')
                 
-    def restore_snapshot(self):
+    def restore_prefit_snapshot(self):
         self.load_snapshot(self.kCurrentSnapshotName)
+        
+    def restore_init_snapshot(self):
+        self.load_snapshot(self.kInitialSnapshotName)
+        self.save_snapshot(self.kCurrentSnapshotName)
     
     def get_variables(self, variable_type:Union[str, WSArgument], sort:bool=True):
         return self.model.get_variables(variable_type, sort=sort)
     
     def save_snapshot(self, snapshot_name:Optional[str]=None, 
                       variables:Optional[Union[ROOT.RooArgSet, str, WSArgument]]=None):
-        if snapshot_name is not None:
-            if variables is None:
-                self.model.workspace.saveSnapshot(snapshot_name, self.model.workspace.allVars())
-            else:
-                if isinstance(variables, (str, WSArgument)):
-                    variables = self.get_variables(variables)
-                self.model.workspace.saveSnapshot(snapshot_name, variables)
-            self.stdout.debug(f'DEBUG: Saved snapshot "{snapshot_name}"')
+        self.model.save_snapshot(snapshot_name, variables=variables)
+        
+    def load_snapshot(self, snapshot_name:Optional[str]=None):
+        self.model.load_snapshot(snapshot_name)
         
     def save(self, filename:str, recreate:bool=True, rebuild:bool=True):
         self.model.save(filename, recreate=recreate, rebuild=rebuild)
-        self.stdout.info(f'INFO: Saved workspace file as "{filename}"')
-        
-        
+        self.stdout.info(f'Saved workspace file as "{filename}"')
+                
     def decompose_nll(self, fmt:str="pandas"):
         from quickstats.utils.roofit_utils import decompose_nll
         if not self.minimizer.nll:
