@@ -104,39 +104,10 @@ class AsimovGenerator(AnalysisObject):
         config['poi_name']  = poi_name
         config['data_name'] = data_name
         config['verbosity'] = verbosity
+        if 'preset_param' not in config:
+            config['preset_param'] = True
         self._inherit_init(super(AsimovGenerator, self).__init__, **config)
         
-    def generate_asimov(self, poi_val:Optional[float]=None, 
-                        poi_profile:Optional[float]=None,
-                        do_fit:bool=True,
-                        modify_globs:bool=True,
-                        do_import:bool=True,
-                        asimov_name:Optional[str]=None,
-                        asimov_snapshot:Optional[str]=None,
-                        channel_asimov_name:Optional[str]=None,
-                        dataset:Optional["ROOT.RooDataSet"]=None,
-                        constraint_option:int=0,
-                        restore_states:int=0,
-                        snapshot_names:Optional[Dict]=None):
-        kwargs = {
-            "poi_name"            : self.poi.GetName(),
-            "poi_val"             : poi_val,
-            "poi_profile"         : poi_profile,
-            "do_fit"              : do_fit,
-            "modify_globs"        : modify_globs,
-            "do_import"           : do_import,
-            "asimov_name"         : asimov_name,
-            "asimov_snapshot"     : asimov_snapshot,
-            "channel_asimov_name" : channel_asimov_name,
-            "dataset"             : dataset,
-            "constraint_option"   : constraint_option,
-            "restore_states"      : restore_states,
-            "minimizer_options"   : self.minimizer_options,
-            "nll_options"         : self.nll_commands,
-            "snapshot_names"      : snapshot_names
-        }
-        return self.model.generate_asimov(**kwargs)
-        
     def generate_standard_asimov(self, asimov_types:Optional[Union[List[AsimovType], List[int], List[str]]]=None,
                                  asimov_names:Optional[Union[List[str], str]]=None,
                                  asimov_snapshots:Optional[Union[List[str], str]]=None,
@@ -159,11 +130,17 @@ class AsimovGenerator(AnalysisObject):
             assert len(asimov_snapshots) == len(asimov_types)            
         if poi_scale is None:
             poi_scale = 1.0
-        poi_name = self.poi.GetName()
-        self.save_snapshot("temp", WSArgument.MUTABLE)
+            
+        import ROOT
+        if isinstance(self.poi, ROOT.RooArgSet):
+            poi_name = [poi.GetName() for poi in self.poi]
+        else:
+            poi_name = self.poi.GetName()
+            
+        self.save_snapshot(self.kTempSnapshotName, WSArgument.MUTABLE)
         asimov_datasets = []
         for i, asimov_type in enumerate(asimov_types):
-            self.load_snapshot("temp")
+            self.load_snapshot(self.kTempSnapshotName)
             kwargs = copy.deepcopy(self.ASIMOV_SETTINGS[asimov_type])
             if kwargs is None:
                 continue
@@ -175,8 +152,9 @@ class AsimovGenerator(AnalysisObject):
                 kwargs['asimov_name'] = asimov_names[i]
             if asimov_snapshots is not None:
                 kwargs['asimov_snapshot'] = asimov_snapshots[i]
-            kwargs['minimizer_options'] = self.minimizer.config
-            kwargs['nll_options'] = list(self.minimizer.nll_commands.values())
+            kwargs['dataset'] = self.model.data
+            kwargs['minimizer_options'] = self.default_minimizer_options
+            kwargs['nll_options'] = self.default_nll_options
             kwargs['do_import'] = do_import
             asimov_dataset = self.model.generate_asimov(**kwargs)
             asimov_datasets.append(asimov_dataset)
