@@ -7,11 +7,10 @@ import numpy as np
 
 import ROOT
 
-from quickstats import semistaticmethod
+from quickstats import semistaticmethod, cls_method_timer
 from quickstats.components import AnalysisObject
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils import common_utils
-from quickstats.utils.io import Verbosity
 from quickstats.maths.numerics import pretty_value
 from quickstats.components.basics import WSArgument
 
@@ -34,13 +33,11 @@ class AsymptoticCLs(AnalysisObject):
                  better_negative_bands:bool=False, do_blind:bool=True,
                  binned_likelihood:bool=True, fix_param:str='', profile_param:str='',
                  ws_name:Optional[str]=None, mc_name:Optional[str]=None,
-                 snapshot_name:Optional[str]=None, timer:bool=False,
+                 snapshot_name:Optional[str]=None, constrain_nuis:bool=True, 
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
                  eps:float=1.0, retry:int=2, strategy:int=1, print_level:int=-1, 
-                 num_cpu:int=1, offset:bool=True, optimize:int=2,
-                 fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1,
-                 max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
-                 int_bin_precision:float=-1., improve:int=0, minimizer_offset:int=1,
+                 num_cpu:int=1, minimizer_offset:int=1, offset:bool=True,
+                 optimize:int=2, batch_mode:bool=False, int_bin_precision:float=-1., 
                  verbosity:Optional[Union[int, str]]="INFO", **kwargs):
         """
         args:
@@ -65,24 +62,18 @@ class AsymptoticCLs(AnalysisObject):
             'retry': retry,
             'strategy': strategy,
             'num_cpu': num_cpu,
+            'minimizer_offset': minimizer_offset,
             'offset': offset,
             'optimize': optimize,
-            'improve': improve,
-            'minimizer_offset': minimizer_offset,
-            'eigen': False,
             'print_level': print_level,
-            'timer': timer,
-            'fix_cache': fix_cache,
-            'fix_multi': fix_multi,
-            'max_calls': max_calls,
-            'max_iters': max_iters,
             'constrain_nuis': constrain_nuis,
             'batch_mode': batch_mode,
             'int_bin_precision': int_bin_precision,
             'preset_param': True,
-            'verbosity': verbosity
+            'verbosity': verbosity,
+            **kwargs
         }
-        super().__init__(**config)
+        self._inherit_init(super().__init__, **config)
         self.do_blind = do_blind
         self.CL = CL
         self.precision = precision
@@ -96,23 +87,11 @@ class AsymptoticCLs(AnalysisObject):
         self.mu_exp = mu_exp
         if mu_guess == mu_exp:            
             mu_guess = mu_guess + 0.1
-            self.stdout.info(f"INFO: Changed mu_guess to {mu_guess} so that it is different from mu_exp")
+            self.stdout.info(f"Changed mu_guess to {mu_guess} so that it is different from mu_exp")
         self.mu_guess = mu_guess
         
-        # define asimov data
-        if asimov_data_name is not None:
-            asimov_data_0 = self.model.workspace.data(asimov_data_name)
-            if not asimov_data_0:
-                raise RuntimeError(f'failed to load dataset \"{asimov_data_name}\"')
-            self.asimov_data_0 = asimov_data_0
-            glob_snap_name = self.get_conditional_snapsot_name(0, "global_observable")
-            nuis_snap_name = self.get_conditional_snapsot_name(0, "nuisance_parameter")
-            if (not self.model.workspace.loadSnapshot(glob_snap_name)) or \
-            (not self.model.workspace.loadSnapshot(nuis_snap_name)):
-                raise RuntimeError("when using user-defined background-only Asimov data, "
-                                   f"snapshots {nuis_snap_name} and {glob_snap_name} must be defined.")
-        else:
-            self.asimov_data_0 = None
+        # use custom bkg asimov data if specified
+        self.load_asimov_data_0(asimov_data_name)
         
         # define simplified names
         self.ws = self.model.workspace
@@ -136,25 +115,34 @@ class AsymptoticCLs(AnalysisObject):
             if not hasattr(ROOT, 'AsymptoticCLsTool'):
                 result = load_macro('AsymptoticCLsTool')
                 if hasattr(ROOT, 'AsymptoticCLsTool'):
-                    self.stdout.info('INFO: Loaded extension module "AsymptoticCLsTool"')
+                    self.stdout.info('Loaded extension module "AsymptoticCLsTool"')
         except Exception as e:
             print(e)
-            
-    def set_poi(self, poi_name:Optional[str]=None):
-        if poi_name is None:
-            self.poi = self.model.pois.first()
-            self.stdout.info('INFO: POI name not given. Set to first poi "{}" by default.'.format(self.poi.GetName()))
-        else:
-            self.poi = self.model.get_poi(poi_name)
-            self.stdout.info('INFO: POI set to "{}"'.format(poi_name))
+
+    def load_asimov_data_0(self, asimov_data_name:Optional[str]=None):
+        if asimov_data_name is None:
+            self.asimov_data_0 = None
+            return None
+        asimov_data_0 = self.model.workspace.data(asimov_data_name)
+        if not asimov_data_0:
+            raise RuntimeError(f'failed to load dataset "{asimov_data_name}"')
+        # check that conditional global observable and nuisance parameter snapshots are defined
+        glob_snap_name = self.get_conditional_snapsot_name(0, "global_observable")
+        nuis_snap_name = self.get_conditional_snapsot_name(0, "nuisance_parameter")
+        glob_snap = self.model.workspace.getSnapshot(glob_snap_name)
+        nuis_snap = self.model.workspace.getSnapshot(nuis_snap_name)
+        if ((not glob_snap) or (not nuis_snap)):
+            raise RuntimeError("when using user-defined background-only Asimov data, "
+                               f"snapshots {nuis_snap_name} and {glob_snap_name} must be defined.")
+        self.asimov_data_0 = asimov_data_0
         
     def set_poi_value(self, val:float):
         if math.isnan(val):
             raise ValueError("cannot set poi to nan")
         if ( val > 0 ) and (self.poi.getMax() < val):
-            self.poi.setMax(2*val)
+            self.poi.setMax(2 * val)
         if ( val < 0 ) and (self.poi.getMin() > val):
-            self.poi.setMin(2*val)
+            self.poi.setMin(2 * val)
         self.poi.setVal(val)
             
     def save_nll_snapshot(self, nll, mu):
@@ -186,17 +174,11 @@ class AsymptoticCLs(AnalysisObject):
         if snapshot_name:
             self.load_snapshot(snapshot_name)
         else:
-            raise RuntimeError('Failed to load snapshot for nll "{}"'.format(nll.GetName()))
-        if Verbosity.DEBUG >= self.stdout.verbosity:
-            self.stdout.debug("DEBUG: Before Fit ----------------------------------------------")
-            self.model.print_summary(items=['poi', 'nuisance_parameter', 'global_observable'])
-        self.last_fit_status = self.minimizer.minimize(nll) 
-        self.global_status += self.last_fit_status
+            raise RuntimeError(f'Failed to load snapshot for nll "{nll.GetName()}"')
+        self.last_fit_status = self.minimizer.minimize(nll)
+        self.global_status   += self.last_fit_status
         self.minimizer_calls += 1
         nll_val = nll.getVal()
-        if Verbosity.DEBUG >= self.stdout.verbosity:
-            self.stdout.debug("DEBUG: After Fit ----------------------------------------------")
-            self.model.print_summary(items=['poi', 'nuisance_parameter', 'global_observable'])          
         self.load_snapshot(self.nom_glob_name)
         return nll_val
     
@@ -206,7 +188,7 @@ class AsymptoticCLs(AnalysisObject):
     def get_qmu(self, nll:"ROOT.RooNLLVar", mu:float):
         nll_muhat = self.nll_maps[nll].get('nll_mu_hat', None)
         if nll_muhat is None:
-            raise RuntimeError('Failed to get nominal nll for "{}"'.format(nll.GetName()))
+            raise RuntimeError(f'Failed to get nominal nll for "{nll.GetName()}"')
         is_const = self.poi.isConstant()
         self.poi.setConstant(1)
         self.set_poi_value(mu)
@@ -249,13 +231,16 @@ class AsymptoticCLs(AnalysisObject):
     def get_limit(self, nll:"ROOT.RooNLLVar", initial_guess:float, summary_label:Optional[int]=None):
         self.global_status = 0
         nll_name = nll.GetName()
-        self.stdout.info("----------------------------------")
-        self.stdout.info(f"Getting limit for nll: {nll_name}")
+        self.stdout.info("----------------------------------", bare=True)
+        self.stdout.info(f"Getting limit for nll: {nll_name}", bare=True)
         self.poi.setConstant(0)
+        # load mu_exp asimov
         asimov_0_nll = self.summary[0]['nll']
+        
         if nll == asimov_0_nll:
             self.set_poi_value(self.mu_exp)
             self.poi.setConstant(1)
+            
         # case for observed limit
         if (nll not in self.nll_maps) or ('nll_mu_hat' not in self.nll_maps[nll]):
             nll_val = self.get_nll_val(nll)
@@ -274,10 +259,8 @@ class AsymptoticCLs(AnalysisObject):
             self.set_poi_value(initial_guess)
 
         mu = self.poi.getVal()
-        from pdb import set_trace
         
         qmu = self.get_qmu(asimov_0_nll, mu)
-        #set_trace()
         sigma_guess = self.LimitTool.getSigma(mu, self.mu_exp, qmu)
         sigma_b = sigma_guess
         mu_guess = self.LimitTool.findCrossing(sigma_guess, sigma_b, mu_hat)
@@ -286,15 +269,15 @@ class AsymptoticCLs(AnalysisObject):
         CLs = self.LimitTool.calcCLs(qmu, sigma_b, mu_guess)
         qmu95 = self.LimitTool.getQmu95(sigma_b, mu_guess)
         self.set_poi_value(mu_guess)
-        self.stdout.info(f"Initial guess:  {mu_guess}")
-        self.stdout.info(f"Sigma(obs):     {sigma_guess}")
-        self.stdout.info(f"Sigma(mu,0):    {sigma_b}")
-        self.stdout.info(f"muhat:          {mu_hat}")
-        self.stdout.info(f"qmu95:          {qmu95}")
-        self.stdout.info(f"qmu:            {qmu}")
-        self.stdout.info(f"pmu:            {pmu}")
-        self.stdout.info(f"1-pb:           {pb}")
-        self.stdout.info(f"CLs:            {CLs}")
+        self.stdout.info(f"Initial guess:  {mu_guess}", bare=True)
+        self.stdout.info(f"Sigma(obs):     {sigma_guess}", bare=True)
+        self.stdout.info(f"Sigma(mu,0):    {sigma_b}", bare=True)
+        self.stdout.info(f"muhat:          {mu_hat}", bare=True)
+        self.stdout.info(f"qmu95:          {qmu95}", bare=True)
+        self.stdout.info(f"qmu:            {qmu}", bare=True)
+        self.stdout.info(f"pmu:            {pmu}", bare=True)
+        self.stdout.info(f"1-pb:           {pb}", bare=True)
+        self.stdout.info(f"CLs:            {CLs}", bare=True)
 
         n_damping = 1
         guess_to_corr = {}
@@ -304,8 +287,8 @@ class AsymptoticCLs(AnalysisObject):
         mu_pre2 = mu_hat
         
         while (abs(mu_pre-mu_guess) > self.precision*mu_guess):
-            self.stdout.info('----------------------------------')
-            self.stdout.info('Starting iteration {} of {}'.format(n_iter, nll_name))
+            self.stdout.info('----------------------------------', bare=True)
+            self.stdout.info(f'Starting iteration {n_iter} of {nll_name}', bare=True)
             # do this to avoid comparing multiple minima in the conditional and unconditional fits           
             if (n_iter == 0):
                 self.load_nll_snapshot(nll, mu_hat)
@@ -340,7 +323,7 @@ class AsymptoticCLs(AnalysisObject):
                 if (abs(first - (mu_guess - corr)) < self.direction*mu_guess*0.02) and \
                 (abs(corr) > self.direction*mu_guess*self.precision):
                     damping_factor *= 0.8
-                    self.stdout.info('Changing damping factor to {}, n_damping = {}'.format(damping_factor, n_damping))
+                    self.stdout.info(f'Changing damping factor to {damping_factor}, n_damping = {n_damping}', bare=True)
                     n_damping += 1
                     if n_damping > 10:
                         n_damping = 1
@@ -358,20 +341,20 @@ class AsymptoticCLs(AnalysisObject):
             CLs = self.LimitTool.calcCLs(qmu, sigma_b, mu_pre)
             qmu95 = self.LimitTool.getQmu95(sigma_b, mu_pre)
             
-            self.stdout.info(f"NLL:            {nll_name}")
-            self.stdout.info(f"Previous guess: {mu_pre}")
-            self.stdout.info(f"Sigma(obs):     {sigma_guess}")
-            self.stdout.info(f"Sigma(mu,0):    {sigma_b}")
-            self.stdout.info(f"muhat:          {mu_hat}")
-            self.stdout.info(f"pmu:            {pmu}")
-            self.stdout.info(f"1-pb:           {pb}")
-            self.stdout.info(f"CLs:            {CLs}")
-            self.stdout.info(f"qmu95:          {qmu95}")
-            self.stdout.info(f"qmu:            {qmu}")
-            self.stdout.info(f"qmuA0:          {qmuA}")
-            self.stdout.info(f"Precision:      {self.direction*mu_guess*self.precision}")
-            self.stdout.info(f"Correction:     {-corr}")
-            self.stdout.info(f"New guess:      {mu_guess}")
+            self.stdout.info(f"NLL:            {nll_name}", bare=True)
+            self.stdout.info(f"Previous guess: {mu_pre}", bare=True)
+            self.stdout.info(f"Sigma(obs):     {sigma_guess}", bare=True)
+            self.stdout.info(f"Sigma(mu,0):    {sigma_b}", bare=True)
+            self.stdout.info(f"muhat:          {mu_hat}", bare=True)
+            self.stdout.info(f"pmu:            {pmu}", bare=True)
+            self.stdout.info(f"1-pb:           {pb}", bare=True)
+            self.stdout.info(f"CLs:            {CLs}", bare=True)
+            self.stdout.info(f"qmu95:          {qmu95}", bare=True)
+            self.stdout.info(f"qmu:            {qmu}", bare=True)
+            self.stdout.info(f"qmuA0:          {qmuA}", bare=True)
+            self.stdout.info(f"Precision:      {self.direction*mu_guess*self.precision}", bare=True)
+            self.stdout.info(f"Correction:     {-corr}", bare=True)
+            self.stdout.info(f"New guess:      {mu_guess}", bare=True)
             
             n_iter += 1
             if (n_iter > 25):
@@ -398,16 +381,18 @@ class AsymptoticCLs(AnalysisObject):
                 self.summary[summary_label] = summary
             else:
                 self.summary[summary_label].update(summary)
-        self.stdout.info('Found limit for nll {}: {}'.format(nll_name, mu_guess))
-        self.stdout.info('Finished in {} itreations'.format(n_iter))
+        self.stdout.info(f'Found limit for nll {nll_name}: {mu_guess}', bare=True)
+        self.stdout.info(f'Finished in {n_iter} itreations', bare=True)
         return mu_guess
     
     @staticmethod
     def get_approx_limits(median_limit:float, target_cls:float=0.05):
         limits = {}
+        # 3.84 = value of delta chi^2 (qmu) corresponding to a coverage probability 1 − alpha = 0.95 in the large data sample limit
         sigma = median_limit/math.sqrt(3.84)
         limits[0]  = median_limit
         limits[2]  = sigma*(ROOT.Math.gaussian_quantile(1 - target_cls*ROOT.Math.gaussian_cdf( 2), 1) + 2)
+        
         limits[1]  = sigma*(ROOT.Math.gaussian_quantile(1 - target_cls*ROOT.Math.gaussian_cdf( 1), 1) + 1)
         limits[-1] = sigma*(ROOT.Math.gaussian_quantile(1 - target_cls*ROOT.Math.gaussian_cdf(-1), 1) - 1)
         limits[-2] = sigma*(ROOT.Math.gaussian_quantile(1 - target_cls*ROOT.Math.gaussian_cdf(-2), 1) - 2)
@@ -434,55 +419,7 @@ class AsymptoticCLs(AnalysisObject):
         self.nll_maps[nll]['mu_hat'] = mu_hat
         self.nll_maps[nll]['nll_mu_hat'] = nll_mu_hat
         self.dataset_maps[dataset] = nll
-        
-    def print_failures(self):
-        has_failtures = any(self.summary[sigma].get('status', 0) > 0 for sigma in self.summary)
-        if has_failtures:
-            self.stdout.info('-----------------------------------------------')
-            self.stdout.info('Unresolved fit failures detected')
-            for sigma in self.summary:
-                label = self._SIGMA_LABEL_.get(sigma, sigma)
-                self.stdout.info("{}:".format(label).ljust(10) + str(self.summary[sigma].get('status', None)))
-            self.stdout.info('-----------------------------------------------')
 
-    def print_summary(self):
-        if (not self.limits) or (not self.approx_limits):
-            print("No limits evaluated")
-            return None
-        print('')
-        self.print_failures()
-        if self.do_better_bands:
-            self.stdout.info('Guess for bands')
-        self.stdout.info(f'+2sigma:  {self.approx_limits[2]}')
-        self.stdout.info(f'+1sigma:  {self.approx_limits[1]}')
-        self.stdout.info(f'-1sigma:  {self.approx_limits[-1]}')
-        self.stdout.info(f'-2sigma:  {self.approx_limits[-2]}')
-        if self.do_better_bands:
-            self.stdout.info('\nCorrect bands')
-            self.stdout.info(f'+2sigma:  {self.limits[2]}')
-            self.stdout.info(f'+1sigma:  {self.limits[1]}')
-            self.stdout.info(f'-1sigma:  {self.limits[-1]}')
-            self.stdout.info(f'-2sigma:  {self.limits[-2]}')
-        self.stdout.info('Injected: {}'.format(self.limits['inj']))
-        self.stdout.info(f'Median:   {self.limits[0]}')
-        self.stdout.info('Observed: {}'.format(self.limits['obs']))
-        self.stdout.info('')
-        
-    def save(self, filename:str='limits.json', parameters:Optional[Dict]=None, summary:bool=False):
-        with open(filename, "w") as file:
-            if parameters is None:
-                json.dump(self.limits, file, indent=2)
-            else:
-                json.dump({**parameters, **self.limits}, file, indent=2)
-            file.truncate()
-        if summary:
-            data = {k:{kk:vv for kk,vv in self.summary[k].items() if kk not in ['nll', 'dataset']} \
-                    for k,v in self.summary.items()}
-            summary_filename = os.path.splitext(filename)[0]+'_summary'+os.path.splitext(filename)[1]
-            with open(summary_filename, "w") as file:
-                json.dump(data, file, indent=2)
-                file.truncate()
-        
     def evaluate_limit_band(self, n:int, med_limit:float):
         init_target_cls = self.LimitTool.getTargetCLs()
         self.LimitTool.setTargetCLs(2*(1 - ROOT.Math.gaussian_cdf(abs(n))))
@@ -494,7 +431,7 @@ class AsymptoticCLs(AnalysisObject):
         n_times_sigma = self.get_limit(asimov_0_nll, mu_guess)
         status = self.global_status
         sigma = n_times_sigma/n
-        self.stdout.info('Found N * sigma = {} * {}'.format(n, sigma))
+        self.stdout.info(f'Found N * sigma = {n} * {sigma}', bare=True)
         snapshot_0_name = self.nll_maps[asimov_0_nll]['snapshot']
         self.load_snapshot(snapshot_0_name)
         asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma,
@@ -520,7 +457,7 @@ class AsymptoticCLs(AnalysisObject):
         limit = self.get_limit(asimov_n_nll, mu_guess, n)
         return limit    
     
-    @common_utils.timer
+    @cls_method_timer
     def evaluate_limits(self):
         # TODO: maybe load nominal snapshot first
         self.reset()
@@ -533,7 +470,7 @@ class AsymptoticCLs(AnalysisObject):
         if self.asimov_data_0 is None:
             # generate asimov dataset on the fly
             fit_text = "postfit" if not self.do_blind else "nominal"
-            self.stdout.info(f"INFO: Generating asimov dataset with mu = {pretty_value(self.mu_exp)} using {fit_text} NP values")
+            self.stdout.info(f"Generating asimov dataset with mu = {pretty_value(self.mu_exp)} using {fit_text} NP values")
             asimov_data_0 = self.model.generate_asimov(self.poi.GetName(),
                                                        poi_val=self.mu_exp,
                                                        poi_profile=self.mu_exp, 
@@ -553,11 +490,14 @@ class AsymptoticCLs(AnalysisObject):
         self.save_nll_snapshot(asimov_0_nll, self.mu_exp)
         self.load_conditional_snapshot(self.mu_exp, target=WSArgument.GLOBAL_OBSERVABLE)
         self.load_conditional_snapshot(self.mu_exp, target=WSArgument.NUISANCE_PARAMETER)
+        if (self.minimizer.has_discrete_nuisance() and self.do_blind):
+            self.minimizer.minimize(asimov_0_nll)
         nll_value = asimov_0_nll.getVal()
         self.fill_mappings(asimov_0_nll, asimov_data_0, self.mu_exp, nll_value)
         
         self.LimitTool.setTargetCLs(1 - self.CL)
-        
+
+        # evaluate median expected limit
         med_limit = self.get_limit(asimov_0_nll, self.mu_guess, 0)
         approx_limits = self.get_approx_limits(med_limit, self.LimitTool.getTargetCLs())
         limits = approx_limits.copy()
@@ -584,7 +524,6 @@ class AsymptoticCLs(AnalysisObject):
         # relax the range for observed limit
         if not self.adjust_fit_range:
             self.poi.setRange(poi_range_tmp[0], poi_range_tmp[1])
-
         obs_limit = 0 if self.do_blind else self.get_limit(obs_nll, med_limit, 'obs')
         limits['obs'] = obs_limit
         
@@ -596,7 +535,7 @@ class AsymptoticCLs(AnalysisObject):
         
         self.print_summary()
         
-        self.stdout.info(f'INFO: Finished with {self.minimizer_calls} calls to minimize(nll)')
+        self.stdout.info(f'Finished with {self.minimizer_calls} calls to minimize(nll)')
         return limits
 
     def evaluate_cls(self, poi:float):
@@ -620,8 +559,8 @@ class AsymptoticCLs(AnalysisObject):
         self.fill_mappings(asimov_0_nll, asimov_data_0, 0, nll_value)
         
         nll_name = asimov_0_nll.GetName()
-        self.stdout.info("----------------------------------")
-        self.stdout.info(f"Getting CLs for nll: {nll_name}")
+        self.stdout.info("----------------------------------", bare=True)
+        self.stdout.info(f"Getting CLs for nll: {nll_name}", bare=True)
         
         qmu = self.get_qmu(asimov_0_nll, poi)
         sigma= self.LimitTool.getSigma(poi, 0, qmu)
@@ -629,11 +568,59 @@ class AsymptoticCLs(AnalysisObject):
         pb = self.LimitTool.calcPb(qmu, sigma, poi)
         CLs = self.LimitTool.calcCLs(qmu, sigma, poi)
         qmu95 = self.LimitTool.getQmu95(sigma, poi)
-        self.stdout.info(f"POI:            {poi}")
-        self.stdout.info(f"Sigma:          {sigma}")
-        self.stdout.info(f"NLL(muhat):     {nll_value}")
-        self.stdout.info(f"qmu95:          {qmu95}")
-        self.stdout.info(f"qmu:            {qmu}")
-        self.stdout.info(f"pmu:            {pmu}")
-        self.stdout.info(f"1-pb:           {pb}")
-        self.stdout.info(f"CLs:            {CLs}")
+        self.stdout.info(f"POI:            {poi}", bare=True)
+        self.stdout.info(f"Sigma:          {sigma}", bare=True)
+        self.stdout.info(f"NLL(muhat):     {nll_value}", bare=True)
+        self.stdout.info(f"qmu95:          {qmu95}", bare=True)
+        self.stdout.info(f"qmu:            {qmu}", bare=True)
+        self.stdout.info(f"pmu:            {pmu}", bare=True)
+        self.stdout.info(f"1-pb:           {pb}", bare=True)
+        self.stdout.info(f"CLs:            {CLs}", bare=True)
+
+    def print_failures(self):
+        has_failtures = any(self.summary[sigma].get('status', 0) > 0 for sigma in self.summary)
+        if has_failtures:
+            self.stdout.info('-----------------------------------------------', bare=True)
+            self.stdout.info('Unresolved fit failures detected', bare=True)
+            for sigma in self.summary:
+                label = self._SIGMA_LABEL_.get(sigma, sigma)
+                self.stdout.info("{}:".format(label).ljust(10) + str(self.summary[sigma].get('status', None)), bare=True)
+            self.stdout.info('-----------------------------------------------', bare=True)
+
+    def print_summary(self):
+        if (not self.limits) or (not self.approx_limits):
+            self.stdout.warning("No limits evaluated")
+            return None
+        self.stdout.info('', bare=True)
+        self.print_failures()
+        if self.do_better_bands:
+            self.stdout.info('Guess for bands', bare=True)
+        self.stdout.info(f'+2sigma:  {self.approx_limits[2]}', bare=True)
+        self.stdout.info(f'+1sigma:  {self.approx_limits[1]}', bare=True)
+        self.stdout.info(f'-1sigma:  {self.approx_limits[-1]}', bare=True)
+        self.stdout.info(f'-2sigma:  {self.approx_limits[-2]}', bare=True)
+        if self.do_better_bands:
+            self.stdout.info('\nCorrect bands', bare=True)
+            self.stdout.info(f'+2sigma:  {self.limits[2]}', bare=True)
+            self.stdout.info(f'+1sigma:  {self.limits[1]}', bare=True)
+            self.stdout.info(f'-1sigma:  {self.limits[-1]}', bare=True)
+            self.stdout.info(f'-2sigma:  {self.limits[-2]}', bare=True)
+        self.stdout.info('Injected: {}'.format(self.limits['inj']), bare=True)
+        self.stdout.info(f'Median:   {self.limits[0]}', bare=True)
+        self.stdout.info('Observed: {}'.format(self.limits['obs']), bare=True)
+        self.stdout.info('', bare=True)
+        
+    def save(self, filename:str='limits.json', parameters:Optional[Dict]=None, summary:bool=False):
+        with open(filename, "w") as file:
+            if parameters is None:
+                json.dump(self.limits, file, indent=2)
+            else:
+                json.dump({**parameters, **self.limits}, file, indent=2)
+            file.truncate()
+        if summary:
+            data = {k:{kk:vv for kk,vv in self.summary[k].items() if kk not in ['nll', 'dataset']} \
+                    for k,v in self.summary.items()}
+            summary_filename = os.path.splitext(filename)[0]+'_summary'+os.path.splitext(filename)[1]
+            with open(summary_filename, "w") as file:
+                json.dump(data, file, indent=2)
+                file.truncate()
\ No newline at end of file
