@@ -12,7 +12,8 @@ from quickstats import DescriptiveEnum
 from quickstats.maths.numerics import pretty_value
 from quickstats.components.basics import WSArgument
 from quickstats.components import AnalysisObject
-from quickstats.utils.common_utils import parse_config
+from quickstats.utils.common_utils import parse_config, combine_dict
+from quickstats.parsers import RooParamSetupParser
 
 class FitMode(DescriptiveEnum):
     HYBRID  = (0, "Execute both conditional and unconditional fits for a given set of POIs")
@@ -23,37 +24,23 @@ class FitMode(DescriptiveEnum):
 
 class Likelihood(AnalysisObject):
     
-    def __init__(self, filename:str, poi_name:Optional[Union[str, List[str]]]=None,
+    def __init__(self, filename:str,
+                 poi_name:Optional[Union[str, List[str]]]=None,
                  data_name:str='combData', 
                  config:Optional[Dict]=None,
-                 verbosity:Optional[Union[int, str]]="INFO"):
+                 verbosity:Optional[Union[int, str]]="INFO",
+                 **kwargs):
         config = parse_config(config)
+        config = combine_dict(config, kwargs)
         config['filename']  = filename
         config['poi_name']  = poi_name
         config['data_name'] = data_name
         config['verbosity'] = verbosity
+        if 'preset_param' not in config:
+            config['preset_param'] = True
         self._inherit_init(super().__init__, **config)
         self.roofit_result = None
     
-    @staticmethod
-    def _parse_poi_val(pois:"ROOT.RooArgSet", poi_val:Optional[Union[Dict[str, float], float]]=None):
-        if poi_val is None:
-            poi_val = {}
-            for poi in pois:
-                poi_val[poi.GetName()] = poi.getVal()
-        elif not isinstance(poi_val, dict):
-            poi_val_tmp = poi_val
-            poi_val = {}
-            for poi in pois:
-                poi_val[poi.GetName()] = poi_val_tmp
-        else:
-            # fill in missing values
-            for poi in pois:
-                poi_name = poi.GetName()
-                if poi_name not in poi_val:
-                    poi_val[poi_name] = poi.getVal()
-        return poi_val
-    
     @staticmethod
     def get_combined_fit_result(uncond_fit_result:Dict, cond_fit_result:Dict):
         combined_fit_result = {}
@@ -70,13 +57,17 @@ class Likelihood(AnalysisObject):
                 # ndof = 1 case
                 poi_name = list(poi_val)[0]
                 x0 = poi_val[poi_name]
-                significance = math.sqrt(qmu)
+                if x0 == 0:
+                    sign = np.sign(uncond_fit_result['muhat'][poi_name])
+                    significance = sign * math.sqrt(qmu)
+                else:
+                    significance = math.sqrt(qmu)
                 pvalue = 1 - ROOT.Math.normal_cdf(significance, 1, x0)
             else:
                 x0 = list(set(poi_val.values()))
                 if len(x0) > 1:
-                    combined_fit_result['significance'] = None
-                    combined_fit_result['pvalue'] = None
+                    pvalue = None
+                    significance = None
                 else:
                     pvalue = ROOT.Math.chisquared_cdf_c(qmu, ndof, x0[0])
                     significance = ROOT.RooStats.PValueToSignificance(pvalue)
@@ -85,6 +76,10 @@ class Likelihood(AnalysisObject):
         else:
             combined_fit_result['significance'] = None
             combined_fit_result['pvalue'] = None
+        if (uncond_fit_result['status'] != 0) or (cond_fit_result['status'] != 0):
+            combined_fit_result['status'] = -1
+        else:
+            combined_fit_result['status'] = 0
         return combined_fit_result
         
     def nll_fit(self, poi_val:Optional[Union[Dict[str, float], float]]=None,
@@ -134,19 +129,16 @@ class Likelihood(AnalysisObject):
             }
         if "uncond" not in snapshot_name:
             snapshot_name["uncond"] = self.kCurrentSnapshotName
-            self.stdout.info(f'INFO: No snapshot given for unconditional fit. The snapshot "{self.kCurrentSnapshotName}" '
+            self.stdout.info(f'No snapshot given for unconditional fit. The snapshot "{self.kCurrentSnapshotName}" '
                              'will be used by default')
         if "cond" not in snapshot_name:
             snapshot_name["cond"] = self.kCurrentSnapshotName
-            self.stdout.info(f'INFO: No snapshot given for conditional fit. The snapshot "{self.kCurrentSnapshotName}" '
+            self.stdout.info(f'No snapshot given for conditional fit. The snapshot "{self.kCurrentSnapshotName}" '
                              'will be used by default')
         if set(snapshot_name.keys()) != set(["uncond", "cond"]):
             raise ValueError('when snapshot name is given as a dictionary, it must be of the '
             'form {"cond": <conditional_fit_snapshot>, "uncond": <unconditional_fit_snapshot>}')
-        
-        self.load_snapshot(snapshot_name["cond"])
-        poi_val = self._parse_poi_val(pois, poi_val)
-        
+
         nll_uncond = None
         nll_cond   = None
         muhat      = None
@@ -167,12 +159,6 @@ class Likelihood(AnalysisObject):
         if mode == FitMode.NOMINAL:
             # restore initial parameter values after each fit
             self.load_snapshot(snapshot_name["cond"])
-            if len(pois) > 0:
-                # fix other pois if they are not studied
-                for poi in self.model.pois:
-                    poi.setConstant(1)
-                for poi in pois:
-                    poi.setConstant(0)
             self.minimizer.create_nll()
             prefit_nll = self.minimizer.nll.getVal()
             fit_status = self.minimizer.minimize()
@@ -209,6 +195,7 @@ class Likelihood(AnalysisObject):
         if mode in [FitMode.HYBRID, FitMode.COND]:
             # restore initial parameter values after each fit
             self.load_snapshot(snapshot_name["cond"])
+            poi_val = RooParamSetupParser.parse(pois, poi_val, fill_missing=True)
             # fix other pois if they are not studied
             for poi in self.model.pois:
                 poi.setConstant(1)
@@ -227,11 +214,11 @@ class Likelihood(AnalysisObject):
         fit_time_cond = time.time() - fit_time_uncond - start_time
         
         fit_result = {}
-        self.stdout.info("INFO: NLL evaluation completed with")
+        self.stdout.info("NLL evaluation completed with")
         if mode in [FitMode.HYBRID, FitMode.UNCOND, FitMode.NOFIT]:
             best_fit_str = ", ".join([f"{name} = {value:.5f}" for name, value in muhat.items()])
-            self.stdout.info("best fit : ".rjust(15) + f"{best_fit_str}")
-            self.stdout.info("uncond NLL = ".rjust(15) + f"{nll_uncond:.5f}")
+            self.stdout.info("best fit : ".rjust(15) + f"{best_fit_str}", bare=True)
+            self.stdout.info("uncond NLL = ".rjust(15) + f"{nll_uncond:.5f}", bare=True)
             fit_result['uncond_fit'] = {
                 'muhat': muhat,
                 'muhat_errlo': muhat_errlo,
@@ -243,12 +230,12 @@ class Likelihood(AnalysisObject):
         if mode in [FitMode.HYBRID, FitMode.COND]:
             mu = {name:pretty_value(value) for name, value in poi_val.items()}
             mu_str = ", ".join([f"{name} = {pretty_value(value)}" for name, value in poi_val.items()])
-            self.stdout.info("mu : ".rjust(15) + f"{mu_str}")
+            self.stdout.info("cond value : ".rjust(15) + f"{mu_str}", bare=True)
             # also add the best-fit value for profiled POIs
             for name, value in poi_val.items():
                 if value == None:
                     muhat[name] = self.model.workspace.var(name).getVal()
-            self.stdout.info("cond NLL = ".rjust(15)+f"{nll_cond:.5f}")
+            self.stdout.info("cond NLL = ".rjust(15)+f"{nll_cond:.5f}", bare=True)
             fit_result['cond_fit'] = {
                 'mu': mu,
                 'muhat': muhat,
@@ -258,44 +245,20 @@ class Likelihood(AnalysisObject):
             }
             
         if mode == FitMode.HYBRID:
-            pnll = nll_cond - nll_uncond
-            qmu  = 2*pnll
-            self.stdout.info("PNLL = ".rjust(15) + "{:.5f}".format(nll_cond - nll_uncond))
-            fit_result['pnll'] = pnll
-            fit_result['qmu']  = qmu
-            
-            if (qmu >= 0):
-                ndof = len(pois)
-                if ndof == 1:
-                    # ndof = 1 case
-                    poi_name = pois.first().GetName()
-                    x0 = poi_val[poi_name]
-                    significance = math.sqrt(qmu)
-                    pvalue = 1 - ROOT.Math.normal_cdf(significance, 1, x0)
-                else:
-                    x0 = list(set(poi_val.values()))
-                    if len(x0) > 1:
-                        fit_result['significance'] = None
-                        fit_result['pvalue'] = None
-                    else:
-                        pvalue = ROOT.Math.chisquared_cdf_c(qmu, ndof, x0[0])
-                        significance = ROOT.RooStats.PValueToSignificance(pvalue)
-                fit_result['significance'] = significance
-                fit_result['pvalue'] = pvalue
-            else:
-                fit_result['significance'] = None
-                fit_result['pvalue'] = None
+            hybrid_result = self.get_combined_fit_result(fit_result['uncond_fit'], fit_result['cond_fit'])
+            fit_result.update(hybrid_result)
                 
         if mode != FitMode.NOMINAL:
             self.stdout.info("time = ".rjust(15) + \
-                             "(uncond_fit) {:.3f}, (cond_fit) {:.3f}".format(fit_time_uncond, fit_time_cond))
+                             "(uncond_fit) {:.3f}, (cond_fit) {:.3f}".format(fit_time_uncond, fit_time_cond), bare=True)
             fit_result['total_time'] = fit_time_uncond + fit_time_cond
         else:
-            self.stdout.info("prefit NLL = ".rjust(18) + f"{prefit_nll:.5f}")
-            self.stdout.info("postfit NLL = ".rjust(18) + f"{postfit_nll:.5f}")
-            self.stdout.info("time = ".rjust(18) + f"{fit_time:.3f}")
+            self.stdout.info("prefit NLL = ".rjust(18) + f"{prefit_nll:.5f}", bare=True)
+            self.stdout.info("postfit NLL = ".rjust(18) + f"{postfit_nll:.5f}", bare=True)
+            self.stdout.info("time = ".rjust(18) + f"{fit_time:.3f}", bare=True)
             fit_result['prefit_nll'] = prefit_nll
             fit_result['postfit_nll'] = postfit_nll
+            fit_result['status'] =  fit_status
             fit_result['total_time'] = fit_time
         
         self.roofit_result = roofit_result
