@@ -27,6 +27,7 @@ class DataModelling(ROOTObject):
         'minos': False,
         'hesse': True,
         'sumw2': True,
+        'asymptotic': False,
         'strategy': 1,
         'range_expand_rate': 1        
     }
@@ -197,7 +198,8 @@ class DataModelling(ROOTObject):
     @semistaticmethod
     def _run(self, filename:str, tree_name:str, model_class:Callable, param_templates:Callable,
              fit_options:Dict, plot_options:Optional[Dict]=None, save_summary_as:Optional[str]=None, 
-             save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None):
+             save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None,
+             save_data_as:Optional[str]=None):
         
         with Timer() as t:
             canvas = ROOT.TCanvas(uuid.uuid4().hex)
@@ -224,6 +226,7 @@ class DataModelling(ROOTObject):
                 'hesse'             : fit_options['hesse'],
                 'strategy'          : fit_options['strategy'],
                 'sumw2'             : fit_options['sumw2'],
+                'asymptotic'        : fit_options['asymptotic'],
                 'min_fit'           : fit_options['min_fit'],
                 'max_fit'           : fit_options['max_fit'],
                 'range_expand_rate' : fit_options['range_expand_rate'],
@@ -242,7 +245,7 @@ class DataModelling(ROOTObject):
             canvas.Close()
             ROOT.gSystem.ProcessEvents()
             
-        self.stdout.info(f"INFO: Task finished. Total time taken: {t.interval:.3f}s")
+        self.stdout.info(f"Task finished. Total time taken: {t.interval:.3f}s")
         
         param_data = self.get_param_data(model_parameters)
         param_data_value_only = self.get_param_data(model_parameters, value_only=True)
@@ -258,7 +261,8 @@ class DataModelling(ROOTObject):
             self.create_plot(dataset, model_pdf, observable,
                              plot_options=plot_options,
                              model_summary=summary,
-                             save_as=save_plot_as)
+                             save_plot_as=save_plot_as,
+                             save_data_as=save_data_as)
 
         if save_param_as is not None:
             with open(save_param_as, "w") as outfile:
@@ -270,12 +274,42 @@ class DataModelling(ROOTObject):
 
         return summary
     
+    @semistaticmethod
+    def get_model_summary_text(self, model_summary:Dict,
+                               value_fmt:str="{:.2f}",
+                               show_params:bool=True,
+                               show_stats:bool=True,
+                               show_error:bool=True,
+                               stats_list:Optional[List[str]]=None):
+        text = ""
+        if show_params:
+            param_data = model_summary["parameters"]
+            for param in param_data:
+                value = value_fmt.format(param_data[param]["value"])
+                if show_error:
+                    error = value_fmt.format(param_data[param]["error"])
+                    text += f"{param} = {value} $\\pm$ {error}\n"
+                else:
+                    text += f"{param} = {value}\n"
+            text += "\n"
+        if show_stats:
+            if stats_list is None:
+                stats_list = list(model_summary["stats"])
+            for stat in stats_list:
+                if stat not in model_summary["stats"]:
+                    raise RuntimeError(f"invalid stats item: {stat}")
+                value = value_fmt.format(model_summary["stats"][stat])
+                text += f"{stat} = {value}\n"
+            text += "\n"
+        return text
+    
     @semistaticmethod
     def create_plot(self, data:ROOT.RooDataSet, pdf:ROOT.RooAbsPdf,
                     observable:Optional[ROOT.RooRealVar]=None,
                     plot_options:Optional[Dict]=None,
                     model_summary:Optional[Dict]=None,
-                    save_as:Optional[str]=None):
+                    save_plot_as:Optional[str]=None,
+                    save_data_as:Optional[str]=None):
         
         if observable is None:
             parameters = data.get()
@@ -301,26 +335,14 @@ class DataModelling(ROOTObject):
         plotter = PdfDistributionPlot.from_roofit_data(data, pdf, observable, **init_kwargs)
         
         if model_summary is not None:
-            value_fmt = plot_options["value_fmt"]
-            annotation_text = ""
-            if plot_options['show_params']:
-                param_data = model_summary["parameters"]
-                for param in param_data:
-                    value = value_fmt.format(param_data[param]["value"])
-                    if plot_options["show_error"]:
-                        error = value_fmt.format(param_data[param]["error"])
-                        annotation_text += f"{param} = {value} $\\pm$ {error}\n"
-                    else:
-                        annotation_text += f"{param} = {value}\n"
-                annotation_text += "\n"
-            if plot_options["show_stats"]:
-                stats_list = plot_options["stats_list"]
-                for stat in stats_list:
-                    if stat not in model_summary["stats"]:
-                        raise RuntimeError(f"invalid stats item: {stat}")
-                    value = value_fmt.format(model_summary["stats"][stat])
-                    annotation_text += f"{stat} = {value}\n"
-                annotation_text += "\n"
+            kwargs = {
+                "value_fmt"   : plot_options["value_fmt"],
+                "show_params" : plot_options['show_params'],
+                "show_stats"  : plot_options["show_stats"],
+                "show_error"  : plot_options["show_error"],
+                "stats_list"  : plot_options["stats_list"]
+            }
+            annotation_text = self.get_model_summary_text(model_summary, **kwargs)
             plotter.add_annotation(annotation_text)
         
         annotations = plot_options.get("annotations", None)
@@ -359,21 +381,27 @@ class DataModelling(ROOTObject):
                           comparison_options=comparison_options, **draw_options)
         
         import matplotlib.pyplot as plt
-        if save_as is not None:
-            plt.savefig(save_as, bbox_inches="tight")
+        if save_plot_as is not None:
+            plt.savefig(save_plot_as, bbox_inches="tight")
+        if save_data_as is not None:
+            from quickstats.utils.common_utils import NpEncoder
+            json.dump(plotter.collective_data, open(save_data_as, 'w'), cls=NpEncoder)
+            
         if in_notebook():
             plt.show()
         return ax
 
     def run(self, filename:str, tree_name:str, save_summary_as:Optional[str]=None, 
-            save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None):
+            save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None,
+            save_data_as:Optional[str]=None):
         summary = self._run(filename, tree_name, model_class=self.model_class, 
                             param_templates=self.param_templates,
                             fit_options=self.fit_options,
                             plot_options=self.plot_options,
                             save_summary_as=save_summary_as,
                             save_param_as=save_param_as,
-                            save_plot_as=save_plot_as)
+                            save_plot_as=save_plot_as,
+                            save_data_as=save_data_as)
         return summary
     
     @staticmethod
