@@ -1,3 +1,3 @@
 from quickstats.components.processors.actions import *
-from quickstats.components.processors.parsers import RooProcConfigParser
+from quickstats.components.processors.roo_config_parser import RooConfigParser
 from quickstats.components.processors.roo_processor import RooProcessor
\ No newline at end of file
