@@ -10,10 +10,11 @@ class RooProcAlias(RooProcRDFAction):
         
     @classmethod
     def parse(cls, main_text:str, block_text:Optional[str]=None):
-        result = re.search(r"^\s*(\w+)\s*=\s*([\w\.]+)\s*$", main_text)
+        result = re.search(r"^\s*(\w+)\s*=\s*([\w\.\${}]+)\s*$", main_text)
         if not result:
-            if re.search(r"^\s*(\w+)\s*=\s*([\w\.]+)", main_text):
-                raise RuntimeError(f"can not alias an expression, please use DEFINE instead")
+            if re.search(r"^\s*(\w+)\s*=\s*([\w\.\${}]+)", main_text):
+                raise RuntimeError(f'can not alias an expression ("{main_text}"), '
+                                   'please use DEFINE instead')
             raise RuntimeError(f"invalid expression {main_text}")
         alias = result.group(1)
         column_name = result.group(2)
