@@ -4,6 +4,8 @@ import json
 
 from .rooproc_helper_action import RooProcHelperAction
 
+from quickstats.utils.common_utils import is_valid_file
+
 class RooProcExport(RooProcHelperAction):
     def __init__(self, filename:str):
         super().__init__(filename=filename)
@@ -15,6 +17,9 @@ class RooProcExport(RooProcHelperAction):
     
     def _execute(self, processor:"quickstats.RooProcessor", **params):
         filename = params['filename']
+        if processor.cache and is_valid_file(filename):
+            processor.stdout.info(f"INFO: Cached output `{filename}`.")
+            return processor   
         data = {k:v.GetValue() for k,v in processor.external_variables.items()}
         dirname = os.path.dirname(filename)
         if dirname and (not os.path.exists(dirname)):
