@@ -13,6 +13,6 @@ class RooProcSaveFrame(RooProcHelperAction):
     def _execute(self, processor:"quickstats.RooProcessor", **params):
         frame_name = params['name']
         if frame_name in processor.rdf_frames:
-            processor.stdout.warning(f"WARNING: Overriding existing rdf frame `{frame_name}`")
+            processor.stdout.warning(f'Overriding existing rdf frame "{frame_name}"')
         processor.rdf_frames[frame_name] = processor.rdf
         return processor
\ No newline at end of file
