@@ -15,27 +15,27 @@ class RooInspector(AbstractObject):
                  verbosity:Optional[Union[int, str]]="INFO"):
         super().__init__(verbosity=verbosity)
         self.tree_name = tree_name
-        self.fnames = self._parse_file_expr(file_expr)
+        self.filenames = self._parse_file_expr(file_expr)
         self.initialise(filter_expr=filter_expr)
         
     def _parse_file_expr(self, file_expr:Union[str, List[str]])->List[str]:
-        fnames = []
+        filenames = []
         if isinstance(file_expr, str):
             return self._parse_file_expr([file_expr])
         else:
             for expr in file_expr:
                 if os.path.isdir(expr):
                     file_expr = os.path.join(expr, "*.root")
-                fnames_i = glob.glob(expr)
-                if not fnames_i:
-                    self.stdout.warning(f"WARNING: No root files found matching the expression {expr}")
-                fnames += fnames_i
-        if not fnames:
+                filenames_i = glob.glob(expr)
+                if not filenames_i:
+                    self.stdout.warning(f"No root files found matching the expression {expr}")
+                filenames.extend(filenames_i)
+        if not filenames:
             raise RuntimeError("no root files found from the given file expression")
-        return fnames
+        return filenames
     
     def initialise(self, filter_expr:Optional[str]=None):
-        self.rdf = ROOT.RDataFrame(self.tree_name, self.fnames)
+        self.rdf = ROOT.RDataFrame(self.tree_name, self.filenames)
         if filter_expr is not None:
             self.rdf = self.rdf.Filter(filter_expr)
         
@@ -77,7 +77,7 @@ class RooInspector(AbstractObject):
             ctype_str = "(" + ctype + ")"
             summary_str += f"{ctype_str:<30}{cname}\n"
         if not suppress_print:
-            self.stdout.info(summary_str)
+            self.stdout.info(summary_str, bare=True)
         if save_as is not None:
             with open(save_as, "w") as f:
                 f.write(summary_str)
