@@ -9,7 +9,6 @@ import numpy as np
 
 import quickstats
 from quickstats import semistaticmethod, AbstractObject, GeneralEnum, PathManager
-from quickstats.utils.io import Verbosity
 from quickstats.utils.xml_tools import TXMLTree
 from quickstats.maths.numerics import to_bool
 from quickstats.utils.root_utils import load_macro, get_macro_TClass, get_macro_dir
@@ -73,7 +72,7 @@ class XMLWSBase(AbstractObject):
         self.path_manager = PathManager(self.basedir)
         if unlimited_stack:
             set_unlimited_stacksize()
-            self.stdout.info("INFO: Set stack size to unlimited")
+            self.stdout.info("Set stack size to unlimited")
         self.suppress_roofit_message()
         self.custom_classes = {}
 
@@ -82,7 +81,7 @@ class XMLWSBase(AbstractObject):
         for extension in extensions:
             result = load_macro(extension)
             if (result is not None) and hasattr(ROOT, extension):
-                self.stdout.info(f'INFO: Loaded extension module "{extension}"')
+                self.stdout.info(f'Loaded extension module "{extension}"')
             try:
                 cls = get_macro_TClass(extension)
             except:
@@ -183,9 +182,9 @@ class XMLWSBase(AbstractObject):
         # throw error? when attemping to define already existing object
         if check_defined:
             if ws.arg(object_name):
-                self.stdout.debug(f"REGTEST: object {object_name} already exists")
+                self.stdout.debug(f"object {object_name} already exists")
                 return object_name
-        self.stdout.debug(f"REGTEST: Generating {object_type} {object_expr}")
+        self.stdout.debug(f"Generating {object_type} {object_expr}")
         status = ws.factory(object_expr)
         if not status:
             raise RuntimeError(f"object creation from expression `{object_expr}` had failed")
@@ -242,7 +241,7 @@ class XMLWSBase(AbstractObject):
             ws.addClassDeclImportDir(macro_dir)
             ws.addClassImplImportDir(macro_dir)
             ws.importClassCode(cls)
-            self.stdout.info(f"INFO: Imported class code for \"{cls.GetName()}\".")
+            self.stdout.info(f"Imported class code for \"{cls.GetName()}\".")
         #ws.importClassCode()    
     
     def _generate_asimov(self, ws:ROOT.RooWorkspace, asimov_definitions:List, data_name:str,
