@@ -1,6 +1,6 @@
 ###############################################################################
 ### This is a reimplementation of workspaceCombiner library in python
-### original author: Hongtao Yang
+### original author: Hongtao Yang, Haoshuang Ji
 ###############################################################################
 import os
 import re
@@ -19,12 +19,12 @@ import ROOT
 import quickstats
 from quickstats import semistaticmethod, AbstractObject, GeneralEnum, Timer
 from quickstats.components import ExtendedModel
-from quickstats.utils.io import Verbosity
 from quickstats.utils.xml_tools import TXMLTree
 from quickstats.utils.root_utils import load_macro, get_macro_dir, is_corrupt
 from quickstats.utils.common_utils import format_delimiter_enclosed_text
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, BracketType
+from quickstats.interface.root import RooDataSet
 
 class XMLWSCombiner(XMLWSBase):
     
@@ -126,7 +126,7 @@ class XMLWSCombiner(XMLWSBase):
             raise RuntimeError(msg)
         else:
             msg = msg[0].capitalize() + msg[1:]
-            self.stdout.warning(f"WARNING: {msg}. Skipped.")
+            self.stdout.warning(f"{msg}. Skipped.")
     
     def _channel_check(self, channel:str, check_poi_map:bool=False):
         if channel not in self.channel_config:
@@ -148,7 +148,7 @@ class XMLWSCombiner(XMLWSBase):
             filename = self._get_comb_ws_path()
         else:
             raise ValueError(f"invalid cache workspace type: {ws_type}")
-        self.stdout.info(f'INFO: Reading temporary workspace file from "{filename}"')
+        self.stdout.info(f'Reading temporary workspace file from "{filename}"')
         if not os.path.exists(filename):
             raise FileNotFoundError(f'temporary workspace file "{filename}" does not exist')
         file = ROOT.TFile(filename)
@@ -259,7 +259,7 @@ class XMLWSCombiner(XMLWSBase):
             "var"  : {}
         }
         self.channel_config[channel]['renamed_constraint'] = {}
-        self.stdout.info(f"INFO: Checking XML input for the channel \"{channel}\".")
+        self.stdout.info(f"Checking XML input for the channel \"{channel}\".")
         # walk through child nodes
         child_nodes = node["children"]
         for child_node in child_nodes:
@@ -272,7 +272,7 @@ class XMLWSCombiner(XMLWSBase):
                 self.read_channel_argset_node(channel, child_node)
             else:
                 raise RuntimeError(f"unknown item `{tag}`")
-        self.stdout.info(f"INFO: Successfully validated XML input for the channel \"{channel}\".")
+        self.stdout.info(f"Successfully validated XML input for the channel \"{channel}\".")
     
     def read_channel_rename_node(self, channel:str, node:Dict) -> None:
         ext_file = self._get_node_attrib(node, "InputFile", required=False)
@@ -296,11 +296,11 @@ class XMLWSCombiner(XMLWSBase):
                 
     def validate_channel_input(self, channel:str):
         self._channel_check(channel)
-        self.stdout.info(f"INFO: Checking input workspace for the channel \"{channel}\".")
+        self.stdout.info(f"Checking input workspace for the channel \"{channel}\".")
         channel_config = self.channel_config[channel]
         model = self._get_channel_model(channel_config)
         strict_mode = self.combination_config["strict"]
-        ws_filename = model.fname
+        ws_filename = model.config['filename']
         renamed_pdfs = list(channel_config["rename_map"]["pdf"])
         renamed_vars = list(channel_config["rename_map"]["var"])
         renamed_pois = list(channel_config["poi_map"].values())
@@ -317,8 +317,8 @@ class XMLWSCombiner(XMLWSBase):
                 self._exception_check(msg, strict_mode)
                 continue
             if not isinstance(pdf, ROOT.RooGaussian):
-                msg = f"constraint PDF \"{pdf_name}\" that is not an instance of RooGaussian is not supported"
-                self._exception_check(msg, strict_mode)
+                #msg = f"constraint PDF \"{pdf_name}\" that is not an instance of RooGaussian is not supported"
+                #self._exception_check(msg, strict_mode)
                 continue
             nuis = model.workspace.var(nuis_name)
             if not nuis:
@@ -368,11 +368,11 @@ class XMLWSCombiner(XMLWSBase):
                 if channel_config["argset_strict"]:
                     raise RuntimeError(f'Argument set "{argset}" does not exist in the channel "{channel}"')
                 else:
-                    self.stdout.warning(f'WARNING: Argument set "{argset}" does not exist in the channel "{channel}". Skipped.')
+                    self.stdout.warning(f'Argument set "{argset}" does not exist in the channel "{channel}". Skipped.')
                     continue
             if not isinstance(argset_obj, ROOT.RooArgSet):
                 raise RuntimeError(f'Argument set "{argset}" from the channel "{channel}" is an instance of RooArgSet object')
-        self.stdout.info(f"INFO: Successfully validated input workspace for the channel \"{channel}\".")
+        self.stdout.info(f"Successfully validated input workspace for the channel \"{channel}\".")
         
     def read_rename_syst_node(self, channel:str, node:Dict) -> None:
         self._channel_check(channel)
@@ -449,7 +449,7 @@ class XMLWSCombiner(XMLWSBase):
             if arg_name in ignore_list:
                 continue
             arg.SetName(f"{arg_name}_{channel}")
-            self.stdout.debug(f'DEBUG: Appended channel suffix "{channel}" to the {arg_type} '
+            self.stdout.debug(f'Appended channel suffix "{channel}" to the {arg_type} '
                               f'"{arg_name}"')
     
     def _get_tmp_ws_path(self):
@@ -527,13 +527,13 @@ class XMLWSCombiner(XMLWSBase):
     
     def print_input_summary(self) -> None:
         summary_text = self.get_summary_text()
-        self.stdout.info(summary_text)
+        self.stdout.info(summary_text, bare=True)
         
     def rename_channel(self, channel:str) -> None:
         self._channel_check(channel, check_poi_map=True)
         channel_config = self.channel_config[channel]
         simplified_import = channel_config["simplified_import"]
-        self.stdout.info(f'INFO: Renaming objects for the channel "{channel}"')
+        self.stdout.info(f'Renaming objects for the channel "{channel}"')
         # do not reuse model
         model = self._get_channel_model(channel_config)
         workspace = model.workspace
@@ -551,10 +551,10 @@ class XMLWSCombiner(XMLWSBase):
             if not simplified_import:
                 pdfs.remove(old_pdf)
             if old_name == new_name:
-                self.stdout.debug(f'DEBUG: Found redundant renaming of the pdf "{old_name}". Skipped.')
+                self.stdout.debug(f'Found redundant renaming of the pdf "{old_name}". Skipped.')
                 continue
             old_pdf.SetName(new_name)
-            self.stdout.debug(f'DEBUG: Renamed pdf from "{old_name}" to "{new_name}"')
+            self.stdout.debug(f'Renamed pdf from "{old_name}" to "{new_name}"')
         # rename variable
         for old_name, new_name in rename_map["var"].items():
             old_var = workspace.var(old_name)
@@ -563,10 +563,10 @@ class XMLWSCombiner(XMLWSBase):
             else:
                 ignore_list.append(new_name)
             if old_name == new_name:
-                self.stdout.debug(f'DEBUG: Found redundant renaming of the variable "{old_name}". Skipped.')
+                self.stdout.debug(f'Found redundant renaming of the variable "{old_name}". Skipped.')
                 continue
             old_var.SetName(new_name)
-            self.stdout.debug(f'DEBUG: Renamed variable from "{old_name}" to "{new_name}"')
+            self.stdout.debug(f'Renamed variable from "{old_name}" to "{new_name}"')
         # rename pois
         poi_map = channel_config['poi_map']
         for comb_poi_name, channel_poi_name in poi_map.items():
@@ -578,12 +578,12 @@ class XMLWSCombiner(XMLWSBase):
             else:
                 ignore_list.append(comb_poi_name)
             if comb_poi_name == channel_poi_name:
-                self.stdout.debug(f'DEBUG: Found redundant renaming of the poi "{channel_poi_name}". Skipped.')
+                self.stdout.debug(f'Found redundant renaming of the poi "{channel_poi_name}". Skipped.')
                 continue
             poi.SetName(comb_poi_name)
-            self.stdout.debug(f'DEBUG: Renamed poi from "{channel_poi_name}" to "{comb_poi_name}"')
+            self.stdout.debug(f'Renamed poi from "{channel_poi_name}" to "{comb_poi_name}"')
         if simplified_import:
-            self.stdout.info(f'INFO: Simplified import requested for the channel "{channel}". '
+            self.stdout.info(f'Simplified import requested for the channel "{channel}". '
                              'Only unspecified constraint PDFs, nuisance parameters, '
                              'global observables and POIs will be renamed.')
             global_observables = model.global_observables
@@ -622,11 +622,11 @@ class XMLWSCombiner(XMLWSBase):
             
             # add channel suffix to functions
             self._append_channel_suffix_to_args(channel, functions, "function")
-            self.stdout.debug(f'DEBUG: All functions in channel "{channel}" have been renamed')
+            self.stdout.debug(f'All functions in channel "{channel}" have been renamed')
             self._append_channel_suffix_to_args(channel, pdfs, "PDF")
-            self.stdout.debug(f'DEBUG: All PDFs in channel "{channel}" have been renamed')
+            self.stdout.debug(f'All PDFs in channel "{channel}" have been renamed')
             self._append_channel_suffix_to_args(channel, variables, "variable")
-            self.stdout.debug(f'DEBUG: All variables in channel "{channel}" have been renamed')
+            self.stdout.debug(f'All variables in channel "{channel}" have been renamed')
         
         # rename key objects
         pdf  = model.pdf
@@ -637,8 +637,7 @@ class XMLWSCombiner(XMLWSBase):
         old_cat_name = cat.GetName()
         new_cat_name = f"{self.KEYWORDS['comb_cat_name']}_{channel}"
         pdf.SetName(f"{self.KEYWORDS['comb_pdf_name']}_{channel}")
-        data.SetName(f"{self.combination_config['data_name']}_{channel}")
-        
+        data.SetName(f"{self.KEYWORDS['comb_data_name']}_{channel}")
         self.import_object(self.tmp_ws, pdf,
                            ROOT.RooFit.RenameVariable(old_cat_name, new_cat_name),
                            ROOT.RooFit.RecycleConflictNodes(), silent=True)
@@ -675,7 +674,7 @@ class XMLWSCombiner(XMLWSBase):
     def rename(self, save_rename_ws:bool=True) -> None:
         title = format_delimiter_enclosed_text(f"Rename PDFs, functions and variables",
                                                delimiter="-", indent_str="")
-        self.stdout.info(title)
+        self.stdout.info(title, bare=True)
         self.reset_tmp_ws()
         for channel in self.channel_config:
             self.rename_channel(channel)
@@ -683,13 +682,13 @@ class XMLWSCombiner(XMLWSBase):
             self.tmp_ws.defineSet(self.KEYWORDS['comb_nuis_name'], self.tmp_nuis, True);
             self.tmp_ws.defineSet(self.KEYWORDS['comb_glob_name'], self.tmp_glob, True);
             filename = self._get_tmp_ws_path()
-            self.stdout.info(f'INFO: Saving temporary workspace to "{filename}"')
+            self.stdout.info(f'Saving temporary workspace to "{filename}"')
             self.tmp_ws.writeToFile(filename, True)
             
     def combine_channel(self, channel:str):
         title = format_delimiter_enclosed_text(f"Channel {channel}",
                                                delimiter="+", indent_str="")
-        self.stdout.info(title)
+        self.stdout.info(title, bare=True)
         channel_pdf_name = f"{self.KEYWORDS['comb_pdf_name']}_{channel}"
         channel_data_name = f"{self.KEYWORDS['comb_data_name']}_{channel}"
         channel_cat_name = f"{self.KEYWORDS['comb_cat_name']}_{channel}"
@@ -708,7 +707,7 @@ class XMLWSCombiner(XMLWSBase):
             channel_cat.setBin(i)
             cat_name_i = channel_cat.getLabel()
             new_cat_name_i = f"{channel_cat_name}_{cat_name_i}"
-            self.stdout.info(f"Category {i+1}: {cat_name_i} --> {new_cat_name_i}")
+            self.stdout.info(f"Category {i+1}: {cat_name_i} --> {new_cat_name_i}", bare=True)
             pdf_i = channel_pdf.getPdf(cat_name_i)
             data_i = channel_data_list.FindObject(cat_name_i)
             observables_i = pdf_i.getObservables(data_i)
@@ -728,7 +727,7 @@ class XMLWSCombiner(XMLWSBase):
     def combine(self, load_rename_ws:bool=True, save_combine_ws:bool=True) -> None:
         title = format_delimiter_enclosed_text(f"Create Combined Pdf and Dataset",
                                                delimiter="-", indent_str="")
-        self.stdout.info(title)
+        self.stdout.info(title, bare=True)
         if load_rename_ws:
             self.load_cache_ws("rename")
         
@@ -738,27 +737,27 @@ class XMLWSCombiner(XMLWSBase):
         
         pdf_name  = self.KEYWORDS['comb_pdf_name']
         data_name = self.combination_config["data_name"]
-        self.stdout.info(f'INFO: Generating combined pdf "{pdf_name}"')
+        self.stdout.info(f'Generating combined pdf "{pdf_name}"')
         self.comb_ws.Import(self.comb_pdf)
-        self.stdout.info(f'INFO: Generating combined dataset "{data_name}"')
+        self.stdout.info(f'Generating combined dataset "{data_name}"')
         self.comb_obs.add(self.comb_cat)
         obs_and_weight, weight_var = self._get_obs_and_weight(self.comb_obs)
-        dataset_map = ExtendedModel.get_dataset_map(self.comb_data_map)
+        dataset_map = RooDataSet.get_dataset_map(self.comb_data_map)
         comb_data = ROOT.RooDataSet(data_name, data_name, obs_and_weight,
                                     ROOT.RooFit.Index(self.comb_cat),
                                     ROOT.RooFit.Import(dataset_map),
                                     ROOT.RooFit.WeightVar(weight_var))
         self.comb_ws.Import(comb_data)
-        self.stdout.info(f'INFO: Defining variable set "{self.KEYWORDS["comb_nuis_name"]}"')
+        self.stdout.info(f'Defining variable set "{self.KEYWORDS["comb_nuis_name"]}"')
         self.comb_ws.defineSet(self.KEYWORDS["comb_nuis_name"], self.tmp_nuis, True)
-        self.stdout.info(f'INFO: Defining variable set "{self.KEYWORDS["comb_glob_name"]}"')
+        self.stdout.info(f'Defining variable set "{self.KEYWORDS["comb_glob_name"]}"')
         self.comb_ws.defineSet(self.KEYWORDS["comb_glob_name"], self.tmp_glob, True)
-        self.stdout.info(f'INFO: Defining variable set "{self.KEYWORDS["comb_obs_name"]}"')
+        self.stdout.info(f'Defining variable set "{self.KEYWORDS["comb_obs_name"]}"')
         self.comb_ws.defineSet(self.KEYWORDS["comb_obs_name"], self.comb_obs, True)
         self.define_argsets(self.comb_ws)    
         if save_combine_ws:
             filename = self._get_comb_ws_path()
-            self.stdout.info(f'INFO: Saving temporary workspace to "{filename}"')
+            self.stdout.info(f'Saving temporary workspace to "{filename}"')
             self.comb_ws.writeToFile(filename, True)
             
     def define_argsets(self, ws:ROOT.RooWorkspace):
@@ -770,7 +769,7 @@ class XMLWSCombiner(XMLWSBase):
                     raise RuntimeError(f'failed to locate object "{arg_name}" from the combined workspace when '
                                        f'defining the ArgSet "{argset_name}"')
                 argset.add(arg)
-            self.stdout.info(f'INFO: Defining argument set "{argset_name}"')
+            self.stdout.info(f'Defining argument set "{argset_name}"')
             ws.Import(argset, argset_name, True)
         
     def get_combined_model_config(self) -> None:
@@ -792,7 +791,7 @@ class XMLWSCombiner(XMLWSBase):
                         poi.setVal(poi_value)
                 pois.add(poi)
             else:
-                self.stdout.warning(f'WARNING: The POI "{poi_name}" does not exist in the combined model')
+                self.stdout.warning(f'The POI "{poi_name}" does not exist in the combined model')
         mc_name = self.combination_config["model_config_name"]
         model_config = ROOT.RooStats.ModelConfig(mc_name, self.comb_ws)
         #model_config.SetWorkspace(self.com_ws)
@@ -852,7 +851,7 @@ class XMLWSCombiner(XMLWSBase):
             self.load_cache_ws("combine")
         
         # create model config
-        self.stdout.info(f'INFO: Creating ModelConfig "{self.combination_config["model_config_name"]}"')
+        self.stdout.info(f'Creating ModelConfig "{self.combination_config["model_config_name"]}"')
         model_config = self.get_combined_model_config()
         self.comb_ws.Import(model_config)
         if import_class_code:
@@ -860,21 +859,21 @@ class XMLWSCombiner(XMLWSBase):
         
         title = format_delimiter_enclosed_text(f"Generate Asimov Dataset",
                                                delimiter="-", indent_str="")
-        self.stdout.info(title)
+        self.stdout.info(title, bare=True)
         
         # generate asimov dataset
         self.generate_asimov()
             
         title = format_delimiter_enclosed_text(f"Workspace Summary",
                                                delimiter="-", indent_str="")
-        self.stdout.info(title)
+        self.stdout.info(title, bare=True)
         
         mc_summary = self.get_model_config_summary()
-        self.stdout.info(mc_summary)
+        self.stdout.info(mc_summary, bare=True)
         
         if save_final_ws:
             filename = self.combination_config["output_file"]
-            self.stdout.info(f'INFO: Saving final combined workspace to "{filename}"')
+            self.stdout.info(f'Saving final combined workspace to "{filename}"')
             self.comb_ws.writeToFile(filename, True)
             
     def reset_combination(self):
@@ -904,4 +903,4 @@ class XMLWSCombiner(XMLWSBase):
             self.combine(load_rename_ws=save_rename_ws, save_combine_ws=save_combine_ws)
             self.finalize(load_combine_ws=save_combine_ws, save_final_ws=save_final_ws,
                           import_class_code=import_class_code)
-        self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
\ No newline at end of file
+        self.stdout.info(f"Total time taken: {t.interval:.3f}s")
\ No newline at end of file
