@@ -22,6 +22,7 @@ from quickstats.utils.common_utils import format_delimiter_enclosed_text, remove
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.basics import WSArgument
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, AsimovHandler
+from quickstats.interface.root import RooAbsPdf
 
 class ModifierAction(GeneralEnum):
     NORMAL     = 0
@@ -94,11 +95,12 @@ class XMLWSModifier(XMLWSBase):
             "dataset_list"       : source.pop("dataset_list", None),
             "fix_parameters"     : source.pop("fix_parameters", None),
             "profile_parameters" : source.pop("profile_parameters", None),
+            "set_parameters"     : source.pop("set_parameters", None),
             "strict"             : source.pop("strict", True)
         }
         if source:
             unknown_attributes = list(source)
-            self.stdout.warning(f"WARNING: The following unrecognized configuration attribute(s) will be ignored: "
+            self.stdout.warning(f"The following unrecognized configuration attribute(s) will be ignored: "
                                 f"{', '.join(unknown_attributes)}")
         actions = {
             "map"        : _actions.pop("map", []),
@@ -132,11 +134,11 @@ class XMLWSModifier(XMLWSBase):
         actions["rename"] = rename
         if _actions:
             unknown_attributes = list(_actions)
-            self.stdout.warning(f"WARNING: Unknown action type: {', '.join(unknown_attributes)}. "
+            self.stdout.warning(f"Unknown action type: {', '.join(unknown_attributes)}. "
                                 "Ignoring...")
         if _rename:
             unknown_attributes = list(_rename)
-            self.stdout.warning(f"WARNING: Unknown object type in rename: {', '.join(unknown_attributes)}. "
+            self.stdout.warning(f"Unknown object type in rename: {', '.join(unknown_attributes)}. "
                                 "Ignoring...")
         
         self.config = config
@@ -155,22 +157,23 @@ class XMLWSModifier(XMLWSBase):
     def parse_config_xml(self, filename:str):
         root = TXMLTree.load_as_dict(filename)
         config = {
-            'input_file'       : self._get_node_attrib(root, "InFile", required=True),
-            'output_file'      : self._get_node_attrib(root, "OutFile", required=True),
-            'model_name'       : self._get_node_attrib(root, "ModelName", required=False),
-            'workspace_name'   : self._get_node_attrib(root, "WorkspaceName", required=False),
-            'model_config_name': self._get_node_attrib(root, "ModelConfigName", required=False),
-            'data_name'        : self._get_node_attrib(root, "DataName", required=False, default="combData"),
-            'poi_names'        : self._get_node_attrib(root, "POINames", required=False, dtype="str_list"),
-            'snapshot_nuis'    : self._get_node_attrib(root, "SnapshotNP", required=False, default=[], dtype="str_list"),
-            'snapshot_globs'   : self._get_node_attrib(root, "SnapshotGO", required=False, default=[], dtype="str_list"),
-            'snapshot_pois'    : self._get_node_attrib(root, "SnapshotPOI", required=False, default=[], dtype="str_list"),
-            'snapshot_all'     : self._get_node_attrib(root, "SnapshotAll", required=False, default=[], dtype="str_list"),
-            'snapshot_list'    : self._get_node_attrib(root, "SnapshotList", required=False, default=[], dtype="str_list"),
-            'dataset_list'     : self._get_node_attrib(root, "DatasetList", required=False, default=None, dtype="str_list"),
+            'input_file'         : self._get_node_attrib(root, "InFile", required=True),
+            'output_file'        : self._get_node_attrib(root, "OutFile", required=True),
+            'model_name'         : self._get_node_attrib(root, "ModelName", required=False),
+            'workspace_name'     : self._get_node_attrib(root, "WorkspaceName", required=False),
+            'model_config_name'  : self._get_node_attrib(root, "ModelConfigName", required=False),
+            'data_name'          : self._get_node_attrib(root, "DataName", required=False, default="combData"),
+            'poi_names'          : self._get_node_attrib(root, "POINames", required=False, dtype="str_list"),
+            'snapshot_nuis'      : self._get_node_attrib(root, "SnapshotNP", required=False, default=[], dtype="str_list"),
+            'snapshot_globs'     : self._get_node_attrib(root, "SnapshotGO", required=False, default=[], dtype="str_list"),
+            'snapshot_pois'      : self._get_node_attrib(root, "SnapshotPOI", required=False, default=[], dtype="str_list"),
+            'snapshot_all'       : self._get_node_attrib(root, "SnapshotAll", required=False, default=[], dtype="str_list"),
+            'snapshot_list'      : self._get_node_attrib(root, "SnapshotList", required=False, default=[], dtype="str_list"),
+            'dataset_list'       : self._get_node_attrib(root, "DatasetList", required=False, default=None, dtype="str_list"),
             "fix_parameters"     : self._get_node_attrib(root, "FixParameters", required=False, default=None),
             "profile_parameters" : self._get_node_attrib(root, "ProfileParameters", required=False, default=None),
-            'strict'           : self._get_node_attrib(root, "Strict", required=False, default="true", dtype="bool")
+            "set_parameters"     : self._get_node_attrib(root, "SetParameters", required=False, default=None),
+            'strict'             : self._get_node_attrib(root, "Strict", required=False, default="true", dtype="bool")
         }
         # for compatibility with older version of workspaceCombiner
         if config['workspace_name'] == "dummy":
@@ -279,7 +282,7 @@ class XMLWSModifier(XMLWSBase):
                 outfile = self.config['output_file']
             ws_name  = self.config['workspace_name']
             mc_name  = self.config['model_config_name']
-            self.stdout.info(f"INFO: Begin modification of the workspace \"{infile}\".")
+            self.stdout.info(f"Begin modification of the workspace \"{infile}\".")
             model    = ExtendedModel(infile, ws_name=ws_name, mc_name=mc_name,
                                      data_name=None, verbosity="WARNING")
             ws_orig  = model.workspace
@@ -291,34 +294,34 @@ class XMLWSModifier(XMLWSBase):
             ws_tmp = ROOT.RooWorkspace(ws_name)
 
             title_str = format_delimiter_enclosed_text("Step 1: Redefine objects", "-")
-            self.stdout.info(title_str)
+            self.stdout.info(title_str, bare=True)
             flag = self.redefine_objects(ws_orig, ws_tmp)
             # nothing is done
             if not flag:
-                self.stdout.info("INFO: No objects are redefined.")
+                self.stdout.info("No objects are redefined.")
 
             title_str = format_delimiter_enclosed_text("Step 2: Create new objects", "-")
-            self.stdout.info(title_str)
+            self.stdout.info(title_str, bare=True)
             flag = self.implement_external_pdfs(ws_tmp)
             flag |= self.implement_objects(ws_tmp)
             # nothing is done
             if not flag:
-                self.stdout.info("INFO: No new objects are defined.")
+                self.stdout.info("No new objects are defined.")
             
             dataset_list = self.config['dataset_list']
             # need to import datasets first so the variables are also imported into the workspace
             self.import_datasets(ws_orig, ws_tmp, dataset_list)
 
             title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
-            self.stdout.info(title_str)
+            self.stdout.info(title_str, bare=True)
             rename_map = self.get_rename_map(ws_orig, ws_tmp)
             actual_rename_map = self.get_actual_rename_map(rename_map, ws_orig, ws_tmp)
             self.rename_variables(mc_orig, ws_tmp, rename_map)
             if not rename_map:
-                self.stdout.info("INFO: No variables are renamed.")
+                self.stdout.info("No variables are renamed.")
 
             title_str = format_delimiter_enclosed_text("Step 4: Making output workspace", "-")
-            self.stdout.info(title_str)
+            self.stdout.info(title_str, bare=True)
 
             # create final workspace
             if len(self.actions['constraint']) > 0:
@@ -353,8 +356,8 @@ class XMLWSModifier(XMLWSBase):
                 os.makedirs(outdir)
 
             ws_final.writeToFile(outfile, recreate)
-            self.stdout.info(f'INFO: Saved output workspace as "{outfile}".')
-        self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
+            self.stdout.info(f'Saved output workspace as "{outfile}".')
+        self.stdout.info(f"Total time taken: {t.interval:.3f}s")
     
     def post_process_workspace(self, ws:ROOT.RooWorkspace):
         self.add_product_terms(ws)
@@ -369,7 +372,7 @@ class XMLWSModifier(XMLWSBase):
             for extra_term in extra_terms:
                 member_var = self.get_workspace_arg(ws, extra_term, strict=True)
                 prod_var.addTerm(member_var)
-            self.stdout.info(f"INFO: The following product terms are added to the variable {target_name}: "
+            self.stdout.info(f"The following product terms are added to the variable {target_name}: "
                              f"{', '.join(extra_terms)}")
     
     def setup_parameters(self, ws:ROOT.RooWorkspace):
@@ -380,6 +383,8 @@ class XMLWSModifier(XMLWSBase):
             model.fix_parameters(self.config["fix_parameters"])
         if self.config["profile_parameters"] is not None:
             model.profile_parameters(self.config["profile_parameters"])
+        if self.config["set_parameters"] is not None:
+            model.set_parameters(self.config["set_parameters"])
 
     def generate_asimov(self, ws:ROOT.RooWorkspace):
         n_asimov = len(self.actions['asimov'])
@@ -411,7 +416,7 @@ class XMLWSModifier(XMLWSBase):
                 # note only renaming of RooRealVar instance will be considered
                 new_variable = new_ws.var(new_name)
                 if not new_variable:
-                    self.stdout.warning(f'WARNING: The argset variable "{name}" not found in the new workspace. Skipped.')
+                    self.stdout.warning(f'The argset variable "{name}" not found in the new workspace. Skipped.')
                 # avoid modifying the original value
                 copied_variable = self.copy_variable(variable, new_variable)
                 renamed_variables_old.add(variable)
@@ -425,7 +430,7 @@ class XMLWSModifier(XMLWSBase):
                                 rename_map:Optional[Dict]=None):
         snapshot = old_ws.getSnapshot(snapshot_name)
         if not snapshot:
-            self.stdout.warning(f"WARNING: Snapshot {snapshot_name} not found in the original workspace")
+            self.stdout.warning(f"Snapshot {snapshot_name} not found in the original workspace")
         # get variables from orginal workspace, make sure to make a clone
         if variable_type is None:
             variables = snapshot.Clone()
@@ -434,7 +439,7 @@ class XMLWSModifier(XMLWSBase):
             variables = snapshot.Clone().selectCommon(target_variables)
         if rename_map is None:
             rename_map = {}
-        self.stdout.info(f"INFO: Copying snapshot \"{snapshot_name}\" from the original workspace "
+        self.stdout.info(f"Copying snapshot \"{snapshot_name}\" from the original workspace "
                          "to the new workspace")
         self.copy_variables(variables, new_ws, rename_map)
         # important: the third argument decide whether to take values from the argset or from the workspace
@@ -496,10 +501,10 @@ class XMLWSModifier(XMLWSBase):
         for generic_object in generic_objects:
             name = generic_object.GetName()
             if not name:
-                self.stdout.warning('WARNING: Found generic object without name. '
+                self.stdout.warning('Found generic object without name. '
                                     'The object will not be imported to the new workspace')
                 continue
-            self.stdout.info(f'INFO: Copying generic object "{name}" from the original workspace to the new workspace"')
+            self.stdout.info(f'Copying generic object "{name}" from the original workspace to the new workspace"')
             if all(isinstance(obj, ROOT.RooRealVar) for obj in generic_object):
                 argset = generic_object.Clone()
                 self.copy_variables(argset, new_ws, rename_map)
@@ -556,10 +561,10 @@ class XMLWSModifier(XMLWSBase):
                 if strict:
                     raise RuntimeError(f"POI {poi_name} does not exist")
                 else:
-                    self.stdout.warning(f"WARNING: POI {poi_name} does not exist. Skipping.")
+                    self.stdout.warning(f"POI {poi_name} does not exist. Skipping.")
                     continue
             pois.add(poi)
-            self.stdout.info(f"INFO: Added POI \"{poi_name}\".")
+            self.stdout.info(f"Added POI \"{poi_name}\".")
         mc.SetParametersOfInterest(pois)
         
         # nuisance parameters and global observables from original workspace
@@ -589,13 +594,13 @@ class XMLWSModifier(XMLWSBase):
         nuis_names += extra_nuis
         
         if len(new_nuis) > 0:
-            self.stdout.info("INFO: The following newly defined nuisance parameter(s) will be added to ModelConfig")
+            self.stdout.info("The following newly defined nuisance parameter(s) will be added to ModelConfig")
             for nuis_name in new_nuis:
-                self.stdout.info(f"    {nuis_name}")
+                self.stdout.info(f"    {nuis_name}", bare=True)
         if len(extra_nuis) > 0:
-            self.stdout.info("INFO: The following additional nuisance parameter(s) will be added to ModelConfig")
+            self.stdout.info("The following additional nuisance parameter(s) will be added to ModelConfig")
             for nuis_name in extra_nuis:
-                self.stdout.info(f"    {nuis_name}")
+                self.stdout.info(f"    {nuis_name}", bare=True)
         
         nuis_names = remove_list_duplicates(nuis_names)
         # set nuisance parameters
@@ -603,11 +608,11 @@ class XMLWSModifier(XMLWSBase):
         for nuis_name in nuis_names:
             nuis = self._get_relevant_variable(nuis_name, new_ws, pdf)
             if not nuis:
-                self.stdout.warning(f"WARNING: The nuisance parameter {nuis_name} no longer exists in "
+                self.stdout.warning(f"The nuisance parameter {nuis_name} no longer exists in "
                                     "the new workspace. It will be removed from the new ModelConfig.")
                 continue
             if nuis.isConstant():
-                self.stdout.warning(f"WARNING: The nuisace parameter {nuis_name} is initialized as a constant. "
+                self.stdout.warning(f"The nuisace parameter {nuis_name} is initialized as a constant. "
                                     "It will be floated in the new workspace.")
                 nuis.setConstant(False)
             nuisance_parameters.add(nuis)
@@ -617,11 +622,11 @@ class XMLWSModifier(XMLWSBase):
         for glob_name in glob_names:
             glob = self._get_relevant_variable(glob_name, new_ws, pdf)
             if not glob:
-                self.stdout.warning(f"WARNING: The global observable {glob_name} no longer exists in "
+                self.stdout.warning(f"The global observable {glob_name} no longer exists in "
                                     "the new workspace. It will be removed from the new ModelConfig.")
                 continue
             if not glob.isConstant():
-                self.stdout.warning(f"WARNING: The global observable {glob_name} is initialized as a floating "
+                self.stdout.warning(f"The global observable {glob_name} is initialized as a floating "
                                     "parameter. It will be set to constant in the new workspace.")
 
                 glob.setConstant(True)
@@ -653,7 +658,7 @@ class XMLWSModifier(XMLWSBase):
                 if strict:
                     raise RuntimeError(f"missing variable {variable_name} in the workspace {ws.GetName()}")
                 else:
-                    self.stdout.warning(f"WARNING: No variable {variable_name} found in the workspace {ws.GetName()}")
+                    self.stdout.warning(f"No variable {variable_name} found in the workspace {ws.GetName()}")
                     continue
             extracted_variables.add(extracted_variable, True)
         return extracted_variables
@@ -665,27 +670,27 @@ class XMLWSModifier(XMLWSBase):
                                   ROOT.RooFit.RenameVariable(old_var_expr, new_var_expr),
                                   ROOT.RooFit.RecycleConflictNodes())
         if rename_map:
-            self.stdout.info("INFO: Renamed variables\n")
+            self.stdout.info("Renamed variables\n")
             rename_table = "\n".join(self.get_name_mapping_str_arrays(rename_map))
-            self.stdout.info(rename_table)
+            self.stdout.info(rename_table, bare=True)
         
     def rename_objects(self, ws:ROOT.RooWorkspace, mc:ROOT.RooStats.ModelConfig):
         if len(self.actions['rename']['workspace']) > 0:
-            self.stdout.info("INFO: Renamed workspace\n")
+            self.stdout.info("Renamed workspace\n")
             rename_map = {}
             for old_name, new_name in self.actions['rename']['workspace'].items():
                 rename_map[ws.GetName()] = new_name
                 ws.SetName(new_name)
             rename_table = "\n".join(self.get_name_mapping_str_arrays(rename_map))
-            self.stdout.info(rename_table)
+            self.stdout.info(rename_table, bare=True)
         if len(self.actions['rename']['model_config']) > 0:
-            self.stdout.info("INFO: Renamed model config\n")
+            self.stdout.info("Renamed model config\n")
             rename_map = {}
             for old_name, new_name in self.actions['rename']['model_config'].items():
                 rename_map[mc.GetName()] = new_name
                 mc.SetName(new_name)
             rename_table = "\n".join(self.get_name_mapping_str_arrays(rename_map))
-            self.stdout.info(rename_table)
+            self.stdout.info(rename_table, bare=True)
         if len(self.actions['rename']['dataset']) > 0:
             rename_map = {}            
             for old_name, new_name in self.actions['rename']['dataset'].items():
@@ -701,9 +706,9 @@ class XMLWSModifier(XMLWSBase):
                 rename_map[old_name] = new_name
                 dataset.SetName(new_name)
             if rename_map:
-                self.stdout.info("INFO: Renamed dataset\n")
+                self.stdout.info("Renamed dataset\n")
                 rename_table = "\n".join(self.get_name_mapping_str_arrays(rename_map))
-                self.stdout.info(rename_table)                
+                self.stdout.info(rename_table, bare=True)                
         
     def import_datasets(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace,
                         dataset_names:Optional[List[str]]=None):
@@ -718,19 +723,19 @@ class XMLWSModifier(XMLWSBase):
                 datasets.append(dataset)
         for dataset in datasets:
             getattr(new_ws, "import")(dataset)
-            self.stdout.info(f"INFO: Imported dataset \"{dataset.GetName()}\".")
+            self.stdout.info(f"Imported dataset \"{dataset.GetName()}\".")
             
     def redefine_objects(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
         flag = False
         for i, expr in enumerate(self.actions['redefine']):
-            self.stdout.info(f"INFO: (Item {i}) {expr}")
+            self.stdout.info(f"(Item {i}) {expr}")
             obj_name, obj_type = self._get_object_name_and_type_from_expr(expr)
             arg = old_ws.arg(obj_name)
             if not arg:
                 if self.config['strict']:
                     raise RuntimeError(f"object {obj_name} does not exist in the original workspace")
                 else:
-                    self.stdout.warning(f"WARNING: object {obj_name} does not exist in the original workspace")
+                    self.stdout.warning(f"object {obj_name} does not exist in the original workspace")
                     continue
             if obj_type == WSObjectType.VARIABLE:
                 result = re.search(r"\[(.+)\]", expr)
@@ -758,7 +763,7 @@ class XMLWSModifier(XMLWSBase):
     def implement_objects(self, ws:ROOT.RooWorkspace):
         flag = False
         for i, expr in enumerate(self.actions['define']):
-            self.stdout.info(f"INFO: (Item {i}) {expr}")
+            self.stdout.info(f"(Item {i}) {expr}")
             if "FlexibleInterpVar" in expr:
                 self.implement_flexible_interp_var(ws, expr)
             elif "RooMultiVarGaussian" in expr:
@@ -775,7 +780,7 @@ class XMLWSModifier(XMLWSBase):
             file = item['file']
             if file is not None:
                 self.load_external_pdf(file, pdf_name, ws)
-                self.stdout.info(f"INFO: Loaded external pdf {pdf_name} from {file}")
+                self.stdout.info(f"Loaded external pdf {pdf_name} from {file}")
                 flag = True
         return flag
                 
@@ -855,7 +860,7 @@ class XMLWSModifier(XMLWSBase):
         for i in range(n_cat):
             category.setBin(i)
             category_name = category.getLabel()
-            self.stdout.info(f"INFO: Creating new pdf for the category {category_name}")
+            self.stdout.info(f"Creating new pdf for the category {category_name}")
             pdf_i = pdf.getPdf(category_name)
             
             base_components = ROOT.RooArgSet()
@@ -881,7 +886,7 @@ class XMLWSModifier(XMLWSBase):
                     raise RuntimeError(f"pdf {pdf_name} does not exist in the new workspace")
                 if independent:
                     base_components.add(new_pdf)
-                    self.stdout.info(f"INFO: Adding independent constraint pdf \"{pdf_name}\" to "
+                    self.stdout.info(f"Adding independent constraint pdf \"{pdf_name}\" to "
                                      f"\"{new_pdf_name}\"")
                     continue
                 # check whether the current category depends on the constraint pdf
@@ -893,7 +898,7 @@ class XMLWSModifier(XMLWSBase):
                         
             
             pdf_map[category_name] = ROOT.RooProdPdf(new_pdf_name, new_pdf_name, base_components)
-        c_pdf_map = ExtendedModel.get_pdf_map(pdf_map)
+        c_pdf_map = RooAbsPdf.get_pdf_map(pdf_map)
         pdf_name = old_mc.GetPdf().GetName()
         sim_pdf = ROOT.RooSimultaneous(pdf_name, f"{pdf_name}__addConstr", c_pdf_map, category)
         return sim_pdf
@@ -903,11 +908,11 @@ class XMLWSModifier(XMLWSBase):
         var = ws.var(var_name)
         if not var:
             if warn:
-                self.stdout.warning(f"WARNING: Variable {var_name} does not exist in the new workspace")
+                self.stdout.warning(f"Variable {var_name} does not exist in the new workspace")
             return None
         if not pdf.dependsOn(var):
             if warn:
-                self.stdout.warning(f"WARNING: Variable {var_name} exists in the new workspace but is "
+                self.stdout.warning(f"Variable {var_name} exists in the new workspace but is "
                                     f"not part of the provided pdf {pdf.GetName()}")
             return None
         return var
@@ -940,8 +945,8 @@ class XMLWSModifier(XMLWSBase):
         function = ROOT.RooStats.HistFactory.FlexibleInterpVar(response_name, response_name, nuis_arglist,
                                                                nominal, sigma_var_low, sigma_var_high, code)
         self.import_object(ws, function)
-        self.stdout.info(f"INFO: Implemented FlexibleInterpVar \"{response_name}\"")
-        self.stdout.info(str(py_V))
+        self.stdout.info(f"Implemented FlexibleInterpVar \"{response_name}\"")
+        self.stdout.info(str(py_V), bare=True)
 
     
     def implement_multi_var_gaussian(self, ws:ROOT.RooWorkspace, expr:str):
@@ -994,5 +999,5 @@ class XMLWSModifier(XMLWSBase):
                     index += 1
         function = RooMultiVarGaussian(function_name, function_name, obs_arglist, mean_arglist, V)
         self.import_object(ws, function)
-        self.stdout.info(f"INFO: Implemented RooMultiVarGaussian \"{function_name}\" with correlation matrix")
-        self.stdout.info(str(py_V))
\ No newline at end of file
+        self.stdout.info(f"Implemented RooMultiVarGaussian \"{function_name}\" with correlation matrix")
+        self.stdout.info(str(py_V), bare=True)
\ No newline at end of file
