@@ -1,4 +1,6 @@
 from .abstract_runner import AbstractRunner
 from .parameterised_runner import ParameterisedRunner
 from .parameterised_asymptotic_cls import ParameterisedAsymptoticCLs
-from .parameterised_likelihood import ParameterisedLikelihood
\ No newline at end of file
+from .parameterised_likelihood import ParameterisedLikelihood
+from .parameterised_significance import ParameterisedSignificance
+from .nuisance_parameter_ranking_runner import NuisanceParameterRankingRunner
\ No newline at end of file
