@@ -1,9 +1,12 @@
 from typing import Optional, List, Dict, Union, Tuple
 import os
 
-from quickstats.concurrent import AbstractRunner
+import ROOT
+
+from quickstats import semistaticmethod
+from . import AbstractRunner
 from quickstats.parsers import ParamParser
-from quickstats.utils.common_utils import combine_dict
+from quickstats.utils.common_utils import combine_dict, batch_makedirs
 
 class ParameterisedRunner(AbstractRunner):
     
@@ -19,28 +22,64 @@ class ParameterisedRunner(AbstractRunner):
                  param_expr:Optional[str]=None,
                  filter_expr:Optional[str]=None,
                  exclude_expr:Optional[str]=None,
-                 parallel:int=-1, timed:bool=True,
+                 outdir:str="output", cachedir:str="cache",
                  save_log:bool=True, cache:bool=True,
-                 allow_none:bool=True,
+                 allow_none:bool=True, parallel:int=-1,
                  verbosity:Optional[Union[int, str]]="INFO"):
         
-        super().__init__(parallel=parallel, timed=timed, save_log=save_log,
+        super().__init__(parallel=parallel, save_log=save_log,
                          cache=cache, verbosity=verbosity)
+
+        self.attributes = {
+            'outdir'   : outdir,
+            'cachedir' : cachedir,
+            'verbosity': verbosity
+        }
+        
         self.filter_expr = filter_expr
         self.exclude_expr = exclude_expr
         self._parser = ParamParser(file_expr, param_expr, allow_none=allow_none)
         self._param_points = None
         self.param_ind_kwargs = {}
         self.param_dpd_kwargs = {}
+
+    def _prerun_batch(self):
+        outdir = self.attributes['outdir']
+        cache_dir = self.get_cache_dir()
+        batch_makedirs([outdir, cache_dir])
+
+    @semistaticmethod
+    def _cached_return(self, outname:str):
+        with open(outname, 'r') as f:
+            result = json.load(f)
+            processed_result = self._process_result(result)
+        return processed_result
+
+    @semistaticmethod
+    def _process_result(self, result:Dict):
+        return result
         
     def setup_parser(self, file_expr:Optional[str]=None,
                      param_expr:Optional[str]=None):
         self._parser.setup(file_expr, param_expr)
+
+    def get_cache_dir(self):
+        return os.path.join(self.attributes['outdir'], self.attributes['cachedir'])
         
     def get_param_points(self, input_path:str):
         param_points = self.parser.get_param_points(input_path, filter_expr=self.filter_expr,
                                                     exclude_expr=self.exclude_expr)
         return param_points
+
+    def get_internal_param_points(self):
+        param_points = self.parser.get_internal_param_points(filter_expr=self.filter_expr,
+                                                             exclude_expr=self.exclude_expr)
+        return param_points
+
+    def get_external_param_points(self, input_path:str):
+        param_points = self.parser.get_external_param_points(input_path, filter_expr=self.filter_expr,
+                                                             exclude_expr=self.exclude_expr)
+        return param_points
         
     def get_serialised_param_data(self, param_points:str, outdir:str="./", outname:str="{param_str}.json"):
         filenames = []
@@ -58,10 +97,9 @@ class ParameterisedRunner(AbstractRunner):
             parameter_list.append(parameters)
             outnames.append(outname)
         if len(outnames) != len(set(outnames)):
-            raise RuntimeError("output names are not distinct, please check your input. "
-                               "(this can be due to 1. multiple tasks have the same set of "
-                               "parameters or 2. multiple zero-parameter tasks with input files of the same"
-                               " basename)")
+            raise RuntimeError("output names are not distinct, please check your input. (this can be "
+                               "due to 1. multiple tasks have the same set of parameters or 2. "
+                               "multiple zero-parameter tasks with input files of the same basename)")
         serialised_param_data = {
             'filenames': filenames,
             'outnames': outnames,
@@ -89,7 +127,20 @@ class ParameterisedRunner(AbstractRunner):
     def run(self):
         kwarg_set, auxiliary_args = self.prepare_task_inputs()
         return self.run_batch(kwarg_set, auxiliary_args=auxiliary_args)
-    
+
     def _end_of_instance_cleanup(self):
-        import ROOT
-        ROOT.gROOT.CloseFiles()
\ No newline at end of file
+        ROOT.gROOT.CloseFiles()
+
+    @staticmethod
+    def join_param_setup(base_setup:Optional[str]=None, new_setup:Optional[str]=None):
+        components = []
+        for setup in [base_setup, new_setup]:
+            if not setup:
+                continue
+            if isinstance(setup, dict):
+                setup = ParamParser.val_encode_parameters(setup)
+            assert isinstance(setup, str)
+            components.append(setup)
+        if not components:
+            return None
+        return ",".join(components)
\ No newline at end of file
