@@ -369,12 +369,12 @@ class ConfigParser(AbstractObject):
                                  filter_labels=required_labels)
         if not text:
             if required_only:
-                self.stdout.critical('ERROR: No required components matches the requested scope. '
+                self.stdout.critical('No required components matches the requested scope. '
                                      'Try using "required_only=False" instead.')
             elif node_label:
-                self.stdout.critical(f'ERROR: Config node "{node_label}" does not exist.')
+                self.stdout.critical(f'Config node "{node_label}" does not exist.')
             else:
-                self.stdout.critical(f'ERROR: Config format is empty.')                
+                self.stdout.critical(f'Config format is empty.')                
         return text
     
     def explain(self, node_label:Optional[str]="",
@@ -411,7 +411,7 @@ class ConfigParser(AbstractObject):
         if isinstance(config, str):
             extension = os.path.splitext(config)[-1]
             if not self.disable_message:
-                self.stdout.info(f'INFO: Reading configuration file "{config}".')
+                self.stdout.info(f'Reading configuration file "{config}".')
             if extension == ".json":
                 import json
                 with open(config, "r") as file:
@@ -469,8 +469,8 @@ class ConfigParser(AbstractObject):
                     if component.required_strict:
                         config[key] = component.validate(MISSING)
                         if not self.disable_message:
-                            self.stdout.info(f'INFO: Auto-filled missing config component "{config_label}" with '
-                                         f'default value "{config[key]}".')
+                            self.stdout.info(f'Auto-filled missing config component "{config_label}" with '
+                                             f'default value "{config[key]}".')
             else:
                 if key == "*":
                     config_keys = extra_keys
