@@ -17,7 +17,7 @@ def compile_macros(macros:Optional[Union[str, List[str]]]=None):
     """
     custom_list = True
     if macros is None:
-        macros = get_all_macros()
+        macros = get_all_macros(required_only=True)
         custom_list = False
     elif isinstance(macros, str):
         from quickstats.utils.string_utils import split_str
@@ -25,17 +25,23 @@ def compile_macros(macros:Optional[Union[str, List[str]]]=None):
     from quickstats.utils.root_utils import compile_macro
     for macro in macros:
         if ((macro == "FlexibleInterpVarMkII") and (quickstats.root_version >= (6, 26, 0))) and (not custom_list):
-            quickstats._PRINT_.info("INFO: Skip compiling macro \"FlexibleInterpVarMkII\" which is "
+            quickstats.stdout.info("INFO: Skip compiling macro \"FlexibleInterpVarMkII\" which is "
                                     "deprecated since ROOT 6.26/00")
             continue
         compile_macro(macro)
         
-def get_all_macros():
+def get_all_macros(required_only:bool=False):
     """
     Get the list of macros names
     
     Note: Only macros ending in .cxx will be considered
     """
+    
+    if required_only:
+        macro_names = ['AsymptoticCLsTool', 'QuickStatsCore']
+        macro_names.extend(get_workspace_extensions())
+        return macro_names
+    
     macro_dir = quickstats.macro_path
     macro_subdirs = [path for path in glob.glob(os.path.join(macro_dir, "*")) if os.path.isdir(path)]
     macro_names = []
@@ -47,7 +53,7 @@ def get_all_macros():
     return macro_names
 
 def set_verbosity(verbosity:Union[str, int]="INFO"):
-    quickstats._PRINT_.verbosity = verbosity
+    quickstats.stdout.verbosity = verbosity
     
 def get_root_version():
     from quickstats.root_checker import ROOTChecker, ROOTVersion
@@ -92,12 +98,12 @@ def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True,
             if force or input(f'WARNING: The macro already exists in {dest_path}, overwrite? [Y/N]') == "Y":
                 shutil.rmtree(dest_path)
                 shutil.copytree(path, dest_path)
-                quickstats._PRINT_.info(f'INFO: Overwritten contents for the macro "{name}" from {path} to {dest_path}.')
+                quickstats.stdout.info(f'INFO: Overwritten contents for the macro "{name}" from {path} to {dest_path}.')
             else:
-                quickstats._PRINT_.info(f'INFO: Overwrite cancelled.')
+                quickstats.stdout.info(f'INFO: Overwrite cancelled.')
         else:
             shutil.copytree(path, dest_path)
-            quickstats._PRINT_.info(f'INFO: Copied contents for the macro "{name}" from {path} to {dest_path}.')
+            quickstats.stdout.info(f'INFO: Copied contents for the macro "{name}" from {path} to {dest_path}.')
     if workspace_extension:
         resource_path = quickstats.resource_path
         extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
@@ -106,27 +112,27 @@ def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True,
             extension_config['required'].append(name)
             with open(extension_config_file, "w") as file:
                 json.dump(extension_config, file, indent=2)
-        quickstats._PRINT_.info(f'INFO: The macro "{name}" has been added to the workspace extension list.')
+        quickstats.stdout.info(f'INFO: The macro "{name}" has been added to the workspace extension list.')
         
 def remove_macro(name:str, remove_files:bool=True, force:bool=False):
     resource_path = quickstats.resource_path
     extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
     extension_config = get_workspace_extension_config()
     if (name not in extension_config):
-        quickstats._PRINT_.info(f'WARNING: Extension "{name}" not found in the workspace extension list. Skipped.')
+        quickstats.stdout.info(f'WARNING: Extension "{name}" not found in the workspace extension list. Skipped.')
     else:
         extension_config.remove(name)
         with open(extension_config_file, "w") as file:
             json.dump(extension_config, file, indent=2)
-        quickstats._PRINT_.info(f'INFO: The extension "{name}" has been removed from the workspace extension list.')
+        quickstats.stdout.info(f'INFO: The extension "{name}" has been removed from the workspace extension list.')
     if remove_files:
         macro_path = quickstats.macro_path
         extension_path = os.path.abspath(os.path.join(macro_path, name))
         if force or input(f'WARNING: Attempting to remove files from {extension_path}, confirm? [Y/N]') == "Y":
-            quickstats._PRINT_.info(f'INFO: Files for the macro "{name}" has been removed from {extension_path}.')
+            quickstats.stdout.info(f'INFO: Files for the macro "{name}" has been removed from {extension_path}.')
             shutil.rmtree(extension_path)
         else:
-            quickstats._PRINT_.info(f'INFO: Macro files removal cancelled.')
+            quickstats.stdout.info(f'INFO: Macro files removal cancelled.')
 
 def load_corelib():
     if not quickstats.corelib_loaded:
