@@ -1,6 +1,11 @@
 from typing import Dict, Union, List, Optional
+import copy
 
+from quickstats import semistaticmethod
 from quickstats.maths.numerics import get_proper_ranges
+from quickstats.interface.root import TArrayData
+
+import cppyy
 
 class RooRealVar:
     
@@ -43,9 +48,11 @@ class RooRealVar:
             self.named_ranges = obj.get("named_ranges", None)
             self.n_bins = obj.get("n_bins", None)
             self.unit = obj.get("unit", None)
+        elif isinstance(obj, RooRealVar):
+            self.__dict__ = copy.deepcopy(obj.__dict__)
         else:
-            import ROOT
-            if not isinstance(obj, ROOT.RooRealVar):
+            isvalid = hasattr(obj, 'ClassName') and (obj.ClassName() == 'RooRealVar')
+            if not isvalid:
                 raise ValueError("object must be an instance of ROOT.RooRealVar")
             self.name   = str(obj.GetName())
             self.title  = str(obj.getTitle())
@@ -56,7 +63,7 @@ class RooRealVar:
             self.range = [_range[0], _range[1]]
             named_ranges = {}
             for name in obj.getBinningNames():
-                if str(name) == "":
+                if str(name) in ["", "cache"]:
                     continue
                 _range = obj.getRange(name)
                 named_ranges[name] = [_range[0], _range[1]]
@@ -131,5 +138,42 @@ class RooRealVar:
             
         if self.unit is not None:
             variable.setUnit(self.unit)
-            
-        return variable
\ No newline at end of file
+        ROOT.SetOwnership(variable, False)
+        return variable
+
+    @semistaticmethod
+    def get_bin_widths(self, obj:"ROOT.RooRealVar"):
+        import ROOT
+        c_vector = ROOT.RFUtils.GetRooRealVarBinWidths(obj)
+        return TArrayData.vec_to_array(c_vector)
+    
+    @semistaticmethod
+    def get_binning(self, obj:"ROOT.RooRealVar"):
+        binning = obj.getBinning()
+        boundaries = binning.array()
+        num_boundaries = binning.numBoundaries()
+        from quickstats.interface.cppyy.vectorize import c_array_to_np_array
+        from quickstats.maths.statistics import bin_edge_to_bin_center, bin_edge_to_bin_width
+        bin_low_edge = c_array_to_np_array(boundaries, num_boundaries)
+        result = {
+            'bin_low_edge': bin_low_edge,
+            'bin_center': bin_edge_to_bin_center(bin_low_edge),
+            'bin_width': bin_edge_to_bin_width(bin_low_edge)
+        }
+        return result
+    
+    @staticmethod
+    def at_boundary(obj:"ROOT.RooRealVar"):
+        return cppyy.gbl.RFUtils.ParameterAtBoundary(obj)
+    
+    @staticmethod
+    def close_to_min(obj:"ROOT.RooRealVar", threshold:float=0.1):
+        return cppyy.gbl.RFUtils.ParameterCloseToMin(obj, threshold)
+
+    @staticmethod
+    def close_to_min(obj:"ROOT.RooRealVar", threshold:float=0.1):
+        return cppyy.gbl.RFUtils.ParameterCloseToMax(obj, threshold)
+    
+    @staticmethod
+    def close_to_boundary(obj:"ROOT.RooRealVar", threshold:float=0.1):
+        return cppyy.gbl.RFUtils.ParameterCloseToBoundary(obj, threshold)
\ No newline at end of file
