@@ -1,12 +1,20 @@
 import array
 import numpy as np
 
+from quickstats import semistaticmethod
 from quickstats.interface.root import TObject, TArrayData
 from quickstats.interface.cppyy.vectorize import c_array_to_np_array
 
 class TH2(TObject):
     
+    DTYPE_MAP = {
+        "TH2I": "int",
+        "TH2F": "float",
+        "TH2D": "double"
+    }
+    
     def __init__(self, h:"ROOT.TH2", underflow_bin:bool=False, overflow_bin:bool=False):
+        import ROOT
         dtype_map = {
             ROOT.TH2I: "int",
             ROOT.TH2F: "float",
@@ -19,12 +27,12 @@ class TH2(TObject):
         
     def get_fundamental_type(self):
         import ROOT
-        raise ROOT.TH2
+        return ROOT.TH2
         
     def init(self, h):
         self.bin_content    = self.GetBinContentArray(h, self.dtype, self.underflow_bin, self.overflow_bin)
-        self.x_labels       = self.GetXLabelArray(h)
-        self.y_labels       = self.GetYLabelArray(h)
+        #self.x_labels       = self.GetXLabelArray(h)
+        #self.y_labels       = self.GetYLabelArray(h)
         self.x_bin_center   = self.GetXBinCenterArray(h, self.dtype, self.underflow_bin, self.overflow_bin)
         self.y_bin_center   = self.GetYBinCenterArray(h, self.dtype, self.underflow_bin, self.overflow_bin)
         self.x_bin_width    = self.GetXBinWidthArray(h, self.dtype, self.underflow_bin, self.overflow_bin)
@@ -39,7 +47,8 @@ class TH2(TObject):
         n_bins_y = h.GetNbinsY()
         # account for underflow and overflow bins
         size = (n_bins_x + 2) * (n_bins_y + 2)
-        np_arr = c_array_to_np_array(arr, size=size, shape=(n_bins_x + 2, n_bins_y + 2))
+        np_arr = c_array_to_np_array(arr, size=size)
+        np_arr = np_arr.reshape((n_bins_x + 2, n_bins_y + 2), order='F')
         x_start = 1 if not underflow_bin else 0
         y_start = x_start
         x_end = -1 if not overflow_bin else n_bins_x + 2
@@ -97,12 +106,12 @@ class TH2(TObject):
     def GetYBinLowEdgeArray(h:"ROOT.TH2", dtype:str='double', underflow_bin:int=0, overflow_bin:int=0):
         return TH2.GetAxisBinLowEdgeArray(h.GetYaxis(), dtype, underflow_bin, overflow_bin)
     
-    @staticmethod
-    def correlationHist_to_dataframe(h:"ROOT.TH2"):
-        dtype = TH2.DTYPE_MAP.get(type(h), "double")
-        data = TH2.GetBinContentArray(h, dtype)
-        xlabels = TH2.GetXLabelArray(h)
-        ylabels = TH2.GetYLabelArray(h)
+    @semistaticmethod
+    def correlationHist_to_dataframe(self, h:"ROOT.TH2"):
+        dtype = self.DTYPE_MAP.get(h.ClassName(), "double")
+        data = self.GetBinContentArray(h, dtype)
+        xlabels = self.GetXLabelArray(h)
+        ylabels = self.GetYLabelArray(h)
         if not np.array_equal(xlabels, ylabels):
             raise RuntimeError("invalid correlation histogram: labels along x-axis and y-axis do not match")
         if (data.shape[0] != len(xlabels)) or (data.shape[1] != len(ylabels)):
@@ -113,4 +122,5 @@ class TH2(TObject):
         import pandas as pd
         df = pd.DataFrame(_data)
         df.index = ylabels[::-1]
+        df = df.loc[df.index[::-1]]
         return df
\ No newline at end of file
