@@ -6,6 +6,7 @@ import glob
 import fnmatch
 import itertools
 from functools import partial
+from collections import ChainMap
 
 import numpy as np
 
@@ -51,7 +52,7 @@ class MatchMode(DescriptiveEnum):
 
 class ParamParser:
     
-    DEFAULT_FORMAT_STR = r"[\w-]+"
+    DEFAULT_FORMAT_STR = r"[\.\w-]+"
     
     DEFAULT_FILE_EXT = ".root"
     
@@ -65,7 +66,7 @@ class ParamParser:
             self._format_str = self.DEFAULT_FORMAT_STR
         else:
             self._format_str = val
-        self.fname_regex   = self.get_fname_regex(self._format_str, self.file_ext)
+        self.filename_regex   = self.get_filename_regex(self._format_str, self.file_ext)
         self.attribute_parser = self.get_attribute_parser(self._format_str)
         
     @property
@@ -116,7 +117,7 @@ class ParamParser:
         return signature_map
     
     @staticmethod
-    def get_fname_regex(format_str:str, ext:Optional[str]=None):
+    def get_filename_regex(format_str:str, ext:Optional[str]=None):
         if ext is None:
             ext = ParamParser.DEFAULT_FILE_EXT    
         expr = format_str
@@ -132,7 +133,9 @@ class ParamParser:
         return regex
    
     @staticmethod
-    def sort_param_points(param_points:List, attributes:List):
+    def sort_param_points(param_points:List, attributes:Optional[List]=None):
+        if attributes is None:
+            attributes = list(param_points[0]['parameters'])
         key = lambda d: tuple(d['parameters'][attrib] for attrib in attributes)
         return sorted(param_points, key=key)
     
@@ -152,14 +155,16 @@ class ParamParser:
         if param_str is None:
             return []
         attributes = []
-        param_expr_list = split_str(param_str, sep=',', remove_empty=True)
-        for expr in param_expr_list:
-            tokens = split_str(expr, sep='=')
-            if len(tokens) not in [1, 2]:
-                raise ValueError('invalid expression for parameterisation')
-            attribute = tokens[0]
-            if attribute not in attributes:
-                attributes.append(attribute)
+        idp_param_strs = split_str(param_str, sep=';', remove_empty=True)
+        for idp_param_str in idp_param_strs:
+            param_expr_list = split_str(idp_param_str, sep=',', remove_empty=True)
+            for expr in param_expr_list:
+                tokens = split_str(expr, sep='=')
+                if len(tokens) not in [1, 2]:
+                    raise ValueError('invalid expression for parameterisation')
+                attribute = tokens[0]
+                if attribute not in attributes:
+                    attributes.append(attribute)
         return attributes
     
     @semistaticmethod
@@ -174,12 +179,14 @@ class ParamParser:
         if param_str is None:
             return {}
         param_points = []
-        idp_param_strs = split_str(param_str, sep=';', remove_empty=True)
-        for idp_param_str in idp_param_strs:
+        # components separated by ; are subject to independent conditions
+        ind_components = split_str(param_str, sep=';', remove_empty=True)
+        all_combinations = []
+        for component in ind_components:
             param_values = {}
-            param_expr_list = split_str(idp_param_str, sep=',', remove_empty=True)
+            param_expr_list = split_str(component, sep=',', remove_empty=True, use_paranthesis=True)
             for expr in param_expr_list:
-                tokens = expr.split('=')
+                tokens = split_str(expr, sep='=')
                 # floating poi
                 if len(tokens) == 1:
                     param_name = tokens[0]
@@ -190,26 +197,31 @@ class ParamParser:
                     continue
                 if len(tokens) != 2:
                     raise ValueError('invalid expression for parameterisation')
-                param_name = tokens[0]
-                values_expr = tokens[1]
-                tokens = split_str(values_expr, sep='_')
-                # fixed value
-                if len(tokens) == 1:
-                    values = [float(tokens[0])]
-                # scan across range
-                elif len(tokens) == 3:
-                    poi_min = float(tokens[0])
-                    poi_max = float(tokens[1])
-                    poi_step = float(tokens[2])
-                    values = np.arange(poi_min, poi_max + poi_step, poi_step)
-                else:
-                    raise ValueError('invalid expression for parameterisation')
+                param_name, values_expr = tokens
                 if param_name not in param_values:
                     param_values[param_name] = np.array([])
                 if None in param_values[param_name]:
                     raise RuntimeError(f"the parameter {param_name} is being profiled and non-profiled at the same time "
                                        f"in the parameter expression: {idp_param_str}")
-                param_values[param_name] = np.concatenate([param_values[param_name], values])
+                # grouped expression
+                if values_expr.startswith('(') and values_expr.endswith(')'):
+                    values_expr_list = split_str(values_expr[1:-1], sep=',')
+                else:
+                    values_expr_list = [values_expr]
+                for expr in values_expr_list:
+                    tokens = split_str(expr, sep='_')
+                    # fixed value
+                    if len(tokens) == 1:
+                        values = [float(tokens[0])]
+                    # scan across range
+                    elif len(tokens) == 3:
+                        poi_min = float(tokens[0])
+                        poi_max = float(tokens[1])
+                        poi_step = float(tokens[2])
+                        values = np.arange(poi_min, poi_max + poi_step, poi_step)
+                    else:
+                        raise ValueError('invalid expression for parameterisation')
+                    param_values[param_name] = np.concatenate([param_values[param_name], values])
             param_names = list(param_values.keys())
             # convert to numpy arrays
             combinations = [np.array(param_values[param_name]) for param_name in param_names]
@@ -218,10 +230,12 @@ class ParamParser:
             # take care of duplicate ploints
             combinations = [np.unique(arr) for arr in combinations]
             combinations = itertools.product(*combinations)
-
-            for combination in combinations:
-                param_point = {k:v for k,v in zip(param_names, combination)}
-                param_points.append(param_point)
+            all_combinations.extend(list(combinations))
+        import pandas as pd
+        all_combinations = pd.unique(all_combinations)
+        for combination in all_combinations:
+            param_point = {k:v for k,v in zip(param_names, combination)}
+            param_points.append(param_point)
         return param_points
     
     @staticmethod
@@ -249,31 +263,34 @@ class ParamParser:
                                   filter_expr:Optional[str]=None,
                                   exclude_expr:Optional[str]=None):
         if os.path.isdir(dirname):
-            fnames = glob.glob(os.path.join(dirname, '*'))
+            filenames = glob.glob(os.path.join(dirname, '*'))
         else:
-            fnames = glob.glob(dirname)
+            filenames = glob.glob(dirname)
         param_points = []
-        for fname in fnames:
-            basename = os.path.basename(fname)
-            match = self.fname_regex.match(basename)
+        for filename in filenames:
+            basename = os.path.basename(filename)
+            match = self.filename_regex.match(basename)
             if not match:
                 continue
-            point = {'filename': fname, 'parameters':{}}
-            point['basename'] = basename.split('.')[0]
+            point = {
+                'filename'   : filename,
+                'basename'   : os.path.splitext(basename)[0],
+                'parameters' : {}
+            }
             for key, value in match.groupdict().items():
                 parser = self.attribute_parser[key]
                 point['parameters'][key] = parser(value)
             param_points.append(point)
         attributes = list(self.attribute_parser)
         param_points = self.sort_param_points(param_points, attributes)
-        selected_param_points = self.select_param_points(param_points, filter_expr,
-                                                         exclude_expr, dict_key="parameters")
+        selected_param_points = self.get_selected_points(param_points, filter_expr,
+                                                         exclude_expr, dict_keys="parameters")
         return selected_param_points
     
     def get_internal_param_points(self, filter_expr:Optional[str]=None,
                                   exclude_expr:Optional[str]=None):
         param_points = self.parse_param_str(self.param_str)
-        selected_param_points = self.select_param_points(param_points, filter_expr,
+        selected_param_points = self.get_selected_points(param_points, filter_expr,
                                                          exclude_expr)
         return selected_param_points
     
@@ -283,66 +300,65 @@ class ParamParser:
             return []
         expr = remove_whitespace(expr)
         match_conditions = []
-        tokens = split_str(expr, sep=';', remove_empty=True)
-        for token in tokens:
+        # components separated by ; are subject to independent conditions
+        ind_components = split_str(expr, sep=';', remove_empty=True)
+        for component in ind_components:
+            conditions = split_str(component, sep=',', remove_empty=True, use_paranthesis=True)
             match_condition = {}
-            param_conditions = split_str(token, sep=',', remove_empty=True)
-            for param_condition in param_conditions:
-                subtokens = split_str(param_condition, sep='=')
-                if len(subtokens) != 2:
-                    raise RuntimeError(f"invalid parameter condition `{param_condition}` in the "
-                                       f"filter/exclude expression `{expr}`")
-                param_name, condition = subtokens
-                if "*" in condition:
-                    match_condition[param_name] = partial(MatchMode.Wildcard.match_func, y=condition)
+            for condition in conditions:
+                tokens = split_str(condition, sep='=')
+                if len(tokens) != 2:
+                    raise ValueError(f'invalid sub-expression: {condition}')
+                param_name, values_expr = tokens
+                if param_name not in match_condition:
+                    match_condition[param_name] = []
+                if values_expr.startswith('(') and values_expr.endswith(')'):
+                    values_expr_list = split_str(values_expr[1:-1], sep=',')
                 else:
-                    match_condition[param_name] = partial(MatchMode.Numerical.match_func, y=condition)
+                    values_expr_list = [values_expr]
+                for values_expr in values_expr_list:
+                    if "*" in values_expr:
+                        match_func = MatchMode.Wildcard.match_func
+                    else:
+                        match_func = MatchMode.Numerical.match_func
+                    match_condition[param_name].append(partial(match_func, y=values_expr))
             match_conditions.append(match_condition)
         return match_conditions
-    
-    @staticmethod
-    def select_param_point(parameters, conditions:List[Dict], default:bool=True):
-        selected = not default
-        for condition in conditions:
-            for param_name, match_func in condition.items():
-                if param_name not in parameters:
-                    continue
-                param_value = parameters[param_name]
-                if not match_func(param_value):
-                    return not default
-        return default
-    
-    @staticmethod
-    def select_param_point(parameters, conditions:List[Dict], default:bool=True):
-        for condition in conditions:
-            for param_name, match_func in condition.items():
-                # consider not matched if any of the param-specific condition not satisfied
-                if (param_name not in parameters) or (not match_func(parameters[param_name])):
-                    break
-            else:
-                return default
-        return not default
-    
-    def select_param_points(self, param_points:List,
+
+    @semistaticmethod
+    def get_selected_points(self, param_points:List,
                             filter_expr:Optional[str]=None,
                             exclude_expr:Optional[str]=None,
-                            dict_key:Optional=None):
-        selected_param_points = []
+                            dict_keys:Optional=None):
         filter_conditions = self.parse_filter_expr(filter_expr)
         exclude_conditions = self.parse_filter_expr(exclude_expr)
-        for param_point in param_points:
-            if dict_key:
-                parameters = param_point[dict_key]
-            else:
-                parameters = param_point               
-            keep = True
-            if filter_conditions:
-                keep = self.select_param_point(parameters, filter_conditions, default=True)
-            if exclude_conditions:
-                keep &= not (self.select_param_point(parameters, exclude_conditions, default=False))
-            if keep:
-                selected_param_points.append(param_point)
-        return selected_param_points
+        def get_attrib(x):
+            if dict_keys is None:
+                return x
+            if isinstance(dict_keys, str):
+                return x[dict_keys]
+            if isinstance(dict_keys, list):
+                return dict(ChainMap(*[x[k] for k in dict_keys]))
+            raise ValueError('invalid value for "dict_keys"')
+        selected_points = param_points
+        if filter_expr is not None:
+            selected_points = [point for point in selected_points \
+                               if self.is_point_selected(get_attrib(point), filter_conditions)]
+        if exclude_expr is not None:
+            selected_points = [point for point in selected_points \
+                               if not self.is_point_selected(get_attrib(point), exclude_conditions)]
+        return selected_points
+                    
+    @staticmethod
+    def is_point_selected(param_point, conditions):
+        for condition in conditions:
+            selected = True
+            for param_name, match_functions in condition.items():
+                param_value = param_point[param_name]
+                selected &= any(match_func(param_value) for match_func in match_functions)
+            if selected:
+                return True
+        return False
     
     def sanity_check(self, internal_param_points:Dict, external_param_points:Dict):
         if not self.allow_none:
@@ -354,21 +370,23 @@ class ParamParser:
     def get_param_points(self, dirname:str="",
                          filter_expr:Optional[str]=None,
                          exclude_expr:Optional[str]=None):
-        external_param_points = self.get_external_param_points(dirname, filter_expr, exclude_expr)     
-        internal_param_points = self.get_internal_param_points(filter_expr, exclude_expr)
+        external_param_points = self.get_external_param_points(dirname)
+        internal_param_points = self.get_internal_param_points()
         self.sanity_check(internal_param_points, external_param_points)
         if len(internal_param_points) == 0:
             internal_param_points = [{}]
         param_points = []
         for ext_point in external_param_points:
-            fname = ext_point['filename']
+            filename = ext_point['filename']
             basename = ext_point['basename']
             ext_params = ext_point['parameters']
             for int_params in internal_param_points:
                 param_point = {}
-                param_point['filename'] = fname
+                param_point['filename'] = filename
                 param_point['basename'] = basename
                 param_point['external_parameters'] = {**ext_params}
                 param_point['internal_parameters'] = {**int_params}
                 param_points.append(param_point)
+        param_points = self.get_selected_points(param_points, filter_expr, exclude_expr,
+                                                dict_keys=['internal_parameters', 'external_parameters'])
         return param_points
\ No newline at end of file
