@@ -14,6 +14,7 @@ from quickstats import GeneralEnum
 class ErrorDisplayFormat(GeneralEnum):
     ERRORBAR = 0
     FILL     = 1
+    SHADE    = 2
 
 class PdfDistributionPlot(AbstractPlot):
     
@@ -50,9 +51,7 @@ class PdfDistributionPlot(AbstractPlot):
     }
     
     def __init__(self, collective_data:Dict,
-                 blind_range:Optional[List[float]]=None,
                  plot_options:Optional[Dict]=None,
-                 scale_map:Optional[Dict]=None,
                  label_map:Optional[Dict]=None,
                  color_cycle:Optional[Dict]=None,
                  styles:Optional[Union[Dict, str]]=None,
@@ -73,17 +72,12 @@ class PdfDistributionPlot(AbstractPlot):
             self.label_map = {}
         else:
             self.label_map = label_map
-        if scale_map is None:
-            self.scale_map = {}
-        else:
-            self.scale_map = scale_map
         self.colors = {}
         
         self.annotation = None
         
     def set_data(self, collective_data:Dict):
         self.collective_data = collective_data
-        self.legend_order = list(collective_data.keys())
         for name in self.collective_data:
             if name not in self.error_options:
                 self.set_error_options(name)
@@ -132,77 +126,96 @@ class PdfDistributionPlot(AbstractPlot):
                          plot_options:Optional[Dict]=None, show_error:bool=True,
                          error_format:Union[ErrorDisplayFormat, str]="error_bar",
                          error_styles:Optional[Dict]=None,
-                         scale_by:Optional[float]=None):
-        pdata = self.process_data(data, blind_range=blind_range, scale_by=scale_by)
+                         plot_type:str="errorbar"):
+        pdata = self.process_data(data, blind_range=blind_range)
         has_errorbar = not ((pdata['xerr'] is None) and (pdata['yerr'] is None))
         draw_blind = blind_range is not None
         if not has_errorbar:
-            combined_options = combine_dict(self.styles['plot'], plot_options)
-            combined_options['label'] = label
-            if draw_blind:
-                # draw sideband low
-                handle_sblo = ax.plot(pdata['x'][pdata['sideband_lo']], 
-                                      pdata['y'][pdata['sideband_lo']],
-                                      **combined_options)
-                # avoid going through internal color cycle for the three regions
-                combined_options['color'] = handle_sblo[0].get_color()
-                # draw sideband high
-                handle_sbhi = ax.plot(pdata['x'][pdata['sideband_hi']], 
-                                      pdata['y'][pdata['sideband_hi']],
-                                      **combined_options)
-                combined_options['linestyle'] = self.config['blind_linestyle']
-                if label_blind is None:
-                    combined_options['label'] = f"{label} (blind)"
+            if plot_type == "errorbar":
+                combined_options = combine_dict(self.styles['plot'], plot_options)
+                combined_options['label'] = label
+                if draw_blind:
+                    # draw sideband low
+                    handle_sblo = ax.plot(pdata['x'][pdata['sideband_lo']], 
+                                          pdata['y'][pdata['sideband_lo']],
+                                          **combined_options)
+                    # avoid going through internal color cycle for the three regions
+                    combined_options['color'] = handle_sblo[0].get_color()
+                    # draw sideband high
+                    handle_sbhi = ax.plot(pdata['x'][pdata['sideband_hi']], 
+                                          pdata['y'][pdata['sideband_hi']],
+                                          **combined_options)
+                    combined_options['linestyle'] = self.config['blind_linestyle']
+                    if label_blind is None:
+                        combined_options['label'] = f"{label} (blind)"
+                    else:
+                        combined_options['label'] = label_blind
+                    x_blind_plus_sideband_edge = [pdata['x'][pdata['sideband_lo']][-1]] + \
+                                                 list(pdata['x'][pdata['blind']]) + \
+                                                 [pdata['x'][pdata['sideband_hi']][0]]
+                    y_blind_plus_sideband_edge = [pdata['y'][pdata['sideband_lo']][-1]] + \
+                                                 list(pdata['y'][pdata['blind']]) + \
+                                                 [pdata['y'][pdata['sideband_hi']][0]]                
+                    handle_blind = ax.plot(x_blind_plus_sideband_edge, 
+                                           y_blind_plus_sideband_edge,
+                                           **combined_options)
+                    handle = [handle_sblo[0], handle_blind[0]]
                 else:
-                    combined_options['label'] = label_blind
-                x_blind_plus_sideband_edge = [pdata['x'][pdata['sideband_lo']][-1]] + \
-                                             list(pdata['x'][pdata['blind']]) + \
-                                             [pdata['x'][pdata['sideband_hi']][0]]
-                y_blind_plus_sideband_edge = [pdata['y'][pdata['sideband_lo']][-1]] + \
-                                             list(pdata['y'][pdata['blind']]) + \
-                                             [pdata['y'][pdata['sideband_hi']][0]]                
-                handle_blind = ax.plot(x_blind_plus_sideband_edge, 
-                                       y_blind_plus_sideband_edge,
-                                       **combined_options)
-                handle = [handle_sblo[0], handle_blind[0]]
-            else:
-                handle = ax.plot(pdata['x'], pdata['y'], **combined_options)
-                handle = handle[0]
+                    handle = ax.plot(pdata['x'], pdata['y'], **combined_options)
+                    handle = handle[0]
+            elif plot_tye == "hist":
+                pass
         else:
-            
-            combined_options = combine_dict(plot_options, {"label": label})
-            if combined_options.get('markersize', None) is None:
-                nbins = len(pdata['x'])
-                combined_options['markersize'] = suggest_markersize(nbins)
-                
-            if draw_blind:
-                x = pdata['x'][pdata['sideband']]
-                y = pdata['y'][pdata['sideband']]
-                xerr = self._get_selected_err(pdata['xerr'], pdata['sideband'])
-                yerr = self._get_selected_err(pdata['yerr'], pdata['sideband'])
-            else:
+            if plot_type == "errorbar":
+                combined_options = combine_dict(plot_options, {"label": label})
+                if combined_options.get('markersize', None) is None:
+                    nbins = len(pdata['x'])
+                    combined_options['markersize'] = suggest_markersize(nbins)
+
+                if draw_blind:
+                    x = pdata['x'][pdata['sideband']]
+                    y = pdata['y'][pdata['sideband']]
+                    xerr = self._get_selected_err(pdata['xerr'], pdata['sideband'])
+                    yerr = self._get_selected_err(pdata['yerr'], pdata['sideband'])
+                else:
+                    x = pdata['x']
+                    y = pdata['y']
+                    xerr = pdata['xerr']
+                    yerr = pdata['yerr']
+                if not show_error:
+                    error_format = None
+                y_nonzero = np.where(y != 0)
+                x, y = x[y_nonzero], y[y_nonzero]
+                if (xerr is not None):
+                    if isinstance(xerr, tuple):
+                        xerr = (xerr[0][y_nonzero], xerr[1][y_nonzero])
+                    else:
+                        xerr = xerr[y_nonzero]
+                if (yerr is not None):
+                    if isinstance(yerr, tuple):
+                        yerr = (yerr[0][y_nonzero], yerr[1][y_nonzero])
+                    else:
+                        yerr = yerr[y_nonzero]
+                handle = self._draw_data(ax, x, y, xerr, yerr,
+                                         plot_options=combined_options,
+                                         error_format=error_format,
+                                         error_styles=error_styles)
+            elif plot_type == "hist":
                 x = pdata['x']
                 y = pdata['y']
                 xerr = pdata['xerr']
                 yerr = pdata['yerr']
-            if not show_error:
-                error_format = None
-            handle = self._draw_data(ax, x, y, xerr, yerr,
-                                     plot_options=combined_options,
-                                     error_format=error_format,
-                                     error_styles=error_styles)
+                width = np.diff(x)[0]
+                combined_options = combine_dict(self.styles['bar'], plot_options)
+                combined_options['label'] = label
+                handle = ax.stairs(x, y, align='center', width=width, **combined_options)
         return handle
     
-    def process_data(self, data, blind_range:Optional[List[float]]=None,
-                     scale_by:Optional[float]=None):
+    def process_data(self, data, blind_range:Optional[List[float]]=None):
         processed_data = {}
         processed_data['x'] = data['x']
         processed_data['y'] = data['y']
         error_data = self.get_error_data(data)
-        if (scale_by is not None) and (scale_by != 1.):
-            if error_data['yerr'] is not None:
-                raise RuntimeError("scaling is not allowed for data points with error")
-            processed_data['y'] *= scale_by
         processed_data['xerr'] = error_data['xerr']
         processed_data['yerr'] = error_data['yerr']
         if blind_range is not None:
@@ -224,7 +237,7 @@ class PdfDistributionPlot(AbstractPlot):
                    plot_options:Optional[Dict]=None,
                    error_format:Optional[Union[ErrorDisplayFormat, str]]="error_bar",
                    error_styles:Optional[Dict]=None):
-        combined_options = combine_dict(plot_options, self.styles['errorbar'])
+        combined_options = combine_dict(self.styles['errorbar'], plot_options)
         if error_format is None:
             combined_options['elinewidth'] = 0
             combined_options['capsize'] = 0
@@ -288,6 +301,7 @@ class PdfDistributionPlot(AbstractPlot):
             vmode = 1
         else:
             raise ValueError(f"unsupported mode \"{mode}\", choose between \"ratio\" and \"difference\"")
+        valid_mask = (pdata_tgt['y'] > 0) & (pdata_ref['y'] > 0)
         if vmode == 0:
             pdata_comp['y'] = pdata_tgt['y'] / pdata_ref['y']
             if isinstance(yerr_ref, tuple):
@@ -312,16 +326,12 @@ class PdfDistributionPlot(AbstractPlot):
             
         draw_blind = blind_range is not None
         if draw_blind:
-            x = pdata_comp['x'][pdata_ref['sideband']]
-            y = pdata_comp['y'][pdata_ref['sideband']]
-            xerr = self._get_selected_err(pdata_comp['xerr'], pdata_ref['sideband'])
-            yerr = self._get_selected_err(pdata_comp['yerr'], pdata_ref['sideband'])
-        else:
-            x = pdata_comp['x']
-            y = pdata_comp['y']
-            xerr = pdata_comp['xerr']
-            yerr = pdata_comp['yerr']
-            
+            valid_mask &= pdata_ref['sideband']
+        x = pdata_comp['x'][valid_mask]
+        y = pdata_comp['y'][valid_mask]
+        xerr = self._get_selected_err(pdata_comp['xerr'], valid_mask)
+        yerr = self._get_selected_err(pdata_comp['yerr'], valid_mask)
+        
         if not show_error:
             error_format = None
         handle = self._draw_data(ax, x, y, xerr, yerr,
@@ -340,12 +350,12 @@ class PdfDistributionPlot(AbstractPlot):
             centralize_axis(ax, which="y", ref_value=1)
             if draw_ratio_line:
                 ratio_linestyles = self.config['ratio_line_styles']
-                ax.axhline(1, zorder=0, **ratio_linestyles)
+                ax.axhline(1, **ratio_linestyles)
         elif vmode == 1:
             centralize_axis(ax, which="y", ref_value=0)
             if draw_ratio_line:
                 ratio_linestyles = self.config['ratio_line_styles']
-                ax.axhline(0, zorder=0, **ratio_linestyles)
+                ax.axhline(0, **ratio_linestyles)
         if not ylabel:
             if vmode == 0:
                 ylabel = "Fit / Data"
@@ -392,6 +402,7 @@ class PdfDistributionPlot(AbstractPlot):
              logx:bool=False, logy:bool=False,
              blind_range:Optional[List[float]]=None,
              show_error:bool=True,
+             plot_type:str='errorbar',
              comparison_options:Optional[Union[Dict, List[Dict]]]=None,
              xmin:Optional[float]=None, xmax:Optional[float]=None,
              ymin:Optional[float]=None, ymax:Optional[float]=None,
@@ -402,22 +413,25 @@ class PdfDistributionPlot(AbstractPlot):
                                            **self.styles["ratio_frames"])
         else:
             ax = self.draw_frame(logx=logx, logy=logy)
-
-        for name in self.collective_data:
-            if (targets is not None) and(name not in targets):
-                continue
-            data = self.collective_data[name]
-            label = self.label_map.get(name, name)
-            label_blind = self.label_map.get(f"{name}_blind", None)
-            plot_options = self.plot_options.get(name, {})
-            scale_by = self.scale_map.get(name, None)
-            if (name not in self.error_options) or \
-               (self.error_options[name]["display_format"] is None):
+        
+        if targets is None:
+            targets = list(self.collective_data)
+        if not self.legend_order:
+            self.legend_order = list(targets)
+        for target in targets:
+            if target not in self.collective_data:
+                raise RuntimeError(f'no input found for the target: {target}')
+            data = self.collective_data[target]
+            label = self.label_map.get(target, target)
+            label_blind = self.label_map.get(f"{target}_blind", None)
+            plot_options = self.plot_options.get(target, {})
+            if (target not in self.error_options) or \
+               (self.error_options[target]["display_format"] is None):
                 error_format = None
                 error_styles = None
             else:
-                error_format = self.error_options[name]["display_format"]
-                error_styles = self.error_options[name]["styles"]
+                error_format = self.error_options[target]["display_format"]
+                error_styles = self.error_options[target]["styles"]
             handle = self.draw_single_data(ax, data, label,
                                            label_blind=label_blind,
                                            blind_range=blind_range,
@@ -425,19 +439,21 @@ class PdfDistributionPlot(AbstractPlot):
                                            show_error=show_error,
                                            error_format=error_format,
                                            error_styles=error_styles,
-                                           scale_by=scale_by)
+                                           plot_type=plot_type)
+            if not handle.get_children():
+                    self.colors[target] = handle.get_color()
+            else:
+                if plot_type == 'errorbar':
+                    self.colors[target] = handle[0].get_color()
+                elif plot_type == 'hist':
+                    self.colors[target] = handle[0].get_edgecolor()
             # case draw blind
             if isinstance(handle, list) and len(handle) == 2:
-                blind_name = f"{name}_blind"
-                self.legend_order.append(blind_name)
-                self.update_legend_handles({name: handle[0], blind_name: handle[1]})
-                self.colors[name] = handle[0].get_color()
+                blind_target = f"{target}_blind"
+                self.legend_order.append(blind_target)
+                self.update_legend_handles({target: handle[0], blind_target: handle[1]})
             else:
-                self.update_legend_handles({name: handle})
-                if not handle.get_children():
-                    self.colors[name] = handle.get_color()
-                else:
-                    self.colors[name] = handle[0].get_color()
+                self.update_legend_handles({target: handle})
 
         self.draw_axis_components(ax, xlabel=xlabel, ylabel=ylabel)
         self.set_axis_range(ax, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax, ypad=ypad)  
@@ -456,6 +472,7 @@ class PdfDistributionPlot(AbstractPlot):
                 styles    = options.get('styles', {})
                 draw_ratio_line = options.get('draw_ratio_line', True)
                 plot_options = self.plot_options.get(target, {})
+                plot_options = combine_dict(plot_options, styles)
                 if ('color' not in plot_options) and (target in self.colors):
                     plot_options['color'] = self.colors[target]
                 plot_options.update(styles)
@@ -466,13 +483,16 @@ class PdfDistributionPlot(AbstractPlot):
                 else:
                     error_format = self.error_options[reference]["display_format"]
                     error_styles = self.error_options[reference]["styles"]
+                comparison_show_error = options.get('show_error', None)
+                if comparison_show_error is None:
+                    comparison_show_error = show_error
                 self.draw_comparison(ax_ratio,
                                      self.collective_data[reference],
                                      self.collective_data[target],
                                      xlabel=ax.get_xlabel(),
                                      ylabel=ylabel, mode=mode,
                                      plot_options=plot_options,
-                                     show_error=show_error,
+                                     show_error=comparison_show_error,
                                      error_format=error_format,
                                      error_styles=error_styles,
                                      blind_range=blind_range,
@@ -514,10 +534,10 @@ class PdfDistributionPlot(AbstractPlot):
     @classmethod
     def from_dataframes(cls, collective_df:Dict[str, "pandas.DataFrame"],
                         observable:str, weight:Optional[Union[float, str]]=None,
-                        range:Optional[Union[List, Tuple]]=None,
-                        bins:int=10, normalize:bool=True,
+                        bin_range:Optional[Union[List, Tuple]]=None,
+                        nbins:int=10, normalize:bool=True,
                         x_scale:Optional[float]=None,                        
-                        error_method:str="auto", **kwargs):
+                        error_option:str="auto", **kwargs):
         from quickstats.maths.statistics import get_hist_data
         collective_data = {}
         for key, df in collective_df.items():
@@ -531,9 +551,8 @@ class PdfDistributionPlot(AbstractPlot):
             else:
                 weights = df[weight].values
             hist_data = get_hist_data(x, weights, normalize=normalize,
-                                      range=range, bins=bins, error_method=error_method)
+                                      bin_range=bin_range, bins=nbins,
+                                      error_option=error_option)
             collective_data[key] = hist_data
         plotter = cls(collective_data, **kwargs)
-        return plotter
-        
-                        
\ No newline at end of file
+        return plotter
\ No newline at end of file
