@@ -1,4 +1,4 @@
-from typing import Optional, Union, Dict, List
+from typing import Optional, Union, Dict, List, Tuple
 import os
 import sys
 import copy
@@ -12,7 +12,7 @@ import collections.abc
 
 import numpy as np
 
-from .string_utils import split_str
+from .string_utils import split_str, parse_as_dict
 
 def timely_info(green_text, normal_text, adjust=40):
     print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '\033[92m[INFO]\033[0m', '\033[92m{}\033[0m'.format(green_text).rjust(40, ' '), normal_text)
@@ -41,19 +41,6 @@ def execute_multi_tasks(func, *iterables, parallel):
             max_workers = parallel
         return parallel_run(func, *iterables, max_workers=max_workers)
 
-
-def timer(func):
-    """Print the runtime of the decorated function"""
-    @functools.wraps(func)
-    def wrapper_timer(*args, **kwargs):
-        start_time = time.perf_counter()    # 1
-        value = func(*args, **kwargs)
-        end_time = time.perf_counter()      # 2
-        run_time = end_time - start_time    # 3
-        print('INFO: All jobs have finished. Total time taken: {:.3f} s'.format(run_time))
-        return value
-    return wrapper_timer
-
 def stdout_print(msg):
     sys.__stdout__.write(msg + '\n')
     
@@ -107,6 +94,38 @@ def parse_config(source:Optional[Union[Dict, str]]=None):
         raise ValueError("invalid config input")
         
 class NpEncoder(json.JSONEncoder):
+    """
+    Custom JSON Encoder for handling NumPy data types in JSON serialization.
+
+    This custom JSONEncoder subclass is designed to handle NumPy data types 
+    (such as np.integer, np.floating, and np.ndarray) during JSON serialization. 
+    
+    Note:
+        To use this custom encoder, you can pass it as the 'cls' argument when 
+        calling the json.dumps() function or json.dump() method.
+
+    Example:
+        import json
+        import numpy as np
+
+        # Create a dictionary with NumPy data types
+        data = {
+            'integer': np.int64(42),
+            'floating': np.float64(3.14),
+            'array': np.array([1, 2, 3])
+        }
+
+        # Serialize the dictionary using the NpEncoder
+        json_string = json.dumps(data, cls=NpEncoder)
+
+        # Deserialize the JSON string
+        decoded_data = json.loads(json_string)
+
+        # The NumPy data types are now properly serialized and deserialized
+        print(decoded_data['integer'])  # outputs: 42
+        print(decoded_data['floating'])  # outputs: 3.14
+        print(decoded_data['array'])  # outputs: [1, 2, 3]
+    """
     def default(self, obj):
         if isinstance(obj, np.integer):
             return int(obj)
@@ -115,22 +134,58 @@ class NpEncoder(json.JSONEncoder):
         if isinstance(obj, np.ndarray):
             return obj.tolist()
         return super(NpEncoder, self).default(obj)
+
     
-def update_nested_dict(d:Dict, u:Dict):
+def update_nested_dict(d: Dict, u: Dict) -> Dict:
+    """
+    Recursively updates nested dictionaries.
+
+    Parameters
+    ----------
+    d : Dict
+        The dictionary to be updated.
+    u : Dict
+        The dictionary containing updates.
+
+    Returns
+    -------
+    Dict
+        The updated dictionary.
+
+    Notes
+    -----
+    If a key exists in `u` but not in `d`, the key-value pair from `u` is added to `d`.
+    """
     for k, v in u.items():
         if isinstance(d.get(k, None), collections.Mapping) and isinstance(v, collections.Mapping):
-            d[k] = update_nested_dict(d[k], v)
+            d[k] = update_nested_dict(d.get(k, {}), v)
         else:
             d[k] = v
     return d
 
-def combine_dict(d:Dict, u:Optional[Dict]=None):
-    d_copy = copy.deepcopy(d)
+def combine_dict(d: Optional[Dict] = None, u: Optional[Dict] = None) -> Dict:
+    """
+    Creates a deep copy of two dictionaries and combines their contents.
+
+    Parameters
+    ----------
+    d : Dict, optional
+        The primary dictionary. Default is None.
+    u : Dict, optional
+        The dictionary containing updates. If `None`, the function returns a 
+        deep copy of `d`. Default is None.
+
+    Returns
+    -------
+    Dict
+        The combined dictionary.
+    """
+    d_copy = copy.deepcopy(d) if d is not None else {}
     if u is None:
         return d_copy
-    else:
-        u_copy = copy.deepcopy(u)
-        return update_nested_dict(d_copy, u_copy)
+
+    u_copy = copy.deepcopy(u)
+    return update_nested_dict(d_copy, u_copy)
     
 def str_list_filter(source:List[str], patterns:List[str], inclusive:bool=True):
     if inclusive:
@@ -158,17 +213,44 @@ def get_class_that_defined_method(meth):
             return cls
     return getattr(meth, '__objclass__', None)
 
-def batch_makedirs(dirnames:Union[str, List[str], Dict[str, str]]):
+def batch_makedirs(dirnames:Union[str, List[str], Dict[str, str]]) -> None:
+    """
+    Create multiple directories in a batch. 
+
+    This function accepts a single directory name as a string, a list of directory 
+    names, or a dictionary where the values are directory names. It then creates 
+    these directories if they do not already exist.
+
+    Parameters
+    ----------
+    dirnames : Union[str, List[str], Dict[str, str]]
+        The directory name(s) to be created. 
+        - If it's a string, a single directory will be created.
+        - If it's a list, each element is treated as a directory name.
+        - If it's a dictionary, each value is treated as a directory name.
+
+    Raises
+    ------
+    ValueError
+        If `dirnames` is not a string, a list, or a dictionary.
+
+    Examples
+    --------
+    >>> batch_makedirs('new_dir')
+    >>> batch_makedirs(['new_dir1', 'new_dir2'])
+    >>> batch_makedirs({'dir1': 'new_dir1', 'dir2': 'new_dir2'})
+    """
     if isinstance(dirnames, str):
-        batch_makedirs([dirnames])
-    elif isinstance(dirnames, list):
-        for dirname in dirnames:
-            if not os.path.exists(dirname):
-                os.makedirs(dirname)
+        dirnames = [dirnames]
     elif isinstance(dirnames, dict):
-        for key in dirnames:
-            if not os.path.exists(dirnames[key]):
-                os.makedirs(dirnames[key])
+        dirnames = list(dirnames.values())
+
+    if isinstance(dirnames, list):
+        for dirname in dirnames:
+            abs_dirname = os.path.abspath(dirname)
+            os.makedirs(abs_dirname, exist_ok=True)
+    else:
+        raise ValueError('invalid format for "dirnames"')
 
 def set_scripts_path(scripts_path, undo=False):
     if (scripts_path in sys.path) and (undo==True):
@@ -179,14 +261,16 @@ def set_scripts_path(scripts_path, undo=False):
         sys.path.insert(0, scripts_path)
         os.environ["PYTHONPATH"] = scripts_path + ":" + os.environ.get("PYTHONPATH", "")
         
-def is_valid_file(fname:str):
-    if not os.path.exists(fname):
+def is_valid_file(filename:str):
+    if not filename:
         return False
-    ext = os.path.splitext(fname)[-1]
+    if not os.path.exists(filename):
+        return False
+    ext = os.path.splitext(filename)[-1]
     if ext == ".root":
         from quickstats.utils.root_utils import is_corrupt
-        return not is_corrupt(fname)
-    return (os.path.isfile(fname)) and (os.path.getsize(fname) > 0)
+        return not is_corrupt(filename)
+    return (os.path.isfile(filename)) and (os.path.getsize(filename) > 0)
 
 def remove_list_duplicates(seq:List, keep_order:bool=True):
     if keep_order:
@@ -237,18 +321,52 @@ def itemize_dict(d:Dict, separator:str=": ", leftmargin:int=0, linebreak:int=100
         texts.append(text)
     return "\n".join(texts) + "\n"
 
-def filter_by_wildcards(targets:List, conditions:Optional[Union[str, List]]=None, exclusion:bool=False):
+def filter_by_wildcards(targets: List[str], conditions: Optional[Union[str, List[str]]] = None, exclusion: bool = False) -> List[str]:
+    """
+    Filters a list of string targets based on wildcard conditions.
+
+    This method filters a list of targets based on provided wildcard patterns.
+    It can either return targets that match the conditions or exclude them.
+
+    Parameters
+    ----------
+    targets : List[str]
+        A list of string targets to be filtered.
+    conditions : Union[str, List[str]], optional
+        Wildcard patterns used for filtering. If a string is provided,
+        it is split by commas into individual conditions.
+    exclusion : bool
+        If True, the method returns targets that do not match the 
+        conditions. If False, it returns targets that match the conditions.
+        Defaults to False.
+
+    Returns
+    -------
+    List[str]
+        A filtered list of string targets based on the provided conditions.
+
+    Examples
+    --------
+    >>> filter_by_wildcards(['apple', 'banana', 'cherry'], 'a*')
+    ['apple']
+
+    >>> filter_by_wildcards(['apple', 'banana', 'cherry'], 'a*', exclusion=True)
+    ['banana', 'cherry']
+
+    >>> filter_by_wildcards(['apple', 'banana', 'cherry'], ['a*', '*rr*'])
+    ['apple', 'cherry']
+
+    """
     if not conditions:
         return targets
-    selected = []
+
     if isinstance(conditions, str):
-        conditions = split_str(conditions, sep=',')
-    for target in targets:
-        if any([fnmatch.fnmatch(target, condition) for condition in conditions]):
-            selected.append(target)
+        conditions = conditions.split(',')
+    
     if exclusion:
-        return [target for target in targets if target not in selected]
-    return selected
+        return [target for target in targets if not any(fnmatch.fnmatch(target, condition) for condition in conditions)]
+    
+    return [target for target in targets if any(fnmatch.fnmatch(target, condition) for condition in conditions)]
 
 def set_unlimited_stacksize():
     import resource
@@ -270,4 +388,63 @@ def in_notebook() -> bool:
         return False
     except NameError:
         return False      # Probably standard Python interpreter    
-    return True
\ No newline at end of file
+    return True
+
+def reindex_dataframe(df, index_values:Union[Tuple[List], List],
+                      index_levels:Optional[Union[Tuple[str], Tuple[int], str, int]]=None):
+    if not isinstance(index_values, tuple):
+        index_values = tuple([index_values])
+    if index_levels is None:
+        index_levels = tuple(range(len(index_values)))
+    if isinstance(index_levels, (str, int)):
+        index_levels = (index_levels,)
+    for values, level in zip(index_values, index_levels):
+        mask = np.in1d(values, df.index.get_level_values(level))
+        df = df.reindex(np.array(values)[mask], level=level)
+    return df
+
+def filter_dataframe_by_index_values(df, index_values:Union[Tuple[List], List],
+                                     index_levels:Optional[Union[Tuple[str], Tuple[int], str, int]]=None):
+    if not isinstance(index_values, tuple):
+        index_values = tuple([index_values])
+    if index_levels is None:
+        index_levels = tuple(range(len(index_values)))
+    if isinstance(index_levels, (int, str)):
+        index_levels = (index_levels,)
+    for values, level in zip(index_values, index_levels):
+        df = df.loc[df.index.get_level_values(level).isin(values)]
+    return df
+
+def parse_config_dict(expr:Optional[Union[str, Dict]]=None):
+    if expr is None:
+        return {}
+    if isinstance(expr, str):
+        return parse_as_dict(expr)
+    if isinstance(expr, dict):
+        return expr
+    raise ValueError(f'expr of type "{type(expr)}" is not convertible to a config dict')
+
+def update_config_dict(orig_dict, new_dict, allow_overlap_keys:bool=True):
+    orig_dict = parse_config_dict(orig_dict)
+    new_dict = parse_config_dict(new_dict)
+    if not allow_overlap_keys:
+        overlap_keys = list(set(orig_dict.keys()) & set(new_dict.keys()))
+        if overlap_keys:
+            raise RuntimeError(f'found overlap keys between two config dict: {", ".join(overlap_keys)}')
+    return combine_dict(orig_dict, new_dict)
+
+def list_of_dict_to_dict_of_list(source:List[Dict], use_first_keys:bool=True):
+    if use_first_keys:
+        return {k: [item[k] for item in source] for k in source[0]}
+    common_keys = set.intersection(*map(set, source))
+    return {k: [item[k] for item in source] for k in common_keys}
+
+def dict_of_list_to_list_of_dict(source:Dict[str, List]):
+    return [dict(zip(source, t)) for t in zip(*source.values())]
+
+def save_as_json(data:Dict, outname:str,
+                 indent:int=2, truncate:bool=True):
+    with open(outname, "w") as file:
+        json.dump(data, file, indent=2)
+        if truncate:
+            file.truncate()
\ No newline at end of file
