@@ -42,7 +42,9 @@ def get_variable_attributes(variable:ROOT.RooAbsReal, asym_error:bool=False):
             'value': variable.getVal()
         }
     else:
-        raise ValueError("variable must be an instance of RooAbsReal")
+        attributes = {
+            'name' : variable.GetName()
+        }
     return attributes
 
 def variable_collection_to_dataframe(var_collection:Union[list, ROOT.RooArgSet],
@@ -81,7 +83,7 @@ def construct_categorized_pdf_dataset(pdf:"ROOT.RooAbsPdf", dataset:"ROOT.RooDat
 
 def factorize_pdf(observables:"ROOT.RooArgSet", pdf:"ROOT.RooAbsPdf", constraints:"ROOT.RooArgSet"):
     pdf_class = pdf.ClassName()   
-    if pdf_class == "RooProdPdf":
+    if pdf_class.InheritsFrom("RooProdPdf"):
         new_factors = ROOT.RooArgList()
         new_owned = ROOT.RooArgSet()
         pdf_list = pdf.pdfList()
@@ -108,7 +110,7 @@ def factorize_pdf(observables:"ROOT.RooArgSet", pdf:"ROOT.RooAbsPdf", constraint
         factorized_pdf.addOwnedComponents(new_owned)
         copy_attributes(pdf, factorized_pdf)
         return factorized_pdf
-    elif pdf_class == "RooSimultaneous":
+    elif pdf_class.InheritsFrom("RooSimultaneous"):
         cat = pdf.indexCat().Clone()
         n_bins = cat.numBins("")
         factorized_pdfs = []
@@ -142,11 +144,10 @@ def factorize_pdf(observables:"ROOT.RooArgSet", pdf:"ROOT.RooAbsPdf", constraint
     elif pdf.dependsOn(observables):
         return pdf
     else:
-        if not constraints.contains(pdf):
+        if (not constraints.contains(pdf)) and (not pdf.getAttribute('ignoreConstraint')):
             constraints.add(pdf)
         return 0
     
-    
 def rebuild_simultaneous_pdf(observables:"ROOT.RooArgSet", sim_pdf:"ROOT.RooSimultaneous"):
     assert sim_pdf.ClassName() == "RooSimultaneous"
     constraints = ROOT.RooArgList()
@@ -337,7 +338,8 @@ def recover_formula(formula:str):
     variables = variables_regex.findall(formula)
     if len(variables) != 1:
         raise RuntimeError("invalid RooFit factory expression")
-    variables = split_str(variables[0], sep=',', remove_empty=True)
+    variable_regex = re.compile(f"([a-zA-Z]\w*)")
+    variables = variable_regex.findall(variables[0])
     recovered_formula = expression
     for i, variable in enumerate(variables):
         recovered_formula = re.sub(f"@{i}(?![0-9])", variable, recovered_formula)
@@ -437,7 +439,31 @@ def get_ProcessNormalization_response_variations(nuis:ROOT.RooRealVar, client):
     else:
         raise RuntimeError(f'invalid kappa values for the systematic "{nuis_name}" from '
                            f'the ProcessNormalization client "{client.GetName()}"')
-            
+
+def get_response_function_variations(nuis:ROOT.RooRealVar, client):
+    result = {}
+    param_names = [param.GetName() for param in client.parameters()]
+    nuis_idx = param_names.index(nuis.GetName())
+    low_names = [param.GetName() for param in client.low()]
+    tokens = split_str(low_names[nuis_idx], '_', remove_empty=True)
+    client_name = "_".join(tokens[1:])
+    low_values = [param.getVal() for param in client.low()]
+    high_values = [param.getVal() for param in client.high()]
+    norm_values = [param for param in client.nominal()]
+    interp_codes = [param for param in client.interpolationCodes()]
+    norm_value, interp_code = round(norm_values[nuis_idx], 8), interp_codes[nuis_idx]
+    low_value, high_value = round(low_values[nuis_idx], 8), round(high_values[nuis_idx], 8)
+    result = {"client": client_name, "nominal": norm_value, "low": low_value, "high": high_value}
+    if interp_code == 0:
+        result['type'] = 'gaus'
+    elif interp_code == 1:
+        result['type'] = 'logn'
+    elif interp_code == 4:
+        result['type'] = 'asym'
+    else:
+        result['type'] = 'unknown'
+    return result
+    
 def get_systematic_variations_from_client(nuis:ROOT.RooRealVar, client):
     result = {"client": None, "nominal": None, "low": None, "high": None, "type": None}
     class_name = client.ClassName()
@@ -459,6 +485,8 @@ def get_systematic_variations_from_client(nuis:ROOT.RooRealVar, client):
             result.update(get_gaus_response_variations(nuis, parent_client))
         elif isinstance(parent_client, ROOT.RooStats.HistFactory.FlexibleInterpVar):
             result.update(get_asym_response_variations(nuis, parent_client))
+    elif class_name == "ResponseFunction":
+        result = get_response_function_variations(nuis, client)
     return result
 
 def get_systematic_variations_from_clients(nuis:ROOT.RooRealVar, constr_pdf:ROOT.RooAbsPdf):
@@ -558,4 +586,18 @@ def get_gaussian_pdf_attributes(pdfs:"ROOT.RooArgSet", fmt:str="pandas"):
 
 def get_poisson_pdf_attributes(pdfs:"ROOT.RooArgSet", fmt:str="pandas"):
     return get_pdf_attributes(pdfs, ["X", "Mean"], pdf_class=ROOT.RooPoisson, fmt=fmt)
-    
\ No newline at end of file
+    
+    
+def get_sim_pdf_ndof(pdf:"ROOT.RooSimultaneous", observables:"ROOT.RooArgSet", exclude_syst:bool=True):
+    import ROOT
+    if not isinstance(pdf, ROOT.RooSimultaneous):
+        raise RuntimeError('not a simultaneous pdf')
+    params = pdf.getVariables()
+    nuis_pdf = ROOT.RooStats.MakeNuisancePdf(pdf, observables, "nuisancePdf")
+    ndof = 0
+    for param in params:
+        param_name = param.GetName()
+        if ((not param.isConstant()) and (not observables.find(param_name))
+            and (exclude_syst and not nuis_pdf.dependsOn(ROOT.RooArgSet(param)))):
+            ndof += 1
+    return ndof
\ No newline at end of file
