@@ -1,4 +1,4 @@
-from typing import Optional, Dict, List, Union
+from typing import Optional, Dict, List, Union, Sequence
 import os
 import glob
 import time
@@ -8,6 +8,7 @@ import numpy as np
 import ROOT
 import quickstats
 
+
 root_type_str_maps = {
     'Char_t'    : 'char',
     'Bool_t'    : 'bool',
@@ -263,7 +264,7 @@ def get_tree_names(f:ROOT.TFile):
 def merge_root_files(source_files:Union[str, List[str]], target_file:str, treename:str=None,
                      multithread:bool=True, keep_order:bool=False, use_template:bool=True,
                      ignore_missing_columns:bool=True, verbosity:str="INFO"):
-    from quickstats.utils.io import VerbosePrint
+    from quickstats import VerbosePrint
     stdout = VerbosePrint(verbosity)
     
     if isinstance(source_files, str):
@@ -276,7 +277,7 @@ def merge_root_files(source_files:Union[str, List[str]], target_file:str, treena
         rfiles += files
     # now all rfiles are guaranteed to exist
     for i, rfile in enumerate(rfiles):
-        stdout.info(f"Source {i+1}: {rfile}")
+        stdout.info(f"Source {i+1}: {rfile}", bare=True)
         if is_corrupt(rfile):
             raise RuntimeError(f"the root file \"{rfile}\" is corrupted")
     if not rfiles:
@@ -288,7 +289,7 @@ def merge_root_files(source_files:Union[str, List[str]], target_file:str, treena
             raise RuntimeError("unable to deduce tree name from source files: "
                                "multiple trees are present")
         treename = treenames[0]
-        stdout.info("INFO: No tree name specified for merging. Will use "
+        stdout.info("No tree name specified for merging. Will use "
                     f"the auto-detected tree name \"{treename}\".")
         
     # configure multi-threading setting
@@ -306,14 +307,14 @@ def merge_root_files(source_files:Union[str, List[str]], target_file:str, treena
             columns = set([str(i) for i in rdf.GetColumnNames()])
             all_columns.update(columns)
             column_map[rfile] = columns
-        for fname, columns in column_map.items():
+        for rfile, columns in column_map.items():
             if columns != all_columns:
                 missing_columns = list(all_columns - columns)
-                stdout.warning(f"WARNING: The root file \"{rfile}\" is missing the following column(s): "
+                stdout.warning(f"The root file \"{rfile}\" is missing the following column(s): "
                                f"{','.join(missing_columns)}. The missing column(s) will not be saved.")
         columns_to_keep = list(set.intersection(*column_map.values()))
         rdf = ROOT.RDataFrame(treename, rfiles)
-        stdout.info(f"Target path: {target_file}")
+        stdout.info(f"Target path: {target_file}", bare=True)
         if use_template:
             snapshot = templated_rdf_snapshot(rdf, columns_to_keep)
         else:
@@ -321,7 +322,7 @@ def merge_root_files(source_files:Union[str, List[str]], target_file:str, treena
         snapshot(treename, target_file, columns_to_keep)
     else:
         rdf = ROOT.RDataFrame(treename, rfiles)
-        stdout.info(f"Target path: {target_file}")
+        stdout.info(f"Target path: {target_file}", bare=True)
         if use_template:
             snapshot = templated_rdf_snapshot(rdf, columns_to_keep)
         else:
@@ -333,4 +334,52 @@ def merge_root_files(source_files:Union[str, List[str]], target_file:str, treena
         if multithread_orig_state:
             ROOT.DisableImplicitMT()
         else:
-            ROOT.EnableImplicitMT() 
\ No newline at end of file
+            ROOT.EnableImplicitMT()
+            
+def list_root_files(filenames:Union[List[str], str], strict:bool=False):
+    all_files = []
+    if isinstance(filenames, str):
+        filenames = [filenames]
+    if not isinstance(filenames, list):
+        raise ValueError("filename must be either string or list of strings")
+    for filename in filenames:
+        if os.path.isdir(filename):
+            all_files += glob.glob(os.path.join(filename, "*.root"))
+        else:
+            all_files += glob.glob(filename)
+    if not all_files:
+        raise FileNotFoundError(f'file "{filename}" does not exist')
+    valid_files = []
+    from quickstats.interface.root import TFile
+    for file in all_files:
+        rf = ROOT.TFile(file)
+        if not TFile.is_corrupt(rf):
+            valid_files.append(file)
+        elif strict:
+            raise RuntimeError(f'Found empty/corrupted file "{file}"')
+    return valid_files
+
+def get_th1d_model(bins:Union[int, Sequence]=128,
+                   bin_range:Optional[Sequence]=None,
+                   name:str='h',
+                   title:Optional[str]=None):
+    if title is None:
+        title = name
+    if isinstance(bins, Sequence):
+        if bin_range is not None:
+            raise ValueError('can not specify bin_range when bins is a sequence')
+        bins = np.array(bins)
+        nbinsx = len(bins)
+        from quickstats.interface.cppyy.vectorize import as_pointer
+        xbins = as_pointer(np.array(bins, dtype='float'))
+        return ROOT.RDF.TH1DModel(name, title, nbinsx, xbins)
+    if bin_range is None:
+        xlow, xup = 0., 0.
+    else:
+        xlow, xup = bin_range
+    return ROOT.RDF.TH1DModel(name, title, bins, xlow, xup)
+
+def close_read_cache(rfile:"ROOT.TFile"):
+    read_cache = rfile.GetCacheRead()
+    if read_cache:
+        read_cache.Close()
\ No newline at end of file
